/*
 * Copyright (c) 1994, 1996 Hellmuth Michaelis. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *---------------------------------------------------------------------------*
 *
 *	run time configuration header
 *      =============================
 *
 *	Last Edit-Date: [Fri Jul  5 12:05:32 1996]
 *
 *	-hm	new telephone number handling
 *	-hm	regular expression log message matching
 * 
 *----------------------------------------------------------------------------*/

#define NO_WILD		0
#define ALLOW_WILD	1

#define MODE_ENTRY	0
#define MODE_SYSTEM	1

/* string variables */

#define	CF_ENTRY	"entry"		/* starts a configuration entry */
#define	CF_SYSTEM	"system"	/* starts the system configuration */

#define CF_NAME		"name"		/* symbolic name for this entry/link */
#define CF_INTERFACE	"interface"	/* name of the interface */

#define CF_RATESFILE	"ratesfile"	/* name of the ratesfile */
#define CF_ACCTFILE	"acctfile"	/* name of the accounting file if useacct = on */
#define CF_REGEXPR	"regexpr"	/* a regular expression */
#define CF_REGPROG	"regprog"	/* program name run if regexp match */
#define CF_ANSWPROG	"answprog"	/* program name for play */
#define CF_PLAYPROG	"playprog"	/* program name for answer */

/* string choice variables */

#define CF_ISDNTYPE	"isdntype"	/* isdn protocol type */
#define CF_REACTION	"reaction"	/* what to do whan remote site calls */
#define CF_PROTOCOL	"protocol"	/* protocol to use for this link */

/* numeric string variables */

#define CF_TELNLOC_LDO	"telnloc-ldo"	/* local telephone number used	*/
					/*	for local dialing out	*/
#define CF_TELNREM_LDO	"telnrem-ldo"	/* remote telephone number used	*/
					/*	for local dialing out	*/
#define CF_TELNLOC_RDI	"telnloc-rdi"	/* local telephone number used	*/
					/*	for remote dialing in	*/
#define CF_TELNREM_RDI	"telnrem-rdi"	/* remote telephone number used	*/
					/*	for remote dialing in	*/

/* numeric variables */

#define	CF_CONTROLLER	"controller"	/* controller number to use for outgoing calls */
#define CF_DIALRETRIES	"dialretries"	/* number of retries when dialing out */
#define CF_RECOVERYTIME	"recoverytime"	/* time to wait between dial retries */
#define CF_CALLBACKWAIT	"callbackwait"	/* time to wait before calling back */
#define CF_IDLETIMEOUT	"idletime-out"	/* close outgoing connection after being idle this time */
#define CF_IDLETIMEIN	"idletime-in"	/* close incoming connection after being idle this time */
#define CF_BLOCKSIZE	"blocksize"	/* max number of byte for an isdn block */
#define CF_RATETYPE	"ratetype"	/* rate structure entry from bisdnd.rates */
#define CF_UNITLENGTH	"unitlength"	/* length of charging unit */
#define CF_EARLYHANGUP	"earlyhangup"	/* early hangup time in seconds */
#define CF_CALLEDBACKWAIT "calledbackwait" /* max time to wait for being called back */

/* on/off variables */

#define CF_UNITLENDYN	"unitlength-dyn" /* enable/disable dynamic unit length calculation */

#define CF_USEACCTFILE	"useacctfile"	/* write accounting info into file */
#define CF_ACCTALL	"acctall"	/* write also no charge connect acct */

/* flag bits for config check */

/* entry */

#define	FL_ENTRY		0x00000001
#define FL_NAME			0x00000002
#define FL_INTERFACE		0x00000004
#define FL_ISDNTYPE		0x00000008
#define FL_REACTION		0x00000010
#define FL_CALLBACKWAIT		0x00000020
#define FL_PROTOCOL		0x00000040
#define FL_TELNLOC_LDO		0x00000080
#define FL_TELNREM_LDO		0x00000100
#define FL_TELNLOC_RDI		0x00000200
#define FL_TELNREM_RDI		0x00000400
#define	FL_CONTROLLER		0x00000800
#define FL_DIALRETRIES		0x00001000
#define FL_RECOVERYTIME		0x00002000
#define FL_IDLETIMEIN		0x00004000
#define FL_IDLETIMEOUT		0x00008000
#define FL_BLOCKSIZE		0x00010000
#define FL_UNITLENDYN		0x00020000
#define FL_RATETYPE		0x00040000
#define FL_UNITLENGTH		0x00080000
#define FL_EARLYHANGUP		0x00100000
#define FL_CALLEDBACKWAIT	0x00200000

/* system */

#define FL_SYSTEM		0x00000001
#define FL_RATESFILE		0x00000002
#define FL_ACCTFILE		0x00000004
#define FL_USEACCTFILE		0x00000008
#define FL_ACCTALL		0x00000010
#define FL_REGPROG		0x00000020
#define FL_REGEXPR		0x00000040
#define FL_ANSWPROG		0x00000080
#define FL_PLAYPROG		0x00000100

/* values a variable can take */

#define	CVAL_ON		"on"
#define	CVAL_OFF	"off"

/* CF_INTERFACE */

#define CVAL_IPI	"ipi"
#define CVAL_TTY	"tty"
#define CVAL_ANSW	"answ"
#define CVAL_PLAY	"play"

/* CF_ISDNTYPE */

#define CVAL_DSS1	"dss1"
#define CVAL_1TR6	"1tr6"

/* CF_REACTION */

#define CVAL_ACCEPT	"accept"
#define CVAL_REJECT	"reject"
#define CVAL_CALLBACK	"callback"
#define CVAL_CALLEDBACK	"calledback"
#define CVAL_ALERTBASE	"alert"

/* CF_PROTOCOL */

#define CVAL_RAW	"raw"
#define CVAL_UI		"ui"
#define CVAL_X75	"x75"
#define CVAL_T70	"t70"
#define CVAL_T90	"t90"
#define CVAL_ISO	"iso"

/*---------------------------------- EOF ------------------------------------*/
