/*
 * Copyright (c) 1995, 1996 Hellmuth Michaelis. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *---------------------------------------------------------------------------*
 *
 *	bisdnd - BSD ISDN management daemon
 *      ===================================
 *
 *	Last Edit-Date: [Fri Jul  5 12:05:53 1996]
 *
 *	-hm	call get_current_rate() for dynamic timeout values
 *	-hm	new get_current_rate()
 *	-hm	timeout/idletime debugging/fixing
 *
 *----------------------------------------------------------------------------*/

#include <sys/types.h>

#include "defs.h"

static dial_t dial[NIPI];
static accept_t ap[NIPI];

static struct conf_tab
{
	short channel;
	struct confdata *conf;
} conf_tab[NIPI];

/*---------------------------------------------------------------------------*
 *	handle connection message for ip stuff
 *---------------------------------------------------------------------------*/
accept_t *
connect_ip(int appl_no, int channel, int active)
{
	accept_t *accept_p;
	struct confdata *confdata_p;

	if(conf_tab[appl_no].channel != INVALID)
		return(NULL);

	conf_tab[appl_no].channel = channel;

	accept_p = &ap[appl_no];

	confdata_p = conf_tab[appl_no].conf;

	accept_p->appl = appl_no;
	accept_p->atyp = ATYP_IP;
	accept_p->ctrl = channel;
	accept_p->service = confdata_p->service;
	accept_p->blocksize = confdata_p->blocksize;

	if(active)
	{
		accept_p->timeout = get_current_rate(confdata_p, 1);
		accept_p->idle_time = confdata_p->idletime_out;
	}
	else
	{
		accept_p->timeout = 0;		
		accept_p->idle_time = confdata_p->idletime_in;
	}

	accept_p->alert = 0;

	return(accept_p);
}

/*---------------------------------------------------------------------------*
 *	invalidate channel after disconnect message
 *---------------------------------------------------------------------------*/
void
disconnect_ip(int appl_no)
{
	conf_tab[appl_no].channel = INVALID;
}

/*---------------------------------------------------------------------------*
 *	fill accept struct with ip connection parameter values
 *---------------------------------------------------------------------------*/
int
ip_fill_accept(accept_t **accept_pp, int channel, struct confdata *confdata_p)
{
	accept_t *accept_p;

	if(conf_tab[confdata_p->appl].channel != INVALID)
		return(FA_ERROR);

	accept_p = &ap[confdata_p->appl];

	accept_p->appl = confdata_p->appl;
	accept_p->atyp = ATYP_IP;
	accept_p->ctrl = channel;
	accept_p->service = confdata_p->service;
	accept_p->blocksize = confdata_p->blocksize;

	accept_p->timeout = 0;
	accept_p->idle_time = confdata_p->idletime_in;

	accept_p->alert = 0;

	*accept_pp = accept_p;
	
	return(FA_GOOD);
}

/*---------------------------------------------------------------------------*
 *	get destination telno
 *---------------------------------------------------------------------------*/
dial_t *
getno_ip(int appl_no)
{
	dial_t *dial_p;
	struct confdata *confdata_p;

	if(conf_tab[appl_no].channel != INVALID)
		return(NULL);

	dial_p = &dial[appl_no];
	confdata_p = conf_tab[appl_no].conf;

	dial_p->appl = appl_no;
	dial_p->atyp = ATYP_IP;
	dial_p->service = confdata_p->service;
	dial_p->blocksize = confdata_p->blocksize;

	dial_p->timeout = get_current_rate(confdata_p, 0);
	dial_p->idle_time = confdata_p->idletime_out;

	strcpy(dial_p->src_telno, confdata_p->telnloc_ldo);
	strcpy(dial_p->dst_telno, confdata_p->telnrem_ldo);

	return(dial_p);
}

/*---------------------------------------------------------------------------*
 *	cold init
 *---------------------------------------------------------------------------*/
void
cold_init_ip(void)
{
	int i;

	for(i= 0; i< NIPI; i++)
	{
		conf_tab[i].channel = INVALID;
		conf_tab[i].conf = NULL;
	}
}

/*---------------------------------------------------------------------------*
 *	configuration time support
 *---------------------------------------------------------------------------*/
int
config_ip(int appl_no, struct confdata *confdata_p)
{
	if(conf_tab[appl_no].conf)
		return(1);

	conf_tab[appl_no].conf = confdata_p;

	return(0);
}

/* EOF */
