/*
 * Copyright (c) 1994, 1996 Hellmuth Michaelis. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *---------------------------------------------------------------------------*
 *
 *	bisdnd - BSD ISDN management daemon
 *      ===================================
 *
 *	Last Edit-Date: [Fri Jul  5 12:06:42 1996]
 *
 *	-hm	no subaddressing anymore
 *	-hm	rewriting routines for new call scheme
 *	-hm	idle time and unit lenght adjustments
 *
 *----------------------------------------------------------------------------*/

#include <stdlib.h>

#include <sys/param.h>
#include <sys/types.h>

#include "defs.h"

static accept_t ap[NITEL];

/*---------------------------------------------------------------------------*
 *	run application for playback/answering a telephone call
 *---------------------------------------------------------------------------*/
accept_t *
connect_tel(int appl_no, int channel, int active)
{
	u_char cmd_buf[MAXPATHLEN];
	register struct answ *answ_p;
	int length;
	int ret;
	
	answ_p = &answ[appl_no];
	
	answ_p->cn = channel;

	/* path to library directory -> buffer */
	
	length = sprintf(&cmd_buf[0], ETCPATH);

	/* application program name -> buffer */
	
	if(answ_p->conf->telaction == ACT_PLAYBK)
		length += sprintf(&cmd_buf[length], "/%s", playprog);
	else
		length += sprintf(&cmd_buf[length], "/%s", answerprog);

	/* if destination telephone number avail, add it as argument */
	
	if(answ_p->dst_telno)
		length += sprintf(&cmd_buf[length]," -d %s", answ_p->dst_telno);
	else
		length += sprintf(&cmd_buf[length]," -d unknown");
	
	/* if source telephone number avail, add it as argument */
	
	if(answ_p->src_telno)
		length += sprintf(&cmd_buf[length]," -s %s", answ_p->src_telno);
	else
		length += sprintf(&cmd_buf[length]," -s unknown");	

#if !defined (DEBUG)
	length += sprintf(&cmd_buf[length]," 1>&0 2>&0 &");
#endif

	/* execute command */

#if DEBUG
	if(do_debug & DEBUGL_NORM)
		log(LL_DBG, "phone: exec system(%s)", cmd_buf);
#endif
	
	ret = system(cmd_buf);

#if DEBUG
	if(do_debug & DEBUGL_NORM)
		log(LL_DBG, "phone: system() returned %d", ret);
#endif
	
	return(NULL);
}

/*---------------------------------------------------------------------------*
 *	disconnect the telephone
 *---------------------------------------------------------------------------*/
void
disconnect_tel(int appl_no)
{
	register struct answ *answ_p;

	answ_p = &answ[appl_no];
	
	answ_p->an = INVALID;
	answ_p->cn = INVALID;
	answ_p->conf = NULL;
}

/*---------------------------------------------------------------------------*
 *	enter parms into free answer slot
 *---------------------------------------------------------------------------*/
int
tel_fill_accept(accept_t **accept_pp, int channel, struct confdata *confdata_p,
                char *dst_telno, char *src_telno)
{
	register struct answ *answ_p;
	register int i;

	answ_p = &answ[0];

	for(i = 0; i < NITEL; i++, answ_p++)
	{
		if((answ_p->cn < 0) && (answ_p->an < 0))
		{
			accept_t *accept_p;

			accept_p = &ap[i];
			
			accept_p->ctrl = channel;
			accept_p->atyp = ATYP_TEL; 
			accept_p->appl = i;
			accept_p->service = confdata_p->service;
			accept_p->blocksize = confdata_p->blocksize;

			accept_p->timeout = 0;
			accept_p->idle_time = confdata_p->idletime_in;

			accept_p->alert = 0;

			answ_p->an = i;
			answ_p->conf = confdata_p;

			strcpy(answ_p->dst_telno, dst_telno);
			strcpy(answ_p->src_telno, src_telno);

			*accept_pp = accept_p;			
			return(FA_GOOD);
		}
	}
	return(FA_ERROR);
}

/*---------------------------------------------------------------------------*
 *	automatic dialing support
 *---------------------------------------------------------------------------*/
dial_t *
getno_tel(int appl_no)
{
	return(NULL);
}

/*---------------------------------------------------------------------------*
 *	init the answer structure array
 *---------------------------------------------------------------------------*/
void
cold_init_tel(void)
{
	register struct answ *answ_p;
	register int i;
	
	answ_p = &answ[0]; 

	for(i = 0; i < NITEL; i++, answ_p++)
	{
		answ_p->an = INVALID;
		answ_p->cn = INVALID;
		answ_p->conf = NULL;
	}
}

/*---------------------------------------------------------------------------*
 *	configuration time support
 *---------------------------------------------------------------------------*/
int
config_tel(int appl_no, struct confdata *cp)
{
	return(0);
}

/* EOF */
