/*
 * Copyright (c) 1996 Hellmuth Michaelis. All rights reserved.
 *
 * Copyright (c) 1996 Gary Jennejohn. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *---------------------------------------------------------------------------*
 *
 *	bisdnd - BSD ISDN management daemon
 *      ===================================
 *
 *	Last Edit-Date: [Fri Jul  5 12:06:53 1996]
 *
 *	-hm	start over where i left a year ago
 *	-hm	short hold implementation
 *	-hm	made getting the rate dynamic, get_current_rate()
 *	-hm	completely rewrote get_current_rate(), intro of timeout-type
 *	-hm	rewrote parse_rate into readrates
 *	-gj	implemented multiple rates
 *	-hm	cleanup, warning return
 *	-hm	idle time cleanup
 *	-hm	unitlength/earlyhangup modifications
 *
 *---------------------------------------------------------------------------*/

/* in case the unit length values in the are file are times 10 the real */
/* value, define this and a division by ten is done when fetching the	*/
/* actual value. From the result, SH_SECS_EARLY is subtracted and the	*/
/* result is the timeout value used.					*/

#define RATES_TIMES_TEN		/* rates are times 10 in rates file */

static char error[256];

#ifdef PARSE_DEBUG_MAIN

#include <stdio.h>

#define MAIN

#define ERROR (-1)

int main()
{
	int ret;
	ret = readrates("/etc/bisdn.rates");
	if(ret == ERROR)
		fprintf(stderr, "readrates returns [%d], [%s]\n", ret, error);
	else
		fprintf(stderr, "readrates returns [%d]\n", ret);
	return(ret);
}

#endif


#include <errno.h>
#include <ctype.h>
#include <stdlib.h>

#include <sys/param.h>
#include <sys/time.h>

#include "defs.h"

static int getrate(struct confdata *cp, int logit);

/*---------------------------------------------------------------------------*
 *	parse rates file
 *---------------------------------------------------------------------------*/
int readrates(char *filename)
{
	char buffer[1024];
	register char *bp;
	struct rates *rt, *ort;
	int rateindx;
	int indx;
	int line = 0;
	FILE *fp;
	int first;
#if DEBUG
	int i, j;
#endif
	
	indx = 0;
	rt = ort = NULL;

	if((fp = fopen(filename, "r")) == NULL)
	{
		sprintf(error, "error open %s: %s", filename, sys_errlist[errno]);
		rate_error = error;
		return(WARNING);
	}

	while((fgets(buffer, MAXPATHLEN, fp)) != NULL)
	{
		line++;

/* comments */
		if(buffer[0] == '#'  || buffer[0] == ' ' ||
		   buffer[0] == '\t' || buffer[0] == '\n')
		{
			continue;
		}

		bp = &buffer[0];

		/* rate type */

		if (*bp == 'r' && *(bp+1) == 'a' && isdigit(*(bp+2)))
		{
				rateindx = *(bp+2) - '0';
				bp += 3;

				/* eat space delimiter */

				while(isspace(*bp))
					bp++;
		}
		else
		{
			sprintf(error, "rates: invalid rate type %c%c%c in line %d", *bp, *(bp+1), *(bp+2), line);
			goto rate_error;
		}
		if (rateindx >= NRATES)
		{
			sprintf(error, "rates: invalid rate index %d in line %d", rateindx, line);
			goto rate_error;
		}

		/* day */
		
		if(isdigit(*bp) && *bp >= '0' && *bp <= '6')
		{
			indx = *bp - '0';
#if DEBUG > 1
			if(do_debug & DEBUGL_RATES)
				log(LL_DBG, "rates: index = %d", indx);
#endif
		}
		else
		{
			sprintf(error, "rates: invalid day digit %c in line %d", *bp, line);
			goto rate_error;
		}

		if(rates[rateindx][indx] == NULL)
		{
			rt = (struct rates *)malloc(sizeof (struct rates));
			if (rt == NULL)
			{
				sprintf(error, "rates: cannot malloc space for rate structure");
				goto rate_error;
		  	}
			rt->next = NULL;
		  	rates[rateindx][indx] = rt;
		}

		bp++;
		
		/* eat space delimiter */

		while(isspace(*bp))
			bp++;

		/* now loop to get the rates entries */

		first = 1;
		
		while(*bp && isdigit(*bp))
		{
			if(first)
			{
				first = 0;
			}
			else
			{
				ort = rt;
	
				rt = (struct rates *)malloc(sizeof (struct rates));
				if (rt == NULL)
				{
					sprintf(error, "rates: cannot malloc space2 for rate structure");
					goto rate_error;
			  	}
				ort->next = rt;
				rt->next = NULL;
			}
			
			/* start hour */
			
			if(isdigit(*bp) && isdigit(*(bp+1)))
			{
				rt->start_hr = atoi(bp);
				bp += 2;
			}
		  	else
			{
				sprintf(error, "rates: start_hr error in line %d", line);
				goto rate_error;
		  	}

			/* point */
			
		  	if(*bp == '.')
		  	{
		  		bp++;
		  	}
		  	else
		  	{
				sprintf(error, "rates: no '.' after start_hr in line %d", line);
				goto rate_error;
			}
		  	
			/* start minute */
			
			if(isdigit(*bp) && isdigit(*(bp+1)))
			{
				rt->start_min = atoi(bp);
				bp += 2;
			}
		  	else
			{
				sprintf(error, "rates: start_min error in line %d", line);
				goto rate_error;
		  	}

			/* minus */
			
		  	if(*bp == '-')
		  	{
		  		bp++;
		  	}
		  	else
		  	{
				sprintf(error, "rates: no '-' after start_min in line %d", line);
				goto rate_error;
			}

			/* end hour */
			
			if(isdigit(*bp) && isdigit(*(bp+1)))
			{
				rt->end_hr = atoi(bp);
				bp += 2;
			}
		  	else
			{
				sprintf(error, "rates: end_hr error in line %d", line);
				goto rate_error;
		  	}

			/* point */
			
		  	if(*bp == '.')
		  	{
		  		bp++;
		  	}
		  	else
		  	{
				sprintf(error, "rates: no '.' after end_hr in line %d", line);
				goto rate_error;
			}
		  	
			/* end minute */
			
			if(isdigit(*bp) && isdigit(*(bp+1)))
			{
				rt->end_min = atoi(bp);
				bp += 2;
			}
		  	else
			{
				sprintf(error, "rates: end_min error in line %d", line);
				goto rate_error;
		  	}

			/* colon */
			
		  	if(*bp == ':')
		  	{
		  		bp++;
		  	}
		  	else
		  	{
				sprintf(error, "rates: no ':' after end_min in line %d", line);
				goto rate_error;
			}

			/* time */
			
			if(isdigit(*bp))
			{
				rt->rate = atoi(bp);
				while(!isspace(*bp))
					bp++;
			}
		  	else
			{
				sprintf(error, "rates: first rate digit error in line %d", line);
				goto rate_error;
		  	}

			/* eat space delimiter */

			while(isspace(*bp))
				bp++;
		}
	}

#if DEBUG
	if(do_debug & DEBUGL_RATES)
	{
		for (j = 0; j < NRATES; j++)
		{
			for (i = 0; i < NDAYS; i++)
			{
				if (rates [j][i] != NULL)
				{
					rt = rates [j][i];
					for (; rt; rt = rt->next)
					{
						log(LL_DBG, "rates: index %d day %d = %d.%d-%d.%d:%d",
							j, i, rt->start_hr, rt->start_min,
							rt->end_hr,rt->end_min,rt->rate);
					}
				}
				else
				{
					log(LL_DBG, "rates: NO entry for day %d !!\n", i);
				}
			}
		}
	}
#endif
	fclose(fp);
	return(GOOD);

rate_error:
	fclose(fp);
	rate_error = error;
	return(ERROR);
}

#ifndef PARSE_DEBUG_MAIN

/*---------------------------------------------------------------------------*
 *	get unit length time from configured source
 *---------------------------------------------------------------------------*/
int get_current_rate(struct confdata *cp, int logit)
{
	int rt;
	
	switch(cp->unit_length_typ)
	{
		case ULTYP_CMDL:	/* specified on commandline     */
			if(logit)
				log(LL_SYS, "              rate=%d sec/unit (commandline)", unit_length);
			return(unit_length - (cp->earlyhangup));
			break;

		case ULTYP_CONF:	/* get it from config file      */
			if(logit)
				log(LL_SYS, "              rate=%d sec/unit (config file)", cp->unitlength);
			return(cp->unitlength - (cp->earlyhangup));

		case ULTYP_RATE:	/* get it dynamic from ratesfile*/
			if(!got_rate)	/* got valid rates struct ?? */
			{
				if(logit)
					log(LL_SYS, "              rate=%d sec/unit (no ratefile)", UNITLENGTH_DEFAULT);
				return(UNITLENGTH_DEFAULT - (cp->earlyhangup));	/* no ... */
			}
			if((cp->ratetyp >= NRATES) ||
			   (cp->ratetyp == INVALID_RATE))
			{
				if(logit)
					log(LL_SYS, "              rate=%d sec/unit (ratetyp out of range)", UNITLENGTH_DEFAULT);
				return(UNITLENGTH_DEFAULT - (cp->earlyhangup));	/* no ... */
			}
			
			if((rt = getrate(cp, logit)) != -1)
				return(rt - (cp->earlyhangup));

			if(logit)			
				log(LL_SYS, "              rate=%d sec/unit (ratescan fail)", UNITLENGTH_DEFAULT);

			return(UNITLENGTH_DEFAULT - (cp->earlyhangup));	/* no ... */		
			break;

		case ULTYP_DYN:	/* dynamically calculated from AOC */
			if((rt = getrate(cp, 0)) != -1)
			{
				if(logit)
					log(LL_SYS, "              rate=dynamically (initially %d secs from rate file)", rt);
				return(rt - (cp->earlyhangup));
			}
			if(logit)
				log(LL_SYS, "              rate=dynamically (initially %d secs, default value)", UNITLENGTH_DEFAULT);
			return(UNITLENGTH_DEFAULT - (cp->earlyhangup));
			break;

		default:
			if(logit)
				log(LL_SYS, "              rate=%d sec/unit (unit_length_typ unknown)", UNITLENGTH_DEFAULT);
			return(UNITLENGTH_DEFAULT - (cp->earlyhangup));	/* no ... */		
			break;
	}
}

/*---------------------------------------------------------------------------*
 *	get a rate from run time configured source
 *---------------------------------------------------------------------------*/
static int getrate(struct confdata *cp, int logit)
{
	struct tm *ptr;
	time_t now;
	register struct rates *hd;

	if((!got_rate) ||
	   (cp->ratetyp >= NRATES) ||
	   (cp->ratetyp == INVALID_RATE))
	{
		return(-1);
	}

	time(&now);			/* get current time */

	ptr = localtime(&now);

	/* walk thru the rates for weekday until rate for current time found */

	for (hd = rates[cp->ratetyp][ptr->tm_wday]; hd; hd = hd->next)
	{
		/* current time within window ? */
		if((hd->start_hr <= ptr->tm_hour) &&
		   (hd->end_hr   >  ptr->tm_hour))
		{
#ifdef RATES_TIMES_TEN
			int myrate = (hd->rate / 10);
			if(logit)
				log(LL_SYS, "              rate=%d (%d) sec/unit (day=%d, beg=%d, end=%d, current=%d)",
					hd->rate, myrate,
					ptr->tm_wday,
					hd->start_hr,
					hd->end_hr,
					ptr->tm_hour);
			return (myrate);
#else
			if(logit)
				log(LL_SYS, "              rate=%d sec/unit (day=%d, beg=%d, end=%d, current=%d)",
					hd->rate,
					ptr->tm_wday,
					hd->start_hr,
					hd->end_hr,
					ptr->tm_hour);
			return (hd->rate);
#endif
		}
	}
	return(-1);
}

#endif /* PARSE_DEBUG_MAIN */

/* EOF */
