/*
 * Copyright (c) 1994, 1996 Hellmuth Michaelis.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Hellmuth Michaelis
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
*---------------------------------------------------------------------------*
 *
 *	history:	last edit-date: [Fri Jul  5 12:15:27 1996]
 *	--------
 *
 *	-hm	1tr6 cause codes
 *	-hm	DSS1 cause codes
 *	-hm	removing bit 8 hack to distinguish protocols
 *	-hm	case values shown with bit = off now
 *
 *---------------------------------------------------------------------------*/

#include <stdio.h>

#include <machine/bisdn_ioctl.h>

#include "pcause.h"

char *
print_error(int prot, unsigned char code)
{
	static char error_message[120];
	char *e;

	code |= 0x80;	/* we always switch on bit 8, because the tables */
			/* use bit=1 and i'm too lazy to rewrite it	 */

	switch(prot)
	{
		case ISDN1TR6:

			switch(code)
			{
				case CAUSE_1TR6_SHUTDN:
					e = "normal D-channel shutdown";
					break;
		
				case CAUSE_1TR6_ICRV:
					e = "invalid call reference value";
					break;
		
				case CAUSE_1TR6_BSNI:
					e = "bearer service not implemented";
					break;
		
				case CAUSE_1TR6_CIDNE:
					e = "call identity does not exist";
					break;
		
				case CAUSE_1TR6_CIIU:
					e = "call identity in use";
					break;
		
				case CAUSE_1TR6_NCA:
					e = "no channel available";
					break;
		
				case CAUSE_1TR6_RFNI:
					e = "requested facility not implemented";
					break;
		
				case CAUSE_1TR6_RFNS:
					e = "requested facility not subscribed";
					break;
		
				case CAUSE_1TR6_OCB:
					e = "outgoing calls barred";
					break;
		
				case CAUSE_1TR6_UAB:
					e = "user access busy";
					break;
		
				case CAUSE_1TR6_NECUG:
					e = "non existent CUG";
					break;
		
				case CAUSE_1TR6_NECUG1:
					e = "non existent CUG";
					break;
		
				case CAUSE_1TR6_SPV:
					e = "kommunikationsbeziehung als SPV nicht erlaubt";
					break;
		
				case CAUSE_1TR6_DNO:
					e = "destination not obtainable";
					break;
		
				case CAUSE_1TR6_NC:
					e = "number changed";
					break;
		
				case CAUSE_1TR6_OOO:
					e = "out of order";
					break;
		
				case CAUSE_1TR6_NUR:
					e = "no user responding";
					break;
		
				case CAUSE_1TR6_UB:
					e = "user busy";
					break;
		
				case CAUSE_1TR6_ICB:
					e = "incoming calls barred";
					break;
		
				case CAUSE_1TR6_CR:
					e = "call rejected";
					break;
		
				case CAUSE_1TR6_NCO:
					e = "network congestion";
					break;
		
				case CAUSE_1TR6_RUI:
					e = "remote user initiated";
					break;
		
				case CAUSE_1TR6_LPE:
					e = "local procedure error";
					break;
		
				case CAUSE_1TR6_RPE:
					e = "remote procedure error";
					break;
		
				case CAUSE_1TR6_RUS:
					e = "remote user suspended";
					break;
		
				case CAUSE_1TR6_RUR:
					e = "remote user resumed";
					break;
		
				case CAUSE_1TR6_UIDL:
					e = "user info discharded locally";
					break;

				default:
					e = "UNKNOWN error occured";
					break;
			}
			break;

		case ISDNDSS1:

			switch(code)
			{
				case CAUSE_DSS1_SHUTDN:
					e = "normal D-channel shutdown";
					break;

				case CAUSE_DSS1_NUNALLC:
					e = "Unallocated (unassigned) number";
					break;

				case CAUSE_DSS1_NRTTN:
					e = "No route to specified transit network";
					break;

				case CAUSE_DSS1_NRTDST:
					e = "No route to destination";
					break;

				case CAUSE_DSS1_SSINFTN:
					e = "Send special information tone";
					break;

				case CAUSE_DSS1_MDIALTP:
					e = "Misdialled trunk prefix";
					break;

				case CAUSE_DSS1_CHUNACC:
					e = "Channel unacceptable";
					break;

				case CAUSE_DSS1_CALLAWD:
					e = "Call awarded and being delivered in an established channel";
					break;

				case CAUSE_DSS1_PREEMPT:
					e = "Preemption";
					break;

				case CAUSE_DSS1_PREECRR:
					e = "Preemption - circuit reserved for reuse";
					break;

				case CAUSE_DSS1_NCCLR:
					e = "Normal call clearing";
					break;

				case CAUSE_DSS1_USRBSY:
					e = "User busy";
					break;

				case CAUSE_DSS1_NOUSRRSP:
					e = "No user responding";
					break;

				case CAUSE_DSS1_NOANSWR:
					e = "No answer from user (user alerted)";
					break;

				case CAUSE_DSS1_SUBSABS:
					e = "Subscriber absent";
					break;

				case CAUSE_DSS1_CALLREJ:
					e = "Call rejected";
					break;

				case CAUSE_DSS1_NUCHNG:
					e = "Number changed";
					break;

				case CAUSE_DSS1_NONSELUC:
					e = "Non-selected user clearing";
					break;

				case CAUSE_DSS1_DSTOOORDR:
					e = "Destination out of order";
					break;

				case CAUSE_DSS1_INVNUFMT:
					e = "Invalid number format";
					break;

				case CAUSE_DSS1_FACREJ:
					e = "Facility rejected";
					break;

				case CAUSE_DSS1_STENQRSP:
					e = "Response to STATUS ENQUIRY";
					break;

				case CAUSE_DSS1_NORMUNSP:
					e = "Normal, unspecified";
					break;

				case CAUSE_DSS1_NOCAVAIL:
					e = "No circuit / channel available";
					break;

				case CAUSE_DSS1_NETOOORDR:
					e = "Network out of order";
					break;

				case CAUSE_DSS1_PFMCDOOSERV:
					e = "Permanent frame mode connection out of service";
					break;

				case CAUSE_DSS1_PFMCOPER:
					e = "Permanent frame mode connection operational";
					break;

				case CAUSE_DSS1_TMPFAIL:
					e = "Temporary failure";
					break;

				case CAUSE_DSS1_SWEQCONG:
					e = "Switching equipment congestion";
					break;

				case CAUSE_DSS1_ACCINFDIS:
					e = "Access information discarded";
					break;

				case CAUSE_DSS1_REQCNOTAV:
					e = "Requested circuit/channel not available";
					break;

				case CAUSE_DSS1_PRECALBLK:
					e = "Precedence call blocked";
					break;

				case CAUSE_DSS1_RESUNAVAIL:
					e = "Resources unavailable, unspecified";
					break;

				case CAUSE_DSS1_QOSUNAVAIL:
					e = "Quality of service unavailable";
					break;

				case CAUSE_DSS1_REQSERVNS:
					e = "Requested facility not subscribed";
					break;

				case CAUSE_DSS1_OCBARRCUG:
					e = "Outgoing calls barred within CUG";
					break;

				case CAUSE_DSS1_ICBARRCUG:
					e = "Incoming calls barred within CUG";
					break;

				case CAUSE_DSS1_BCAPNAUTH:
					e = "Bearer capability not authorized";
					break;

				case CAUSE_DSS1_BCAPNAVAIL:
					e = "Bearer capability not presently available";
					break;

				case CAUSE_DSS1_INCSTOACISC:
					e = "Inconsistenciy in designated outg. access info and subscriber class";
					break;

				case CAUSE_DSS1_SOONOTAVAIL:
					e = "Service or option not available, unspecified";
					break;

				case CAUSE_DSS1_BCAPNOTIMPL:
					e = "Bearer capability not implemented";
					break;

				case CAUSE_DSS1_CHTYPNIMPL:
					e = "Channel type not implemented";
					break;

				case CAUSE_DSS1_REQFACNIMPL:
					e = "Requested facility not implemented";
					break;

				case CAUSE_DSS1_ORDINBCAVL:
					e = "Only restricted digital information bearer capability is available";
					break;

				case CAUSE_DSS1_SOONOTIMPL:
					e = "Service or option not implemented, unspecified";
					break;

				case CAUSE_DSS1_INVCLRFVAL:
					e = "Invalid call reference value";
					break;

				case CAUSE_DSS1_IDCHDNOEX:
					e = "Identified channel does not exist";
					break;

				case CAUSE_DSS1_SUSCAEXIN:
					e = "A suspended call exists, but this call identity does not";
					break;

				case CAUSE_DSS1_CLIDINUSE:
					e = "Call identity in use";
					break;

				case CAUSE_DSS1_NOCLSUSP:
					e = "No call suspended";
					break;

				case CAUSE_DSS1_CLIDCLRD:
					e = "Call having the requested call identity has been cleared";
					break;

				case CAUSE_DSS1_UNOTMEMCUG:
					e = "User not member of CUG";
					break;

				case CAUSE_DSS1_INCDEST:
					e = "Incompatible destination";
					break;

				case CAUSE_DSS1_NONEXCUG:
					e = "Non-existent CUG";
					break;

				case CAUSE_DSS1_INVNTWSEL:
					e = "Invalid transit network selection";
					break;

				case CAUSE_DSS1_INVMSG:
					e = "Invalid message, unspecified";
					break;

				case CAUSE_DSS1_MIEMISS:
					e = "Mandatory information element is missing";
					break;

				case CAUSE_DSS1_MSGTNI:
					e = "Message type non-existent or not implemented";
					break;

				case CAUSE_DSS1_MSGNCMPT:
					e = "Msg incompatible with call state/message type non-existent/not implemented";
					break;

				case CAUSE_DSS1_IENENI:
					e = "Information element/parameter non-existent or not implemented";
					break;

				case CAUSE_DSS1_INVIEC:
					e = "Invalid information element contents";
					break;

				case CAUSE_DSS1_MSGNCWCS:
					e = "Message not compatible with call state";
					break;

				case CAUSE_DSS1_RECOTIMEXP:
					e = "Recovery on timer expiry";
					break;

				case CAUSE_DSS1_PARMNENIPO:
					e = "Parameter non-existent or not implemented, passed on";
					break;

				case CAUSE_DSS1_MSGUNRDPRM:
					e = "Message with unrecognized parameter, discarded";
					break;

				case CAUSE_DSS1_PROTERR:
					e = "Protocol error, unspecified";
					break;

				case CAUSE_DSS1_INTWRKU:
					e = "Interworking, unspecified";
					break;

				default:
					e = "UNKNOWN error occured";
					break;
			}
			break;
		default:
			e = "ERROR - unknown protocol type specified !!!";
			break;
	}
	sprintf(error_message, "0x%02x: %s", code & 0x7f, e);	
	return(error_message);
}

/* EOF */
