/*
 * Copyright (c) 1994, 1996 Hellmuth Michaelis.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Hellmuth Michaelis
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *---------------------------------------------------------------------------*
 *
 *	pcause.h - 1TR6/DSS1 causes definitions
 *	---------------------------------------
 *
 *	last edit-date: [Thu Apr 18 14:27:20 1996]
 *
 *	-hm	1TR6 causes from another project ...
 *	-hm	adding DSS1 causes
 *	-hm	print_error declaration
 *	-hm	removing bit 8 hack to distinguish protocols
 *
 *---------------------------------------------------------------------------*/

char *print_error(int prot, unsigned char code);

/* 1TR6 protocol causes */

#define	CAUSE_1TR6_SHUTDN	0x80	/* normal D-channel shutdown */
#define CAUSE_1TR6_ICRV		0x81	/* invalid call reference value */
#define CAUSE_1TR6_BSNI		0x83	/* bearer service not implemented */
#define CAUSE_1TR6_CIDNE	0x87	/* call identity does not exist */
#define CAUSE_1TR6_CIIU		0x88	/* call identity in use */
#define CAUSE_1TR6_NCA		0x8A	/* no channel available */
#define CAUSE_1TR6_RFNI		0x90	/* requested facility not implemented */
#define CAUSE_1TR6_RFNS		0x91	/* requested facility not subscribed */
#define CAUSE_1TR6_OCB		0xA0	/* outgoing calls barred */
#define CAUSE_1TR6_UAB		0xA1	/* user access busy */
#define CAUSE_1TR6_NECUG	0xA2	/* non existent CUG */
#define CAUSE_1TR6_NECUG1	0xA3	/* non existent CUG */
#define CAUSE_1TR6_SPV		0xA5	/* kommunikationsbeziehung als SPV nicht erlaubt */
#define CAUSE_1TR6_DNO		0xB5	/* destination not obtainable */
#define CAUSE_1TR6_NC		0xB8	/* number changed */
#define CAUSE_1TR6_OOO		0xB9	/* out of order */
#define CAUSE_1TR6_NUR		0xBA	/* no user responding */
#define CAUSE_1TR6_UB		0xBB	/* user busy */
#define CAUSE_1TR6_ICB		0xBD	/* incoming calls barred */
#define CAUSE_1TR6_CR		0xBE	/* call rejected */
#define CAUSE_1TR6_NCO		0xD9	/* network congestion */
#define CAUSE_1TR6_RUI		0xDA	/* remote user initiated */
#define CAUSE_1TR6_LPE		0xF0	/* local procedure error */
#define CAUSE_1TR6_RPE		0xF1	/* remote procedure error */
#define CAUSE_1TR6_RUS		0xF2	/* remote user suspended */
#define CAUSE_1TR6_RUR		0xF3	/* remote user resumed */
#define CAUSE_1TR6_UIDL		0xFF	/* user info discharded locally */

/* DSS1 protocol causes */

#define	CAUSE_DSS1_SHUTDN	0x80	/* normal D-channel shutdown */
#define CAUSE_DSS1_NUNALLC	0x81	/* Unallocated (unassigned) number */
#define CAUSE_DSS1_NRTTN	0x82	/* No route to specified transit network */
#define CAUSE_DSS1_NRTDST	0x83	/* No route to destination */
#define CAUSE_DSS1_SSINFTN	0x84	/* Send special information tone */
#define CAUSE_DSS1_MDIALTP	0x85	/* Misdialled trunk prefix */
#define CAUSE_DSS1_CHUNACC	0x86	/* Channel unacceptable */
#define CAUSE_DSS1_CALLAWD	0x87	/* Call awarded and being delivered in an established channel */
#define CAUSE_DSS1_PREEMPT	0x88	/* Preemption */
#define CAUSE_DSS1_PREECRR	0x89	/* Preemption - circuit reserved for reuse */
#define CAUSE_DSS1_NCCLR	0x90	/* Normal call clearing */
#define CAUSE_DSS1_USRBSY	0x91	/* User busy */
#define CAUSE_DSS1_NOUSRRSP	0x92	/* No user responding */
#define CAUSE_DSS1_NOANSWR	0x93	/* No answer from user (user alerted) */
#define CAUSE_DSS1_SUBSABS	0x94	/* Subscriber absent */
#define CAUSE_DSS1_CALLREJ	0x95	/* Call rejected */
#define CAUSE_DSS1_NUCHNG	0x96	/* Number changed */
#define CAUSE_DSS1_NONSELUC	0x9A	/* Non-selected user clearing */
#define CAUSE_DSS1_DSTOOORDR	0x9B	/* Destination out of order */
#define CAUSE_DSS1_INVNUFMT	0x9C	/* Invalid number format */
#define CAUSE_DSS1_FACREJ	0x9D	/* Facility rejected */
#define CAUSE_DSS1_STENQRSP	0x9E	/* Response to STATUS ENQUIRY */
#define CAUSE_DSS1_NORMUNSP	0x9F	/* Normal, unspecified */
#define CAUSE_DSS1_NOCAVAIL	0xA2	/* No circuit / channel available */
#define CAUSE_DSS1_NETOOORDR	0xA6	/* Network out of order */
#define CAUSE_DSS1_PFMCDOOSERV	0xA7	/* Permanent frame mode connection out of service */
#define CAUSE_DSS1_PFMCOPER	0xA8	/* Permanent frame mode connection operational */
#define CAUSE_DSS1_TMPFAIL	0xA9	/* Temporary failure */
#define CAUSE_DSS1_SWEQCONG	0xAA	/* Switching equipment congestion */
#define CAUSE_DSS1_ACCINFDIS	0xAB	/* Access information discarded */
#define CAUSE_DSS1_REQCNOTAV	0xAC	/* Requested circuit/channel not available */
#define CAUSE_DSS1_PRECALBLK	0xAE	/* Precedence call blocked */
#define CAUSE_DSS1_RESUNAVAIL	0xAF	/* Resources unavailable, unspecified */
#define CAUSE_DSS1_QOSUNAVAIL	0xB1	/* Quality of service unavailable */
#define CAUSE_DSS1_REQSERVNS	0xB2	/* Requested facility not subscribed */
#define CAUSE_DSS1_OCBARRCUG	0xB5	/* Outgoing calls barred within CUG */
#define CAUSE_DSS1_ICBARRCUG	0xB6	/* Incoming calls barred within CUG */
#define CAUSE_DSS1_BCAPNAUTH	0xB9	/* Bearer capability not authorized */
#define CAUSE_DSS1_BCAPNAVAIL	0xBA	/* Bearer capability not presently available */
#define CAUSE_DSS1_INCSTOACISC	0xBE	/* Inconsistenciy in designated outgoing access information and subscriber class */
#define CAUSE_DSS1_SOONOTAVAIL	0xBF	/* Service or option not available, unspecified */
#define CAUSE_DSS1_BCAPNOTIMPL	0xC1	/* Bearer capability not implemented */
#define CAUSE_DSS1_CHTYPNIMPL	0xC2	/* Channel type not implemented */
#define CAUSE_DSS1_REQFACNIMPL	0xC5	/* Requested facility not implemented */
#define CAUSE_DSS1_ORDINBCAVL	0xC6	/* Only restricted digital information bearer capability is available */
#define CAUSE_DSS1_SOONOTIMPL	0xCF	/* Service or option not implemented, unspecified */
#define CAUSE_DSS1_INVCLRFVAL	0xD1	/* Invalid call reference value */
#define CAUSE_DSS1_IDCHDNOEX	0xD2	/* Identified channel does not exist */
#define CAUSE_DSS1_SUSCAEXIN	0xD3	/* A suspended call exists, but this call identity does not */
#define CAUSE_DSS1_CLIDINUSE	0xD4	/* Call identity in use */
#define CAUSE_DSS1_NOCLSUSP	0xD5	/* No call suspended */
#define CAUSE_DSS1_CLIDCLRD	0xD6	/* Call having the requested call identity has been cleared */
#define CAUSE_DSS1_UNOTMEMCUG	0xD7	/* User not member of CUG */
#define CAUSE_DSS1_INCDEST	0xD8	/* Incompatible destination */
#define CAUSE_DSS1_NONEXCUG	0xDA	/* Non-existent CUG */
#define CAUSE_DSS1_INVNTWSEL	0xDB	/* Invalid transit network selection */
#define CAUSE_DSS1_INVMSG	0xDF	/* Invalid message, unspecified */
#define CAUSE_DSS1_MIEMISS	0xE0	/* Mandatory information element is missing */
#define CAUSE_DSS1_MSGTNI	0xE1	/* Message type non-existent or not implemented */
#define CAUSE_DSS1_MSGNCMPT	0xE2	/* Message not compatible with call state or message type non-existent or not implemented */
#define CAUSE_DSS1_IENENI	0xE3	/* Information element/parameter non-existent or not implemented */
#define CAUSE_DSS1_INVIEC	0xE4	/* Invalid information element contents */
#define CAUSE_DSS1_MSGNCWCS	0xE5	/* Message not compatible with call state */
#define CAUSE_DSS1_RECOTIMEXP	0xE6	/* Recovery on timer expiry */
#define CAUSE_DSS1_PARMNENIPO	0xE7	/* Parameter non-existent or not implemented, passed on */
#define CAUSE_DSS1_MSGUNRDPRM	0xEE	/* Message with unrecognized parameter, discarded */
#define CAUSE_DSS1_PROTERR	0xEF	/* Protocol error, unspecified */
#define CAUSE_DSS1_INTWRKU	0xFF	/* Interworking, unspecified */

/* EOF */
