/*
 * Copyright (c) 1996 Hellmuth Michaelis.  All rights reserved.
 *
 * Copyright (c) 1996 Gary Jennejohn.  All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *---------------------------------------------------------------------------*
 *
 *	bisdntrc - print debugging traces of channel activity
 *	-----------------------------------------------------
 *
 *	last edit-date: [Fri Jul  5 15:21:17 1996]
 *
 *	-gj	split dumpbuf into decode_lapd and decode_q931
 *	-hm	use getopt for option decoding
 *	-hm	decoding 1TR6
 *	-hm	decoding DSS1
 *	-hm	setting output file to unbuffered
 *	-hm	splitting
 *	-hm	printing into buffer
 *	-hm	implementing protocol autodetection
 *	-hm	some cosmetic changes
 *	-hm	header print & msg len bugfix
 *	-hm	fixing B-channel dump
 *
 *---------------------------------------------------------------------------*/

#include "bisdntrc.h"

unsigned char buf[BSIZE];
FILE *Fout = NULL;
int outflag = 0;
int header = 1;
char *device = ISPYDEV;
int print_q921 = 0;

static void dumpbuf(int n, unsigned char *buf, unsigned char dir, int raw);
static void switch_driver(char *device, int value);

int
main(int argc, char *argv[])
{
	extern int optind;
	extern int opterr;
	extern char *optarg;
	
	int f;
	int n;
	int c;
	char *b = &buf[sizeof(struct timeval)+sizeof(unsigned long)+1];

	int enable_trace = 1;	
	char *outfile = "";
	int raw = 0;
	int noct = -1;
	
	while( (c = getopt(argc, argv, "d:o:i:thlrn:")) != EOF)
	{
		switch(c)
		{
			case 'd':
				device = optarg;
				break;

			case 'o':
				outflag = 1;
				outfile = optarg;
				break;

			case 'i':
				enable_trace = atoi(optarg);
				break;

			case 'n':
				noct = atoi(optarg);
				break;

			case 'h':
				header = 0;
				break;

			case 'l':
				print_q921 = 1;
				break;

			case 'r':
				raw = 1;
				break;

			case '?':
			default:
				usage();
				break;
		}
	}

	switch_driver(device, enable_trace);
	
	if ((f = open(SPYDEVICE, O_RDONLY)) < 0)
	{
		char buffer[80];

		sprintf(buffer, "Error opening trace device [%s]", SPYDEVICE);
		perror(buffer);
		exit(1);
	}

	if (outflag)
	{
		if((Fout = fopen(outfile, "w")) == NULL)
		{
			char buffer[80];

			sprintf(buffer, "Error opening file [%s]", outfile);
			perror(buffer);
			exit(1);
		}
		if((setvbuf(Fout, (char *)NULL, _IONBF, 0)) != 0)
		{
			char buffer[80];

			sprintf(buffer, "Error setting file [%s] to unbuffered", outfile);
			perror(buffer);
			exit(1);
		}
	}
			
	(void) signal(SIGHUP, catchsig);
	(void) signal(SIGTERM, catchsig);
	(void) signal(SIGKILL, catchsig);
	(void) signal(SIGINT, catchsig);
	(void) signal(SIGQUIT, catchsig);

	for (;;)
	{
		n = read(f, buf, BSIZE);
		n -= sizeof(struct timeval) + sizeof(unsigned long) +1;

		if((n > 0) && (n > noct))
		{
			dumpbuf(n, b, *buf, raw);
		}
	}
}

/*---------------------------------------------------------------------------*
 *	format header into static buffer, return buffer address
 *---------------------------------------------------------------------------*/
char *
fmt_hdr(int n)
{
	struct timeval *t = (struct timeval *) &buf[sizeof(unsigned long)+1];
	struct tm *s;

	static char hbuf[256];

	s = localtime(&t->tv_sec);

	sprintf(hbuf,"----- %s ---- Seq-No %6.6ld ---- Timestamp %2.2d:%2.2d:%2.2d.%2.2ld ----- Length %2.2d -----\n",
		((*buf & 1) ? "TE->NT" : "NT->TE"),
		*(unsigned long *) &buf[1],
		s->tm_hour,
		s->tm_min,
		s->tm_sec,
		t->tv_usec / 10000,
		n);

	return(hbuf);
}

/*---------------------------------------------------------------------------*
 *	decode protocol and output to file(s)
 *---------------------------------------------------------------------------*/
static void
dumpbuf(int n, unsigned char *buf, unsigned char dir, int raw)
{
	static unsigned char l2buf[32000];
	static unsigned char l3buf[32000];
	int cnt;
	int nsave = n;

	l2buf[0] = '\0';
	l3buf[0] = '\0';
	
	if((dir & BIT_DB) == D_CHAN)
	{
		/* D-channel data */

		cnt = decode_lapd(l2buf, n, buf, (dir & BIT_DIR), raw, print_q921);
	
		n -= cnt;
		buf += cnt;
	
		if(n)
		{
			switch(*buf)
			{
				case 0x40:
				case 0x41:
					decode_1tr6(l3buf, n, buf, raw);
					break;
					
				default:
					decode_q931(l3buf, n, buf, raw);
					break;
			}
		}
	}
	else
	{
		/* B-channel data */

		unsigned char *pbuf = &l2buf[0];
		int i, j;

		for (i = 0; i < n; i += 16)
		{
			sprintf((pbuf+strlen(pbuf)),"B%d:%.3x  ", (dir >> 6) & 0x01, i);
			for (j = 0; j < 16; j++)
				if (i + j < n)
					sprintf((pbuf+strlen(pbuf)),"%02x ", buf[i + j]);
				else
					sprintf((pbuf+strlen(pbuf)),"   ");
			sprintf((pbuf+strlen(pbuf)),"      ");
			for (j = 0; j < 16 && i + j < n; j++)
				if (isprint(buf[i + j]))
					sprintf((pbuf+strlen(pbuf)),"%c", buf[i + j]);
				else
					sprintf((pbuf+strlen(pbuf)),".");
			sprintf((pbuf+strlen(pbuf)),"\n");
		}
	}
	
	if(header && ((l2buf[0] != '\0') || (l3buf[0] != 0)))
	{
		char *p;
		p = fmt_hdr(nsave);
		printf("%s", p);
		if(outflag)
			fprintf(Fout, "%s", p);
	}

	if(l2buf[0] != '\0')
	{	
		printf("%s", l2buf);
		if(outflag)
			fprintf(Fout, "%s", l2buf);
	}

	if(l3buf[0] != '\0')
	{
		printf("%s", l3buf);
		if(outflag)
			fprintf(Fout, "%s", l3buf);
	}
}

/*---------------------------------------------------------------------------*
 *	signal (=abort) handler
 *---------------------------------------------------------------------------*/
void
catchsig()
{
	switch_driver(device, 0);
	printf("EXIT\n");
	exit(1);
}

/*---------------------------------------------------------------------------*
 *	switch driver debugging output on/off
 *---------------------------------------------------------------------------*/
static void
switch_driver(char *device, int value)
{
	int fd, v;

	if((fd = open(device, O_RDWR)) < 0)
	{
		char buffer[80];

		sprintf(buffer, "Error opening trace enable device [%s]", device);
		perror(buffer);
		exit(1);
	}

	v = value;

	if(ioctl(fd, ISDN_SPY, &v) < 0)
	{
		char buffer[80];

		sprintf(buffer, "Error ioctl ISDN_SPY: val=%d, dev=%s", v, device);
		perror(buffer);
		exit(1);
	}
	close(fd);
}

/*---------------------------------------------------------------------------*
 *	usage intructions
 *---------------------------------------------------------------------------*/
void
usage(void)
{
	fprintf(stderr,"\n");
	fprintf(stderr,"bisdntrc - display ISDN D- and/or B-channel traces\n");
	fprintf(stderr,"usage: bisdntrc -d <dev> -h -i <val> -l -n <val> -o <file> -r\n");
	fprintf(stderr,"       -d <device>   device to enable traceing, default %s\n", ISPYDEV);
	fprintf(stderr,"       -h            don't print the header for each message\n");
	fprintf(stderr,"       -i <value>    1=D, 2=B, 3=D+B-channel display\n");
	fprintf(stderr,"       -l            decode low layer Q.921 messages\n");
	fprintf(stderr,"       -n <value>    only print header if packet has more than n octetts\n");
	fprintf(stderr,"       -o <filename> write output to file filename\n");
	fprintf(stderr,"       -r            print raw hex/ASCII dump of protocol\n");
	fprintf(stderr,"\n");
	exit(1);
}

/* EOF */
