/*
 * Copyright (c) 1996 Gary Jennejohn.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *---------------------------------------------------------------------------*
 *
 *	bisdntrc_lapd - print Q.921 traces
 *	----------------------------------
 *
 *	last edit-date: [Fri Jul  5 12:19:13 1996]
 *
 *	-hm	splitting
 *	-hm	printing to buffer
 *
 *---------------------------------------------------------------------------*/

#include "bisdntrc.h"

/*---------------------------------------------------------------------------*
 *	decode LAPD (Q.921) protocol
 *---------------------------------------------------------------------------*/
int
decode_lapd(char *pbuf, int n, unsigned char *buf, int is_te, int raw, int printit)
{
	int sap, tei, cmd, p_f, ea;
	int cnt = 0;
	int i;

	*pbuf = '\0';
	
	if(raw && printit)
	{
		int j;
		for (i = 0; i < n; i += 16)
		{
			sprintf((pbuf+strlen(pbuf)),"Dump:%.3x  ", i);
			for (j = 0; j < 16; j++)
				if (i + j < n)
					sprintf((pbuf+strlen(pbuf)),"%02x ", buf[i + j]);
				else
					sprintf((pbuf+strlen(pbuf)),"   ");
			sprintf((pbuf+strlen(pbuf)),"      ");
			for (j = 0; j < 16 && i + j < n; j++)
				if (isprint(buf[i + j]))
					sprintf((pbuf+strlen(pbuf)),"%c", buf[i + j]);
				else
					sprintf((pbuf+strlen(pbuf)),".");
			sprintf((pbuf+strlen(pbuf)),"\n");
		}
	}

	sap = (buf[0] >> 2) & 0x3f;
	cnt++;
	tei = buf[1] >> 1;
	cnt++;
	if (is_te)
		cmd = !(buf[0] & 2);
	else
		cmd = buf[0] & 2;

	if(printit)
		sprintf((pbuf+strlen(pbuf)), "LAPD: SAP = %d %c TEI = %d ", sap, cmd?'C':'R', tei);

	switch (sap)
	{
		/* SAPI control procedures */
		case 0:
		{
			if(printit)
				 sprintf((pbuf+strlen(pbuf)), "Control Proc ");
			if (!(buf [2] & 1))
			{
				if(printit)
					sprintf((pbuf+strlen(pbuf)), "I ");
				p_f = buf [3] & 1;
				if(printit)
					sprintf((pbuf+strlen(pbuf)), "N(S) %d N(R) %d P %d ", buf [2] >> 1, buf [3] >> 1, p_f);
				cnt += 2;
			}
			if ((buf [2] & 3) == 1)
			{
				if(printit)
					sprintf((pbuf+strlen(pbuf)), "S ");
				p_f = buf [3] & 1;
				cmd = buf [2] & 0x0c;
				if(printit)
				{
					if (cmd == 0)
						sprintf((pbuf+strlen(pbuf)), "RR N(R) %d PF %d ", buf [3] >> 1, p_f);
					if (cmd == 4)
						sprintf((pbuf+strlen(pbuf)), "RNR N(R) %d PF %d ", buf [3] >> 1, p_f);
					if (cmd == 8)
						sprintf((pbuf+strlen(pbuf)), "REJ N(R) %d PF %d ", buf [3] >> 1, p_f);
				}
				cnt += 2;
		  	}
		  	if ((buf [2] & 3) == 3)
			{
		  		if(printit)
					sprintf((pbuf+strlen(pbuf)), "U ");
				p_f = (buf [2] & 0x10) >> 4;
				cmd = buf [2] & 0xec;
				if(printit)
				{
					if (cmd == 0x6c)
						sprintf((pbuf+strlen(pbuf)), "SABME PF %d ", p_f);
			  		if (cmd == 0x0c)
						sprintf((pbuf+strlen(pbuf)), "DM PF %d ", p_f);
			  		if (cmd == 0)
						sprintf((pbuf+strlen(pbuf)), "UI PF %d ", p_f);
			  		if (cmd == 0x40)
						sprintf((pbuf+strlen(pbuf)), "DISC PF %d ", p_f);
			  		if (cmd == 0x60)
						sprintf((pbuf+strlen(pbuf)), "UA PF %d ", p_f);
			  		if (cmd == 0x84)
						sprintf((pbuf+strlen(pbuf)), "FRMR PF %d ", p_f);
			  		if (cmd == 0xac)
						sprintf((pbuf+strlen(pbuf)), "XID PF %d ", p_f);
			  		/* information field ??? */
			  	}
		  		cnt++;
		  	}
			break;
		}
		case 16:
			if(printit)
				sprintf((pbuf+strlen(pbuf)), "X.25 !?!\n");
			return (n);
		/* SAPI layer 2 management functions */
		case 63:
		{
			if(printit)
				sprintf((pbuf+strlen(pbuf)), "Management Func ");
			if (tei != 127)
			{
				if(printit)
					sprintf((pbuf+strlen(pbuf)), "ILLEGAL TEI\n");
				return (n);
			}
			if (buf [2] != 3 && buf [3] != 0xf)
			{
				if(printit)
					sprintf((pbuf+strlen(pbuf)), "invalid format!\n");
				return (n);
			}
			cnt+= 2;
			if(printit)
				sprintf((pbuf+strlen(pbuf)), "Ri 0x%x ", *(short *)&buf [4]);
			cnt += 2;
			switch (buf[6])
			{
				case 1: 
					if(printit)
						sprintf((pbuf+strlen(pbuf)), "IdReq Ai 0x%x ", (buf [7] >> 1));
					cnt += 3;
					break;
		  		case 2:
					if(printit)
						sprintf((pbuf+strlen(pbuf)), "IdAss Ai 0x%x ", (buf [7] >> 1));
					cnt += 3;
					break;
		  		case 3:
					if(printit)
						sprintf((pbuf+strlen(pbuf)), "IdDen Ai 0x%x ", (buf [7] >> 1));
					cnt += 3;
					break;
		  		case 4:
					if(printit)
						sprintf((pbuf+strlen(pbuf)), "IdChkReq ");
					cnt += 2;
					for (i = 7; i < n; i++)
					{
						ea = buf [i] & 1;
						if(printit)
							sprintf((pbuf+strlen(pbuf)), "Ai 0x%x ", (buf [i] >> 1));
						cnt++;
						if (ea)
						{
							if(printit)
								sprintf((pbuf+strlen(pbuf)), "\n");
							break;
					 	}
					}
					break;
		  		case 5:
					if(printit)
						sprintf((pbuf+strlen(pbuf)), "IdChkRes Ai 0x%x ", (buf [7] >> 1));
					cnt += 3;
					break;
		  		case 6:
					if(printit)
						sprintf((pbuf+strlen(pbuf)), "IdRem Ai 0x%x ", (buf [7] >> 1));
					cnt += 3;
					break;
		  		case 7:
					if(printit)
						sprintf((pbuf+strlen(pbuf)), "IdVer Ai 0x%x ", (buf [7] >> 1));
					cnt += 3;
					break;
		  		default:
					if(printit)
						sprintf((pbuf+strlen(pbuf)), "Unknown Msg Type\n");
					return (n);
	  		}
			break;
		}
		default:
			if(printit)
				sprintf((pbuf+strlen(pbuf)), "ILLEGAL SAP\n");
			return (n);
	}

	if(printit)
		sprintf((pbuf+strlen(pbuf)), "\n");
	return (cnt);
}

/* EOF */
