/*
 * Copyright (c) 1995 Dietmar Friede.  All rights reserved.
 *
 * Copyright (c) 1995, 1996 Hellmuth Michaelis. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Dietmar Friede
 *	and Hellmuth Michaelis.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *---------------------------------------------------------------------------*
 *
 *	bisdn ioctl header file
 *	-----------------------
 *
 *	last edit-date: [Thu May  9 16:46:01 1996]
 *
 *	-hm	added ISDN_GET and ISDN_DONTACCEPT ioctl for b_disdn.c
 *	-hm	moved protocol types from b_disdn.c to here
 *	-hm	isdn_ipacct report types
 *	-hm	ISDN_STIMEOUT ioctl
 *	-hm	ISDN_DISDNRESET ioctl
 *	-hm	ISDN Information Element Identifiers moved here
 *	-hm	moving protocol info to bisdn_isdn.h
 *
 *---------------------------------------------------------------------------*/

/* ISDN protocol identifiers */

#define ISDNUNKN	0	/* unknown or invalid ISDN type		*/
#define ISDNDSS1	1	/* DSS1 protocol			*/
#define ISDN1TR6	2	/* 1TR6 protocol			*/

/* type of accounting messages for MSG_IPACCT */

#define IPIACCT_MID	0	/* accounting message while connected	*/
#define IPIACCT_FIN	1	/* accounting message at end of call	*/

/* low layer protocols */


/* maximum length of a telephone number (+ 1 for '\0') */
#define TELNO_MAX	33	/* max length of a telephone number (+ '\0') */
#define INFO_MAX	55	/* maximum length of an info message body    */
#define MSG_MAX		240	/* maximum length of a msg message body      */


#define DEBUG_INFO_LEN	10000		/* length of debug info buffer	*/

#define ISBUSY(x)	(((x) & 0x80) == 0)
#define ISFREE(x)	(((x) & 0x80) == 0x80)

#define N_ISDN_CTRL	2		/* max no of controllers supported */

/*---------------------------------------------------------------------------*
 *	stuff related to sending messages from the kernel to the daemon
 *---------------------------------------------------------------------------*/
 
#pragma pack(1)

typedef struct {
	unsigned char  message_type;	/* message identifier		*/
#define	MSG_ACCEPT	'a'
#define MSG_DIAL	'd'
#define MSG_INFO	'i'
#define MSG_IPACCT	'A'
#define MSG_CONNECT	'C'
#define MSG_DISCONN	'D'
#define MSG_MESSAGE	'M'

	unsigned char  appl_type;	/* application type		*/
#define ATYP_IP	 0			/* ip 				*/
#define ATYP_TTY 1			/* tty/login 			*/
#define ATYP_TEL 2			/* telephony			*/

	unsigned char  appl_no;		/* numeric application id	*/
	unsigned char  channel;		/* channel/controller number	*/

	unsigned short service;		/* type of service used		*/
#define NO_B_PROTOCOL	0x00
#define TELEFON		0x01
#define RAW_ISDN	0x02
#define CEPT		0x03

#define UIFRAMES	0x10
#define X75_LAP		0x20

#define T70		0x100
#define ISO		0x200
#define T90		0x300
} message_header_t;

typedef struct {
	message_header_t header;
	unsigned short   alert_id;
	unsigned char    dst_telno[TELNO_MAX];
	unsigned char    src_telno[TELNO_MAX];
} message_accept_t;

typedef struct {
	message_header_t header;
} message_dial_t;

typedef struct {
	message_header_t header;
	unsigned char    protocol;
	int              length;
	unsigned char    info[INFO_MAX];
} message_info_t;

typedef struct {
	message_header_t header;	
	unsigned char    flag;
	unsigned int     input_bytes;
	unsigned int     output_bytes;
	unsigned int     compr_input_bytes;
	unsigned int     compr_output_bytes;
	unsigned int     input_packets;
	unsigned int     output_packets;
} message_ipacct_t;

typedef struct {
	message_header_t header;	
	unsigned char    active_flag;
} message_connect_t;

typedef struct {
	message_header_t header;	
	unsigned char    localflag;
#define DISC_LOCAL  1
#define DISC_REMOTE 0
	unsigned char    protocol;
	unsigned char    cause;
} message_disconnect_t;

typedef struct {
	message_header_t header;
	int              length;
	unsigned char    msg[MSG_MAX];
} message_msg_t;

#pragma pack()

/*---------------------------------------------------------------------------*
 *	stuff related to sending ioctls from the daemon to the kernel
 *---------------------------------------------------------------------------*/

typedef struct
{
	short	atyp;			/* application type		*/
	short	appl;			/* application id		*/
	short	ctrl;			/* controller number		*/
	time_t	timeout;		/* timeout until idle hangup	*/
	time_t	idle_time;		/* idle time			*/
	u_short service;		/* service profile		*/
	u_short blocksize;		/* isdn max packet size		*/
	char	dst_telno[TELNO_MAX];	/* destination number		*/
	char	src_telno[TELNO_MAX];	/* source number		*/
} dial_t;

typedef struct
{
	short	atyp;			/* application type		*/
	short	appl;			/* application id		*/
	short	ctrl;			/* controller number		*/
	short	alert;			/* if alert, number of alerts	*/
	time_t	timeout;		/* timeout until idle hangup	*/
	time_t	idle_time;		/* idle time			*/
	u_short	service;		/* service profile		*/
	u_short	blocksize;		/* isdn max packet size		*/
} accept_t;

typedef struct
{
	short	atyp;			/* application type		*/
	short	appl;			/* application id		*/
	u_short len;			/* length of data		*/
	u_char	data[2048];		/* data buffer			*/
} appl_mesg_t;

typedef struct
{
	short	appl;			/* application id		*/
	time_t	timeout;		/* timeout until idle hangup	*/
} timo_t;

typedef struct
{
	unsigned char appl_type;	/* application type		*/
	unsigned char appl_no;		/* application number		*/
	unsigned char channel;		/* channel			*/
	unsigned char cause;		/* cause			*/
} hangup_t;

typedef u_char dbg_type[DEBUG_INFO_LEN];/* debug info buffer		*/

/* ioctls */

#define	ISDN_DIAL		_IOWR('I', 1, dial_t)
#define	ISDN_ACCEPT		_IOWR('I', 2, accept_t)
#define	ISDN_DONTACCEPT		_IOWR('I', 3, int)
#define	ISDN_HANGUP		_IOWR('I', 4, hangup_t)
#define	ISDN_DEBUG		_IOWR('I', 5, dbg_type)
#define	ISDN_RESET		_IOWR('I', 6, int)
#define	ISDN_SET		_IOWR('I', 7, int)
#define	ISDN_GET		_IOWR('I', 8, int)
#define	ISDN_SPY		_IOWR('I', 9, int)
#define	ISDN_APPL_MESG		_IOWR('I',10, appl_mesg_t)
#define	ISDN_STIMEOUT		_IOWR('I',11, timo_t)
#define	ISDN_DISDNRESET		_IOWR('I',12, int)

/* EOF */
