/*
 * Copyright (c) 1995 Dietmar Friede and Juergen Krause.  All rights reserved.
 *
 * Copyright (c) 1996 Arne Helme. All rights reserved.
 *
 * Copyright (c) 1996 Gary Jennejohn. All rights reserved. 
 *
 * Copyright (c) 1995, 1996 Hellmuth Michaelis. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Dietmar Friede,
 *	Juergen Krause, Arne Helme, Gary Jennejohn and Hellmuth Michaelis.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *---------------------------------------------------------------------------
 *
 * 	A "low intermediate" driver for Dumb ISDN-cards.
 * 	Main objective is serving the ISDN D channel protocols.
 * 	Low level driving is done by i386/isa/teles.c
 *
 *	last edit-date: [Sun Jul  7 18:03:08 1996]
 *
 *	-hm	enabled sending a disconnect on a only half connected
 *			(incoming) call to implement callback
 *	-hm	fixing a bug for DSS1 for incorrectly handling the
 *			(incoming) calling party message (0x6c)
 *	-hm	distinguishing between DSS1 and national ISDN cause
 *			values by stipping bit 8 for national ISDN
 *	-hm	removing several "not implemented" console messages
 *			for obviously unnecessary messages
 *	-hm	removing the Check Request TEI type console message
 *	-hm	debugging decoding of cause info element for DSS1
 *	-hm	debugging channel id and related stuff
 *	-hm	different disconnect cause handling for DSS1 and 1TR6
 *	-hm	handling of channelid == 0 in setup messages
 *	-hm	fix from Arne Helme to enable telephony analog phones
 *	-hm	protecting buffer search in disdn_setup_intr with splimp()
 *	-hm	adding ioctl ISDN_DONTACCEPT and disdn_dontaccept()
 *	-hm	new format for isdn_info(), added protocol type
 *	-hm	date & charging info now passed to iid for all protocols
 *	-hm	removed boguous type argument from isdn_info parameters
 *	-hm	new interface to bisdnd, rewrote ioctls/messages
 *	-hm	no telno_t anymore
 *	-hm	removed support for subaddressing
 *	-hm	debugging new numbering scheme
 *	-hm	replacing more numbers with #defines
 *	-hm	moredata / sending complete debugging
 *	-hm	ioctl ISDN_DISDNRESET
 *	-hm	converting all IEI codes to defines
 *	-hm	IEI definitions moved to bisdn_ioctl.h
 *	-hm	1TR6 connected address bug fixed
 *	-hm	message type bug fixed
 *	-hm	converting to #defined message types
 *	-hm	added some 1TR6 message type defines
 *	-hm	fixing -Wall compiler warning
 *	-hm	adding support for Arne's Teles driver
 *	-hm	local/remote disconnect indication
 *	-hm	adding SINGLEPARAM for disdn_putmsg
 *	-hm	sending SENDING COMPLETE in DSS1 outgoing SETUP msg
 *	-hm	fixing CALL PROCEEDING ignore bug
 *	-hm	integrating Gary's unified patches
 *	-hm	integrating Arnes NetBSD patches
 *	-hm	bugfix from Greg Lehey for delimiting tel-numbers properly
 *	-hm	removing NOTDEF'ed code, debugging levels
 *	-hm	fixing things from Joerg's code review
 *	-gj	remove redundancies between FreeBSD and NetBSD (includes and
 *			SC_UNIT) to improve readability
 *	-hm	adding a bugfix from Greg Lehey (invalid application)
 *
 *---------------------------------------------------------------------------*/

#include "disdn.h"
#if NDISDN > 0

#define DISDN_DEBUG	1	/* 0 = no status/debug messages at all      */
				/* 1 = basic messages (TEI, unusual states) */
				/* 2 = all "not implemented" messages       */
				/* 3 = everything printed                   */
#include <sys/param.h>
#include <sys/ioctl.h>
#include <sys/kernel.h>
#include <sys/systm.h>
#ifdef __NetBSD__
#include <sys/device.h>
#endif /* __NetBSD__ */
#include <sys/mbuf.h>

#include <machine/stdarg.h>

#if BSD > 199306 || defined (__NetBSD__)
#include <i386/include/bisdn_ioctl.h>
#include <i386/include/bisdn_isdn.h>
#else
#include <machine/bisdn_ioctl.h>
#include <machine/bisdn_isdn.h>
#endif

#include <bisdn/b_isdn.h>
#include <bisdn/b_isdnlap.h>
#include <bisdn/b_disdn.h> 

#ifdef __FreeBSD__
#define TIMEOUT_FUNC_T  timeout_func_t
#endif /* __FreeBSD__ */
#ifdef __NetBSD__
#define TIMEOUT_FUNC_T  void *
#define memcpy(x,y,z)   bcopy(y,x,z)
#endif /* __NetBSD__ */

#define TEITIMEOUT	1

#define INI_I		0
#define INI_UI		1
#define PARAM		2
#define SPARAM		3
#define ESCPARAM	4
#define SINGLEPARAM	5

/* layer 3 message parameters */

#define	NUMBER_TYPEPLAN	0x81	/* called/calling party octett 3	*/
#define CHANNELID_ANY	0x83	/* channel = any channel		*/
#define LOC_USER	0x80	/* cause location = user		*/
#define DEF_CAUSEQ931	0x9f	/* default cause (if no cause)		*/
#define DEF_CAUSE1TR6	0x00	/* default cause (if no cause)		*/

/* some layer 3 message lengths */

#define CALLREF_LEN	0x01	/* length of call reference		*/
#define CHANNELID_LEN	0x01	/* length of channel id			*/
#define BEARERCAP_LEN	0x0c	/* max length of bearer capabilities	*/

#define IS_RUNNING	1
#define IS_FIRST	2

/* number of message buffers in disdn_softc */

#define MAX_BUFS	4	/* 4 buffers rx, 4 buffers tx		*/
#define MAX_CHAN	2	/* 2 channels per line			*/

/* chan->state's */

typedef enum    
{               
	DISCONNECT, ISDISCON, DIAL, CALLED, CONNECT, IDLE, ACTIVE
} io_state;

#define min(a,b)        ((a)<(b)?(a):(b))
#define C_CHAN(x)       ((x)&1)

extern isdn_appl_t isdn_appl[];
extern u_short isdn_state;
extern isdn_ctrl_t isdn_ctrl[];
extern int ispy_applnr;
extern int Isdn_Appl, Isdn_Ctrl;

static int Disdn_cnt = 0;

static int disdn_connect __P((int, int, int, char *, char *));
static int disdn_disconnect __P((int, int));
static int disdn_accept __P((int, int, int, int, int));
static void disdn_output __P((int));
static void disdn_firstout __P((int cn));
static int disdn_ioctl __P((int, int, caddr_t, int));
static int disdn_putmsg __P((struct mbuf *, int, ...));
static int disdn_dequeue __P((struct hdlc *));

static int disdn_d_sendctl __P((struct hdlc *, int, int, int, int));
static int disdn_d_resend __P((struct hdlc *));
static int disdn_d_connect __P((struct hdlc *));
static int disdn_d_disconnect __P((struct hdlc *));

static int disdn_b_startout __P((int));
static int disdn_b_sendctl __P((struct hdlc *, int, int, int, int));
static int disdn_b_resend __P((struct hdlc *));
static int disdn_b_connect __P((struct hdlc *));
static int disdn_b_disconnect __P((struct hdlc *));

static int tei_get __P((int));
static void tei_checkresp __P((int));

typedef struct
{
	u_char		mind;			/* message indicator	*/
	u_char          ctrl;			/* controller		*/
	u_char		appl;			/* application id	*/
	u_char		chan;			/* incoming channel no	*/
	u_char		channelid;		/* channel id		*/
	u_char		isalert;		/* alerting flag	*/
	u_char		servvec[BEARERCAP_LEN+1]; /* bearer capabilities*/
	u_char		dst_telno[TELNO_MAX];	/* destination number	*/
	u_char		src_telno[TELNO_MAX];	/* source number	*/
	u_short		cause;			/* cause value		*/
	short           state;			/* usage state		*/
#define MESG_FREE 0
#define MESG_USED 1
	u_short		service;		/* service profile	*/
} mesg_t;

typedef struct
{
	char            ctrl;
	mesg_t		*msg;
	short           state;
	struct hdlc     hdlc;
	u_short		hdlc_nunack;
	struct isqueue	co_q;
	u_short		hdlc_state;
} chan_t;

typedef enum
{
	DISCON, GOTTEI, GOTD2CON
} stateD2_t;

struct disdn_softc
{
#ifdef __NetBSD__
	struct device		*sc_dev;
#endif /* __NetBSD__ */
	u_char			sc_unit;
	u_char			sc_ctrl;
	u_char			sc_tei;
	u_char			sc_isdn;
	u_char			sc_typ;
	struct hdlc		hdlc;
	u_short			sc_ri;
	u_short			sc_flags;
	stateD2_t		sc_state;
	mesg_t			sc_msgin[MAX_BUFS];
	mesg_t			sc_msgout[MAX_BUFS];
	struct disdn_ll_func	*sc_ll_func;
	struct isqueue		sc_o_q;
	struct isqueue		sc_co_q;
	chan_t			sc_chan[MAX_CHAN];
} disdn_sc[NDISDN];

#ifdef __FreeBSD__
#define SC_UNIT sc_unit
#endif
#ifdef __NetBSD__
#define SC_UNIT sc_dev
#endif

#ifdef __NetBSD__
/* Dummy attach routine for NetBSD */
void
disdnattach(int unused)
{
}
#endif /* __NetBSD__ */

int
disdn_attach(
#ifdef __FreeBSD__
int unit,
#endif /* __FreeBSD__ */
#ifdef __NetBSD__
struct device *dev,
#endif /* __NetBSD__ */
struct disdn_ll_func *ifs, int typ, int n)
{
	struct disdn_softc *sc;
	isdn_ctrl_t *ctrl0, *ctrl1;
	unsigned cn;
	int i;

	if(Disdn_cnt >= NDISDN)
	{
		return(-1);
	}

	if((cn = isdn_ctrl_attach(n)) == -1)
	{
		return(-1);
	}

	sc = &disdn_sc[Disdn_cnt];

	sc->sc_ctrl = cn;
        /* sc->sc_ctrl = -1; */	

#ifdef __FreeBSD__
	sc->sc_unit= unit;
#endif /* __FreeBSD__ */

#ifdef __NetBSD__
	sc->sc_dev = dev;
	sc->sc_unit = dev->dv_unit;
#endif /* __NetBSD__ */

	sc->sc_typ = typ;
	sc->sc_ri = 0xa5a5 + cn;
	ctrl0 = &isdn_ctrl[cn];
	ctrl1 = &isdn_ctrl[cn + 1];
	sc->sc_chan[0].ctrl = ctrl0->ctrl = cn;
	sc->sc_chan[1].ctrl = ctrl1->ctrl = cn + 1;
	sc->sc_isdn = ISDNDSS1;
	ctrl0->disconnect = ctrl1->disconnect = disdn_disconnect;
	ctrl0->accept = ctrl1->accept = disdn_accept;
	ctrl0->connect = ctrl1->connect = disdn_connect;
	ctrl0->output = ctrl1->output = disdn_output;
	ctrl0->ioctl = ctrl1->ioctl = disdn_ioctl;
	ctrl0->unit = ctrl1->unit = sc->sc_unit;
	ctrl0->appl = ctrl1->appl = -1;

	sc->sc_tei = 0xff;
	sc->hdlc.unit = sc->sc_unit;
	sc->hdlc.proto = MOD128HDLC;
	sc->hdlc.mmask = 127;
	sc->hdlc.toutval[0] = 0;
	sc->hdlc.toutval[1] = hz;
	sc->hdlc.toutval[2] = 10*hz;
	sc->hdlc.resend = disdn_d_resend;
	sc->hdlc.sendctl = disdn_d_sendctl;
	sc->hdlc.start_out = disdn_d_startout;
        sc->hdlc.connect = disdn_d_connect;
        sc->hdlc.disconnect = disdn_d_disconnect;
        sc->hdlc.dequeue = disdn_dequeue;

	for(i=0; i < MAX_CHAN; i++)
	{
		chan_t *chan = &sc->sc_chan[i];
		chan->hdlc_nunack = 1;
		chan->hdlc.unit = (sc->sc_unit<<1) | i;
		chan->hdlc.proto = MOD8HDLC;
		chan->hdlc.mmask = 7;
		chan->hdlc.state = LAP_DISCONNECTED;
		chan->hdlc.toutval[0] = 0;
		chan->hdlc.toutval[1] = hz;
		chan->hdlc.toutval[2] = 3*hz;
		chan->hdlc.resend = disdn_b_resend;
		chan->hdlc.sendctl = disdn_b_sendctl;
		chan->hdlc.start_out = disdn_b_startout;
		chan->hdlc.connect = disdn_b_connect;
		chan->hdlc.disconnect = disdn_b_disconnect;
		chan->hdlc.dequeue = disdn_dequeue;
	}

	sc->sc_ll_func = ifs;

	cn |= (Disdn_cnt << 8);
	Disdn_cnt++;
	return(cn);
}

static int
disdn_putmsg(struct mbuf *m, int cmd, ...)
{
	int i;
	u_char *p1;
	u_char l;
	va_list ap;
	u_char *p = &m->m_data[m->m_len];

	va_start(ap, cmd);

	switch(cmd)
	{
		case INI_I:
			p = m->m_data;

			for(i=0; i<4; i++,p++)
				*p = 0;

			for(i=0; i<4; i++,p++)
				*p = (u_char)va_arg(ap, int);

			m->m_len = 8;
			break;

		case INI_UI:
			p = m->m_data;

			for(i=0; i<2; i++,p++)
				*p = 0;

			*p = 3;
			p++;

			for(i=0; i<4; i++,p++)
				*p = (u_char)va_arg(ap, int);

			m->m_len = 7;
			break;

		case ESCPARAM:
			*p = 0x9e;
			p++;
			m->m_len++;

		case PARAM:
			*p = (u_char)va_arg(ap, int);
			p++;
			m->m_len++;

			*p = l = (u_char)va_arg(ap, int);

			p++;
			m->m_len++;

			if(l)
			{
				for(i=0; i<l; i++,p++)
					*p = (u_char)va_arg(ap, int);
				m->m_len += l;
			}	
			break;

		case SPARAM:
			*p = (u_char)va_arg(ap, int);

			p++;
			m->m_len++;

			*p = l = (u_char)va_arg(ap, int);

			p++;
			m->m_len++;
	
			p1 = va_arg(ap, u_char *);
	
			for(i=0; i<l; i++,p++,p1++)
				*p = *p1;
	
			m->m_len += l;
			break;

		case SINGLEPARAM:
			*p = (u_char)va_arg(ap, int);
			m->m_len++;
			break;
	}
	va_end(ap);

	return(m->m_len);
}

static int
disdn_connect(int cn, int ap, int service, char *d, char *s)
{
	int unit = isdn_ctrl[cn].unit;
	struct disdn_softc *sc = &disdn_sc[unit];
	mesg_t *msg = NULL;
	struct mbuf *m;
	int i, x, l;
	u_char tb[TELNO_MAX+8];

	if(!(sc->sc_flags & TEITIMEOUT))
	{
		tei_get(unit);
	}

	for(i=0; i < MAX_BUFS; i++)
	{
		if(sc->sc_msgout[i].state == MESG_FREE)
		{
			sc->sc_msgout[i].state = MESG_USED;
			msg = &sc->sc_msgout[i];
			msg->mind = i+1;
			break;
		}
	}
	
	l = strlen(d) + strlen(s) + 24;

	if((msg == NULL) || ((m = isdn_getbuf(l)) == NULL))
		return(EBUSY);

	msg->appl = ap;
	msg->ctrl = cn;
	msg->service = service;
	m->m_len = 0;

	if(sc->sc_isdn == ISDNDSS1)
	{
		disdn_putmsg(m, INI_I, PD_Q931, 1, msg->mind, SETUP);

		switch(service & 0xf)
		{
			case 1:
				msg->servvec[1] = 0x80;
				msg->servvec[2] = 0x90;
				break;
			case 2:
				msg->servvec[1] = 0x88;
				msg->servvec[2] = 0x90;
				break;
		}

		disdn_putmsg(m, PARAM, IEI_BEARERCAP, 2, msg->servvec[1], msg->servvec[2]);

		disdn_putmsg(m, PARAM, IEI_CHANNELID, CHANNELID_LEN, CHANNELID_ANY);

		tb[0] = NUMBER_TYPEPLAN;	/* type and plan */
		strncpy(&tb[1], s, strlen(s));	/* source telno */
		disdn_putmsg(m, SPARAM, IEI_CALLINGPN, strlen(s) + 1, &tb[0]);

		tb[0] = NUMBER_TYPEPLAN;	/* type and plan */
		strncpy(&tb[1], d, strlen(d));	/* destination telno */
		
		disdn_putmsg(m, SPARAM, IEI_CALLEDPN, strlen(d) + 1, &tb[0]);

		disdn_putmsg(m, SINGLEPARAM, SOIE_SENDC);
	}
	else	/* 1TR6 protocol */ 
	{		
		disdn_putmsg(m, INI_I, PD_N1, CALLREF_LEN, msg->mind, SETUP);

		disdn_putmsg(m, PARAM, IEI_CHANNELID, CHANNELID_LEN, CHANNELID_ANY);

		tb[0] = NUMBER_TYPEPLAN;	/* type and plan */
		strncpy(&tb[1], s, strlen(s));	/* source telno */
		
		disdn_putmsg(m, SPARAM, IEI_CALLINGPN, strlen(s) + 1, &tb[0]);

		tb[0] = NUMBER_TYPEPLAN;	/* type and plan */
		strncpy(&tb[1], d, strlen(d));	/* destination telno */
		
		disdn_putmsg(m, SPARAM, IEI_CALLEDPN, strlen(d) + 1, &tb[0]);
		
		switch(service&0xf)
		{
			case 1:
				msg->servvec[1] = 1;	/* Phone */
				msg->servvec[2] = 0;
				break;
			case 2:
				msg->servvec[1] = 7;	/* Datacom 64 kbit/s */
				msg->servvec[2] = 0;
				break;
		}
		/* Service Indicator */
		disdn_putmsg(m, ESCPARAM, IEI_SERVICEI, 2,
				msg->servvec[1], msg->servvec[2]);
	}

	x = splimp();
	IS_ENQUEUE(&sc->sc_o_q,m);
	disdn_d_startout(sc->sc_unit);
	splx(x);
	return(0);
}

static int
disdn_disconnect(int cn, int cause)
{
	int unit = isdn_ctrl[cn].unit;
	struct disdn_softc *sc = &disdn_sc[unit];
	chan_t *chan = &sc->sc_chan[C_CHAN(cn)];
	mesg_t *msg = chan->msg;
	int x;
	struct mbuf *m;

	if(msg == NULL)
		return(0);

	/* the following is needed to get a disconnect message sent	*/
	/* immediately after the receiving of a setup message in case	*/
	/* we want to reject a call and callback, in this case the	*/
	/* setup message must be answered with a disconnect message	*/
	/* although the connection has not yet been made. (-hm)		*/

	if( (chan->state <= ISDISCON)		&&
	    (isdn_ctrl[cn].islisten != 1)	&&
	    (msg->state == MESG_FREE) )
	{
		return(0);
	}

	if((m = isdn_getbuf(24)) == NULL)
		return(EBUSY);

	m->m_len = 0;
	chan->state = ISDISCON;

	if(sc->sc_isdn == ISDNDSS1)
	{
		disdn_putmsg(m, INI_I, PD_Q931, 1, msg->mind, DISCO);

		if(cause)
			disdn_putmsg(m, PARAM, IEI_CAUSE, 2, LOC_USER, cause); /* GJ */
		else
			disdn_putmsg(m, PARAM, IEI_CAUSE, 2, LOC_USER, DEF_CAUSEQ931); /* GJ */
	}
	else
	{
		disdn_putmsg(m, INI_I, PD_N1, 1, msg->mind, DISCO);

		if(cause)
			disdn_putmsg(m, PARAM, IEI_CAUSE, 1, cause);
		else
			disdn_putmsg(m, PARAM, IEI_CAUSE, 0);
	}

	x = splimp();
	IS_ENQUEUE(&sc->sc_o_q,m);
	disdn_d_startout(sc->sc_unit);
	splx(x);

	return(0);
}

static void
disdn_resend(int cn)
{
	isdn_ctrl_t *ctrl = &isdn_ctrl[cn];
	struct disdn_softc *sc = &disdn_sc[ctrl->unit];
	chan_t *chan = &sc->sc_chan[C_CHAN(cn)];
	mesg_t *msg = chan->msg;
	int x;

	x = splimp();
	if(msg->state == MESG_USED)
	{
		isdn_accept_ind(msg->appl, sc->sc_typ, msg->ctrl,
				msg->service, msg->isalert,
				msg->dst_telno, msg->src_telno);
	}
	splx(x);
}

static int
disdn_accept(int cn, int an, int service, int alert, int blen)
{
	isdn_ctrl_t *ctrl = &isdn_ctrl[cn];
	struct disdn_softc *sc = &disdn_sc[ctrl->unit];
	chan_t *chan = &sc->sc_chan[C_CHAN(cn)];
	isdn_appl_t *appl  = &isdn_appl[an];
	mesg_t *msg = chan->msg;
	int x;
	struct mbuf *m;
	struct timeval ltime;
	int cs;

	ctrl->service = service >> 4;

	/*
	 * At least under NetBSD I frequently crash here unless I
	 * expliclitly check for a NULL msg pointer before I try to
	 * dereference it.  This happens when two channels are active
	 * simultaneously.  Is it an indication of a problem
	 * elsewhere?  XXX Arne.
	 */

	if(!ctrl->islisten || !msg || (msg->state == MESG_FREE))
		return(0);

	if(ISFREE(ctrl->appl))
		return(ENODEV);

	msg->appl = an;
	msg->ctrl = cn;

	if(alert)
	{
		if(!msg->isalert)
		{
			if((m = isdn_getbuf(64)) == NULL)
				return(EBUSY);

			m->m_len = 0;

			x = splimp();
			if(sc->sc_isdn == ISDNDSS1)
			{
				disdn_putmsg(m, INI_I, PD_Q931, 1, msg->mind, ALERT);
			}
			else
			{
				u_char tb[TELNO_MAX + 8];

				disdn_putmsg(m, INI_I, PD_N1, 1, msg->mind, ALERT);

				tb[0] = NUMBER_TYPEPLAN;
				strncpy(&tb[1], msg->dst_telno, strlen(msg->dst_telno));
		
				disdn_putmsg(m, SPARAM, IEI_CONNADDR, strlen(msg->dst_telno) + 1, &tb[0]);
			}

			IS_ENQUEUE(&sc->sc_o_q,m);
			disdn_d_startout(sc->sc_unit);
			splx(x);
		}
		msg->isalert = alert;
		timeout((TIMEOUT_FUNC_T)disdn_resend, (caddr_t)cn, 2*hz);
		return(0);
	}
	msg->isalert = 0;

	if(ctrl->appl != 0x7f)
		return(ENODEV);

	ctrl->appl = an;

	cs = splclock();
	ltime = time;
	splx(cs);

	ctrl->lastact = ltime.tv_sec;

	appl->ctrl = cn;
	appl->state = 4;

	x = splimp();

	msg->service = service;

	if((m = isdn_getbuf(64)) == NULL)
		return(EBUSY);

	m->m_len = 0;

	if(sc->sc_isdn == ISDNDSS1)
	{
		disdn_putmsg(m, INI_I, PD_Q931, 1, msg->mind, CONN);
	}
	else
	{
		u_char tb[TELNO_MAX + 8];

		disdn_putmsg(m, INI_I, PD_N1, 1, msg->mind, CONN);

		tb[0] = NUMBER_TYPEPLAN;
		strncpy(&tb[1], msg->dst_telno, strlen(msg->dst_telno));
		
		disdn_putmsg(m, SPARAM, IEI_CONNADDR, strlen(msg->dst_telno) + 1, &tb[0]);

		if(msg->channelid)
			disdn_putmsg(m, PARAM, IEI_CHANNELID, 1, msg->channelid);

		switch(service&0xf)
		{
			case 1:
				msg->servvec[1] = 1;
				msg->servvec[2] = 0;
				break;
			case 2:
				msg->servvec[1] = 7;
				msg->servvec[2] = 0;
				break;
		}
		/* Service Indicator */
		disdn_putmsg(m, ESCPARAM, IEI_SERVICEI, 2,
				msg->servvec[1],msg->servvec[2]);
	}
	IS_ENQUEUE(&sc->sc_o_q,m);
	disdn_d_startout(sc->sc_unit);
	splx(x);
	return(0);
}

void
disdn_input(int cn, struct mbuf *m)
{
	isdn_ctrl_t *ctrl = &isdn_ctrl[cn];
	struct disdn_softc *sc = &disdn_sc[ctrl->unit];
	chan_t *chan = &sc->sc_chan[C_CHAN(cn)];

	if(chan->state < CONNECT)
	{
		m_freem(m);
		return;
	}

	switch(ctrl->service & 0xf)
	{
		case TELEFON:	/* UI Frames */
			if(m->m_data[1] != 3)
			{
				m_freem(m);
				return;
			}
			m->m_data += 2;
			m->m_len -= 2;
			break;

		case RAW_ISDN:	/* HDLC Frames */
			if(!lap_input(&chan->hdlc, /*m->m_data[0] & */ 2, m->m_data+1))
			{
				m_freem(m);
				return;
			}
			m->m_data += 2;
			m->m_len -= 2;

			if(ctrl->service & 0xf0)
			{
				m->m_data += 2;
				m->m_len -= 2;
			}
			break;
	}

	if(m->m_len <= 0)
	{
		m_freem(m);
		return;
	}

	if(isdn_minput(ctrl->appl, m, ctrl->islisten))
	{
		struct timeval ltime;
		int cs;

		cs = splclock();
		ltime = time;
		splx(cs);

		ctrl->lastact = ltime.tv_sec;
	}
}

void
disdn_startout(int unit, int c)
{
	struct disdn_softc *sc = &disdn_sc[unit];
	chan_t *chan = &sc->sc_chan[c];
	isdn_ctrl_t *ctrl = &isdn_ctrl[(int)chan->ctrl];
	struct disdn_ll_func *ifs = sc->sc_ll_func;
	struct mbuf *m;

	if(ctrl->state & IS_FIRST)
		return;

	m = NULL;

	if(ctrl->appl == -1)	/* XXX bugreport from Greg */
	{
#if DISDN_DEBUG > 0
		printf ("disdn_startout: invalid application 0x%x\n", ctrl->appl);
#endif
		return;
	}
	
	switch(ctrl->service & 0xf)
	{
	case NO_B_PROTOCOL:
		m = isdn_out(ctrl->appl);
		break;

	case TELEFON:	/* UI */
		if((m = isdn_out(ctrl->appl)) != NULL)
		{
			m->m_len +=2;
			m->m_data -= 2;
			m->m_data[0] = ctrl->islisten? 1 : 3;
			m->m_data[1] = 3;
		}
		break;

	case RAW_ISDN:	/* HDLC */
		IS_DEQUEUE(&chan->co_q, m, 12);

		if(m)
			break;

		if((chan->hdlc_state == GOTD2CON) &&
			(chan->hdlc.unack < chan->hdlc_nunack))
		{
			struct hdlc *hd = &chan->hdlc;

			if((m = isdn_out(ctrl->appl)) != NULL)
			{
				if(ctrl->service & 0xf0)
				{
					m->m_len += 2;
					m->m_data -= 2;
					m->m_data[0] = 1;
					m->m_data[1] = 0;
				}
				m->m_len += 2;
				m->m_data -= 2;
				m->m_data[0] = ctrl->islisten? 3 : 1;
				m->m_data[1] = hd->vs<<1;
				m->m_data[1] |= hd->vr<<5;
				hd->unack++;
				m->m_flags |= 0x10;
				IS_ENQUEUE(&hd->uack_q,m);
				hd->vs = (hd->vs + 1) & hd->mmask;
			}
		}
		break;

	default:
#if DISDN_DEBUG > 0
	        printf ("disdn_startout: invalid service 0x%x\n", ctrl->service);
#endif
	        break;
	}

	if(m)
	{
		struct timeval ltime;
		int cs;

		cs = splclock();
		ltime = time;
		splx(cs);

		ctrl->lastact = ltime.tv_sec;

		(*ifs->ll_out)(sc->SC_UNIT, c, m);
		return;
	}

	if(chan->state > IDLE)
		chan->state = IDLE;

	ctrl->state &= ~IS_RUNNING;
}

void
disdn_firstout(int cn)
{
	isdn_ctrl_t *ctrl = &isdn_ctrl[cn];
	struct disdn_softc *sc = &disdn_sc[ctrl->unit];
	chan_t *chan = &sc->sc_chan[C_CHAN(cn)];
	int x;

	if((ctrl->state & IS_RUNNING) || (chan->state < ACTIVE))
		return;

	x = splimp();
	ctrl->state = IS_RUNNING;
	disdn_startout(ctrl->unit, C_CHAN(cn));
	splx(x);
}

void
disdn_output(int cn)
{
	isdn_ctrl_t *ctrl = &isdn_ctrl[cn];
	struct disdn_softc *sc = &disdn_sc[ctrl->unit];
	chan_t *chan = &sc->sc_chan[C_CHAN(cn)];

	if((ctrl->state & IS_RUNNING) || (chan->state != IDLE))
		return;

	ctrl->state |= IS_RUNNING;
	chan->state = ACTIVE;
	disdn_startout(ctrl->unit, C_CHAN(cn));
}

void
disdn_release(int unit)
{
	struct disdn_softc *sc = &disdn_sc[unit];
	chan_t *chan = &sc->sc_chan[0];
	struct disdn_ll_func *ifs = sc->sc_ll_func;
	int i;
	
	for(i=0; i < MAX_CHAN; i++, chan++)
	{
		if(chan->state)
		{
			isdn_ctrl_t *ctrl = &isdn_ctrl[(int)chan->ctrl];
			mesg_t *msg = chan->msg;
			(*ifs->ll_init)(sc->SC_UNIT, i, -1, 0xff);

			if(msg)
			{
				if(ISBUSY(msg->appl))
				{
					isdn_disconn_ind(msg->appl,
						sc->sc_isdn, 0, DISC_LOCAL);
					isdn_appl[msg->appl].ctrl = -1;
					isdn_appl[msg->appl].state = 0;
				}
				msg->state = MESG_FREE;
				msg->ctrl = 0;
				msg->appl = -1;
				ctrl->appl = -1;
			}
			chan->state = DISCONNECT;
			ctrl->state = 0;
			chan->msg = NULL;
		}
	}

	sc->sc_state = DISCON;

	isdn_clean_q(&sc->sc_o_q, 1);
	isdn_clean_q(&sc->sc_co_q, 2);
	isdn_clean_q(&sc->hdlc.uack_q, 3);

	sc->hdlc.unack = 0;
}

static void
disdn_notimpl(int w, u_char *buf, int len)
{
#if DISDN_DEBUG > 1
	int i;
	printf("b_disdn.c: Not implemented at %d, len %d:", w, len);
	for(i = 0; i < len; i++)
		printf(" 0x%02x", buf[i]);
	printf("\n");
#endif
}

static void
disdn_get_3_par(int unit, mesg_t *msg, u_char *buf, int len)
{
	struct disdn_softc *sc = &disdn_sc[unit];
	int gr, ogr, ff;

	gr = ogr = ff = 0;

	while(len > 0)
	{
		switch(gr)
		{

		case CODESET_0:

			if(*buf & 0x80)
			{
				/* single octet information elements */
				
				switch(*buf & 0xf0)
				{
					case SOIE_SHIFT:	/* shift codeset */
						/* bit 4 = 0 is a locking shift */
	
						if((ff = (*buf & SHIFT_LOCK)) ? 2 : 0)
							ogr = gr; /* save old codeset */
	
						gr = *buf & 0x07; /* gr = new codeset */
						break;
	
					/* more data/sending complete */
					case SOIE_MDSC:
						break;
	
					/* congestion level, repeat indicator. */
					default:
						disdn_notimpl(100, buf, len);
						break;
				}

				len--;
				buf++;
				break;
			}

			switch(*buf)
			{
			case IEI_BEARERCAP:	/* bearer capability */

				memcpy(msg->servvec, &buf[1], min(10,buf[1]+1));

				switch(msg->servvec[1])
				{
				case 0x80:	/* speech */
				case 0x89:	/* restricted digital info */
				case 0x90:	/* 3.1KHz audio XXX Arne */
					msg->service = 1;
					break;
				case 0x88:	/* unrestricted digital info */
					msg->service = 2;
					break;
				default:	/* 3.1kHz audio/video etc. */
					msg->service = 0;
				}
				break;

			case IEI_CAUSE:		/* cause */
				if(sc->sc_isdn == ISDN1TR6)
				{	/* 1TR6 */
					if(buf[1] == 0)
						msg->cause = 0;
					else
	                                	msg->cause = buf[2];
				}
				else
				{	/* DSS1 */
					if(buf[2] & 0x80)
						msg->cause = buf[3];
					else
						msg->cause = buf[4];
				}
				break;

			case IEI_CHANNELID:	/* channel id */
				msg->channelid = buf[2];
				break;

			case IEI_CALLINGPN:	/* calling party no */

				if(sc->sc_isdn != ISDNDSS1)
				{
					memcpy(msg->src_telno, &buf[3], min(TELNO_MAX, buf[1]-1));
					msg->src_telno[min(TELNO_MAX, buf[1] - 1)] = '\0'; /* delimit */
				}
				else
				{
					if(buf[2] & 0x80) /* no presentation/screening indicator ? */
					{
						memcpy(msg->src_telno, &buf[3], min(TELNO_MAX, buf[1]-1));
						msg->src_telno[min(TELNO_MAX, buf[1] - 1)] = '\0'; /* delimit */
					}
					else
					{
						memcpy(msg->src_telno, &buf[4], min(TELNO_MAX, buf[1]-2));
						msg->src_telno[min(TELNO_MAX, buf[1] - 2)] = '\0'; /* delimit */
					}
				}
				break;

			case IEI_CALLEDPN:	/* called party number */
				memcpy(msg->dst_telno, &buf[3], min(TELNO_MAX, buf[1]-1));
				msg->dst_telno[min(TELNO_MAX, buf [1] - 1)] = '\0'; /* delimit */
				break;

			/* 1TR6 */
			case IEI_CONNADDR:	/* connected address	*/

			/* Q.931 */
			case IEI_CALLSTATE:	/* call state		*/
			case IEI_PROGRESSI:	/* progress indicator	*/
			case IEI_DISPLAY:	/* display		*/
			case IEI_DATETIME:	/* date/time		*/
			case IEI_REDIRNO:	/* redirecting number	*/
			case IEI_USERUSER:	/* user-user		*/

			/* Q.932 */
			case IEI_FACILITY:	/* facility		*/

			/* Q.95x */
			case IEI_CONCTDNO:	/* connected number	*/
				isdn_info(msg->appl, sc->sc_isdn,
						buf[1]+3, buf);
				break;

/*XXX*/			case IEI_NETSPCFAC:	/* network specific fac */
/*XXX*/			case IEI_LLCOMPAT:	/* low layer compat	*/
/*XXX*/			case IEI_HLCOMPAT:	/* high layer compat	*/
				break;

			default:
				disdn_notimpl(200,buf,buf[1]+2);
			}
			
			len -= buf[1] + 2;
			buf += buf[1] + 2;
			break;

		case CODESET_6:
			if(*buf & 0x80)
			{
				switch(*buf & 0xf0)
				{
				case SOIE_SHIFT:
					if((ff = (*buf & SHIFT_LOCK)) ? 2 : 0)
						ogr = gr;
					gr = *buf & 0x07;
					break;
				default:
					disdn_notimpl(300, buf, buf[1]+2);
					break;
				}
				len--;
				buf++;
				break;
			}
			switch(*buf)
			{
			case IEI_SERVICEI:	/* service indicator */
				memcpy(msg->servvec, &buf[1], min(10,buf[1]+1));

				switch(msg->servvec[1])
				{
				case 1:
				case 2:
					msg->service = 1;
					break;
				case 5:
				case 7:
				case 15:
					msg->service = 2;
					break;
				default:
					msg->service = 0;
				}
				break;

			case IEI_CHARGINFO:	/* charging info	*/
			case IEI_DATE:		/* date			*/
				isdn_info(msg->appl, sc->sc_isdn,
						buf[1]+3, buf);
				break;
				
			case IEI_FACSELECT:	/* facility select	*/
			case IEI_STATFAC:	/* status of facilities	*/
			case IEI_STATGTLN:	/* status des gerufenen */
			case IEI_ADDTXATTR:	/* additional tx attrib	*/
				if(sc->sc_isdn == ISDN1TR6)
					break;

			default:
				disdn_notimpl(400, buf, buf[1]+2);
			}
			len -= buf[1] + 2;
			buf += buf[1] + 2;
			break;
		default:
			disdn_notimpl(500, buf, len);
			len = 0;
		}
		if(ff)
		{
			if(ff == 1)
			{
				ff = 0;
				gr = ogr;
			}
			else
			{
				ff = 1;
			}
		}
	}
}

static void
disdn_setup_intr(int unit, u_char * buf, int len)
{
	struct disdn_softc *sc = &disdn_sc[unit];
	isdn_ctrl_t *ctrl;
	u_short mind = buf[2];	/* call reference */
	mesg_t *msg = NULL;
	u_short	ch;
	int i, x;
	struct timeval ltime;
	int cs;

	/* The following check is necessary in bilingual environements */

	if(sc->sc_isdn == ISDNDSS1)
	{
		if(*buf != PD_Q931)	/* Q.931 protocol discriminator */
		{
#if DISDN_DEBUG > 0
			printf("disdn_setup_intr: expected PD_Q931 instead of 0x%x\n", *buf);
#endif
			return;
		}
	}
	else
	{
		if(*buf != PD_N1)	/* 1TR6 N1 protocol discriminator */
		{
#if DISDN_DEBUG > 0
			printf("disdn_setup_intr: expected PD_N1 instead of 0x%x\n", *buf);
#endif
			return;
		}
	}

	/* search for a free buffer */

	x = splimp();	/* protect us */
	
	for(i=0; i < MAX_BUFS; i++)
	{
		msg = &sc->sc_msgin[i];

		/* buffer unused or buffer == same call reference value ? */

		if((msg->state == MESG_FREE) || (msg->mind == (mind | 0x80)))
		{
			msg->state = MESG_USED;		/* buffer in use */
			msg->mind = mind | 0x80;	/* call reference */
			break;
		}
		msg = NULL;
	}

	splx(x);
	
	if(msg == NULL)
	{
#if DISDN_DEBUG > 0
		printf("disdn_setup_intr: msg = NULL\n");
#endif
		return;
	}

	disdn_get_3_par(unit, msg, &buf[4], len - 4);

	if(!(sc->sc_flags & TEITIMEOUT))
	{
		tei_get(unit);
	}

	ch = (msg->channelid) & 0x03;

	switch(ch)
	{
		case 0:
#if DISDN_DEBUG > 0
			printf("disdn_setup_intr got channel id of 0 = no channel\n");
#endif			
			/* Im 1TR6 bedeutet dieser Zustand offensichtlich, */
			/* dass beide B-Kanaele belegt sind, und die VSt   */
			/* beim Anrufer "anklopft"                   (-hm) */

			msg->state = MESG_FREE;
			return;			
			break;

		case 1:
		case 2:
			ch--;
			break;

		case 3:
			/* cannot happen (-hm) in setup messages */
#if DISDN_DEBUG > 0
			printf("disdn_setup_intr got channel id of 3 = any channel\n");
#endif
			break;
	}

	msg->chan = ch;
	
	msg->ctrl = sc->sc_chan[msg->chan].ctrl;
	sc->sc_chan[msg->chan].msg = msg;
	ctrl = &isdn_ctrl[msg->ctrl];

	cs = splclock();
	ltime = time;
	splx(cs);

	ctrl->lastact = ltime.tv_sec;

	ctrl->islisten = 1;
	msg->isalert = 0;

	isdn_accept_ind(msg->appl, sc->sc_typ, msg->ctrl, msg->service, 0,
		msg->dst_telno, msg->src_telno);

	msg->appl = ctrl->appl = 0x7f;
}

static void
disdn_d_3_intr(int unit, int tei, u_char * buf, int len)
{
	struct disdn_softc *sc = &disdn_sc[unit];
	struct disdn_ll_func *ifs = sc->sc_ll_func;
	u_short mind = buf[2];
	mesg_t *msg = NULL;
	chan_t *chan;
	isdn_ctrl_t *ctrl;
	struct mbuf *m;
	u_char ch;
	struct timeval ltime;
	int cs;

	int from_release = 0;
	
	if(mind & 0x80)
	{
		msg = &sc->sc_msgout[(mind&3)-1];
	}
	else
	{
		int i;

		mind |= 0x80;

		for(i=0; i < MAX_BUFS; i++)
		{
			if((sc->sc_msgin[i].mind == mind) &&
					sc->sc_msgin[i].state == MESG_USED)
			{
				msg = &sc->sc_msgin[i];
				break;
			}
		}
		if(msg == NULL)
		{
			return;
		}
	}

	disdn_get_3_par(unit, msg, &buf[4], len - 4);

	chan = NULL;
	ctrl = NULL;

	if(msg->chan < MAX_CHAN)
	{
		chan = &sc->sc_chan[msg->chan];
		ctrl = &isdn_ctrl[(int)chan->ctrl];
	}

	switch(*buf)
	{
	case PD_N0:
		switch(buf[3])
		{
			case REG_IND_1TR6:	/* REG IND */
				return;
			default:
				disdn_notimpl(600, buf, len);
		}
		return;

	case PD_N1:
	case PD_Q931:
		switch(buf[3])
		{
		case ALERT:	/* ALERT */
			return;

		case CONN:	/* CONNECT */
			if(chan && ctrl)
			{
				isdn_conn_ind(msg->appl,chan->ctrl,1);
				ctrl->appl = msg->appl;
				isdn_appl[msg->appl].ctrl = chan->ctrl;
				(*ifs->ll_init)(sc->SC_UNIT,msg->chan,(msg->channelid & 3) - 1,msg->service);

				if(chan->state < IDLE)
				{
					chan->state = ACTIVE;
					ctrl->state = IS_FIRST;
					timeout((TIMEOUT_FUNC_T)disdn_firstout,(caddr_t)(int)chan->ctrl,hz/5);
				} 
			}
			return;

		case SETUP_ACK:	/* SETUP ACK */
		case CALL_PROC:	/* CALL PROCEEDING */

			ch = (msg->channelid) & 0x03;

			switch(ch)
			{
				case 0:
#if DISDN_DEBUG > 0
					printf("disdn_d_3_intr got channel id of 0 = no channel\n");
#endif
					break;
		
				case 1:
				case 2:
					ch--;
					break;
		
				case 3:
#if DISDN_DEBUG > 0
					printf("disdn_d_3_intr got channel id of 3 = any channel\n");
#endif
					break;
			}
		
			msg->chan = ch;
		
			msg->ctrl = sc->sc_chan[msg->chan].ctrl;
			sc->sc_chan[msg->chan].msg = msg;
			ctrl = &isdn_ctrl[msg->ctrl];

			cs = splclock();
			ltime = time;
			splx(cs);

			ctrl->lastact = ltime.tv_sec;
			ctrl->appl = 0x7f;
			ctrl->islisten = 0;
			return;

		case CONN_ACK:	/* CONN ACK */
			if(chan && ctrl)
			{
				isdn_conn_ind(msg->appl,chan->ctrl,0);
				ctrl->appl = msg->appl;
				isdn_appl[msg->appl].ctrl = chan->ctrl;
				(*ifs->ll_init)(sc->SC_UNIT,msg->chan,(msg->channelid & 3) - 1,msg->service);

				if(chan->state < IDLE)
				{
					chan->state = ACTIVE;
					ctrl->state = IS_FIRST;
					timeout((TIMEOUT_FUNC_T)disdn_firstout,(caddr_t)(int)chan->ctrl,hz/5);
				} 
			}
			return;

		case DISCO:	/* DISCONNECT */
			if((m = isdn_getbuf(64)) == NULL)
				return;
			m->m_len = 0;

			if(sc->sc_isdn == ISDNDSS1)
			{
				disdn_putmsg(m, INI_I, PD_Q931, 1, msg->mind, RELEASE);
				disdn_putmsg(m, PARAM, IEI_CAUSE, 2, LOC_USER, DEF_CAUSEQ931);
			}
			else
			{
				disdn_putmsg(m, INI_I, PD_N1, 1, msg->mind, RELEASE);
				disdn_putmsg(m, PARAM, IEI_CAUSE, 0);
			}
			IS_ENQUEUE(&sc->sc_o_q,m);
			disdn_d_startout(sc->sc_unit);
			chan->state = ISDISCON;
			return;

		case RELEASE:	/* RELEASE */
			if((m = isdn_getbuf(64)) == NULL)
				return;

			m->m_len = 0;

			if(sc->sc_isdn == ISDNDSS1)
			{
				disdn_putmsg(m, INI_I, PD_Q931, 1, msg->mind, RELEASE_COMPL);
			}
			else
			{
				disdn_putmsg(m, INI_I, PD_N1, 1, msg->mind, RELEASE_COMPL);
			}
			
			IS_ENQUEUE(&sc->sc_o_q,m);
			disdn_d_startout(sc->sc_unit);

			from_release = 1;	/* tell RELEASE_COMPL that */
						/* we are coming from here */
			
			/* fall through */

		case RELEASE_COMPL:	/* RELEASE COMPLETE */
			(*ifs->ll_init)(sc->SC_UNIT,msg->chan,(msg->channelid & 3) - 1,0xff);

			if(ISBUSY(msg->appl))
			{
				isdn_disconn_ind(msg->appl,
					sc->sc_isdn, msg->cause,
					from_release == 1 ? DISC_LOCAL : DISC_REMOTE);
				isdn_appl[msg->appl].ctrl = -1;
				isdn_appl[msg->appl].state = 0;
			}

			if(chan && ctrl)
			{
				ctrl->appl = -1;
				chan->state = DISCONNECT;
				ctrl->state = 0;
				chan->msg = NULL;
			}
			msg->state = MESG_FREE;
			msg->ctrl = 0;
			msg->appl = -1;
			return;

		case FACILITY:	/* FACILITY */
		case INFO_1TR6:	/* 1TR6 INFORMATION */
			return;
			
		default:
			disdn_notimpl(700, buf, len);
		}
		break;

	default:
		disdn_notimpl(800, buf, len);
	}
}

static void
disdn_badframe(int w, u_char *buf, int len)
{
#if DISDN_DEBUG > 0
	int i;
	printf("b_disdn.c: Bad Frame at %d len %d", w, len);
	for(i = 0; i < len; i++)
		printf(" %x", buf[i]);
	printf("\n");
#endif
}

void
disdn_d_2_intr(int unit, struct mbuf *m)
{
	u_char my_tei;
	u_char *b = m->m_data;
	u_char tei = b[1] >> 1;
	u_short ri;
	struct disdn_softc *sc = &disdn_sc[unit];

	switch((b[0] >> 2) & 0x3f)	/* SAPI */
	{

	/* SAPI call control procedures */

	case 0:		
		if(tei == 127)		/* broadcast TEI ? */
		{
			if((m->m_len > 6) && (b[2] == 3) /* UI */ && (b[6] == 5)) /* SETUP */
				disdn_setup_intr(unit, &b[3], m->m_len - 3);
			m_freem(m);
			return;
		}
		if(sc->sc_flags & TEITIMEOUT)
			my_tei = sc->sc_tei >> 1;
		else
			my_tei = tei_get(unit) >> 1;

		if(tei == my_tei)
		{
			if(lap_input(&sc->hdlc, b[0] & 2, &b[2]))
			{
				disdn_d_3_intr(unit, tei, &b[4], m->m_len - 4);
				m_freem(m);
				return;
			}
		}
		m_freem(m);
		return;

	/* SAPI layer 2 management functions */
	
	case 63:
		if((b[2] != 3) && (b[3] != 0xf))	/* UI && management ent */
		{
			disdn_badframe(100, b, m->m_len);
			m_freem(m);
			return;
		}

		if(sc->sc_flags & TEITIMEOUT)
		{
			sc->sc_flags &= ~(TEITIMEOUT);
			untimeout((TIMEOUT_FUNC_T) tei_get, (caddr_t) unit);
		}
		
		ri = *(u_short *) & b[4];

		switch(b[6])	/* Message Type */
		{

		case 2:	/* Assigned */
			if(ri == sc->sc_ri)
			{
				sc->sc_tei = b[7] | 1;
				sc->sc_state = GOTTEI;
#if DISDN_DEBUG > 0
				printf("Assigned: TEI = 0x%x = %d\n", sc->sc_tei, sc->sc_tei >> 1);
#endif
				lap_estlink(&sc->hdlc,11);
				lapstate(&sc->hdlc, LAP_SETUP);
				break;
			}
			break;

		case 3:	/* Denied */
			if(ri == sc->sc_ri)
			{
#if DISDN_DEBUG > 0
				printf("Denied: TEI = 0x%x = %d\n", sc->sc_tei, sc->sc_tei >> 1);
#endif
				sc->sc_tei = 0xff;
				sc->sc_state = DISCON;
				sc->hdlc.state = LAP_DISCONNECTED;
				sc->sc_flags |= TEITIMEOUT;
				timeout((TIMEOUT_FUNC_T) tei_get, (caddr_t) unit, 2*hz);
			}
			break;

		case 4:	/* Check Request */
			if(sc->sc_tei != 0xff)
			{
				if(b[7] == 0xff)
				{
					tei_checkresp(unit);
					break;
				}
				if(b[7] == sc->sc_tei)
				{
					tei_checkresp(unit);
					if(sc->sc_state < GOTTEI)
					{
						sc->sc_state = GOTTEI;
#if DISDN_DEBUG > 0
						printf("Check Request: TEI = 0x%x = %d\n", sc->sc_tei, sc->sc_tei >> 1);
#endif
						lap_estlink(&sc->hdlc,110);
						lapstate(&sc->hdlc, LAP_SETUP);
					}
				}
			}
			break;

		case 6:	/* Remove */
			if((b[7] == 0xff) || (b[7] == sc->sc_tei))
			{
#if DISDN_DEBUG > 0
				printf("Remove: TEI = 0x%x = %d\n", sc->sc_tei, sc->sc_tei >> 1);
#endif
				sc->sc_tei = 0xff;
				sc->sc_state = DISCON;
				sc->hdlc.state = LAP_DISCONNECTED;
				sc->sc_flags |= TEITIMEOUT;
				timeout((TIMEOUT_FUNC_T) tei_get, (caddr_t) unit, 10);
			}
			break;

		default:
			disdn_badframe(200, b, m->m_len);
			m_freem(m);
			return;
		}
		break;

	default:
		disdn_badframe(300, b, m->m_len);
	}
	m_freem(m);
}

static void
disdn_dontaccept(int cn)
{
	isdn_ctrl_t *ctrl = &isdn_ctrl[cn];
	struct disdn_softc *sc = &disdn_sc[ctrl->unit];
	chan_t *chan = &sc->sc_chan[C_CHAN(cn)];
	mesg_t *msg = chan->msg;

	msg->state = MESG_FREE;
}

int
disdn_ioctl(int dev, int cmd, caddr_t data, int flag)
{
	isdn_ctrl_t *ctrl = &isdn_ctrl[dev&0xf];
	struct disdn_softc *sc = &disdn_sc[ctrl->unit];
	struct disdn_ll_func *ifs = sc->sc_ll_func;
	int error;
	int x;
	int i;

	error = 0;
	
	switch(cmd)
	{
		case ISDN_SET:
			sc->sc_isdn = data[0];
			break;

		case ISDN_GET:
			data[0] = sc->sc_isdn;
			break;

		case ISDN_DEBUG:
			data[0] = sc->sc_typ;

			data[1] = (*ifs->ll_getstate)(sc->SC_UNIT);
			data[2] = sc->sc_chan[0].state;
			data[3] = sc->sc_chan[0].hdlc_state;
			data[4] = sc->sc_chan[1].state;
			data[5] = sc->sc_chan[1].hdlc_state;
			data[6] = sc->sc_state;
			data[7] = sc->hdlc.state;
			data[8] = sc->sc_flags;
			data[9] = ctrl->state;
			break;

		case ISDN_SPY:
			/* There are illegal states. So I use them to toggle */
			if(data[0] & 4)
			{
				if(!sc->sc_chan[0].state)
					(*ifs->ll_init)(sc->SC_UNIT,0,-1,7);
				if(!sc->sc_chan[1].state)
					(*ifs->ll_init)(sc->SC_UNIT,1,-1,7);
                	}
			(*ifs->ll_setstate)(sc->SC_UNIT,ISDN_SPY,data);
			return(0);

		case ISDN_RESET:
			x = splimp();
			disdn_release(ctrl->unit);
			splx(x);
			return(0);

		case ISDN_DONTACCEPT:
			x = splimp();
			disdn_dontaccept(data[0]);
			splx(x);
			return(0);

		case ISDN_DISDNRESET:
			x = splimp();

			for(i=0; i < MAX_BUFS; i++)
			{
				sc->sc_msgout[i].state = MESG_FREE;
				sc->sc_msgin[i].state = MESG_FREE;
			}
			splx(x);

			return(0);

		default:
			error = ENODEV;
	}
	return(error);
}

static int
disdn_b_startout(int unit)
{
	struct disdn_softc *sc = &disdn_sc[unit>>1];
	chan_t *chan = &sc->sc_chan[unit&1];
	isdn_ctrl_t *ctrl = &isdn_ctrl[(int)chan->ctrl];

	if((ctrl->state & IS_RUNNING) || (chan->state != IDLE))
		return(0);

	ctrl->state |= IS_RUNNING;
	chan->state = ACTIVE;
	disdn_startout(unit>>1, unit & 1);
	return(1);
}

static int
disdn_b_resend(struct hdlc *hd)
{
	struct disdn_softc *sc = &disdn_sc[hd->unit>>1];
	chan_t *chan = &sc->sc_chan[hd->unit&1];
	struct mbuf *m = hd->uack_q.isq_head;
	struct disdn_ll_func *ifs = sc->sc_ll_func;
	isdn_ctrl_t *ctrl = &isdn_ctrl[(int)chan->ctrl];
	struct timeval ltime;
	int cs;
	
	if(!m || (chan->hdlc_state < GOTD2CON) || (ctrl->state & IS_RUNNING))
		return(0);

	cs = splclock();
	ltime = time;
	splx(cs);

	ctrl->lastact = ltime.tv_sec;

#ifdef __FreeBSD__
	return((*ifs->ll_out)(hd->unit>>1, hd->unit & 1, m));
#endif /* __FreeBSD__ */
#ifdef __NetBSD__
	return((*ifs->ll_out)(sc->sc_dev, hd->unit & 1, m));
#endif /* __NetBSD__ */
	
}

static int
disdn_b_sendctl(struct hdlc *hd, int cmdrsp, int cmd, int pf, int nr)
{
	chan_t *chan = &disdn_sc[hd->unit>>1].sc_chan[hd->unit&1];
	struct mbuf *m;

	if((m = isdn_getbuf(4)) == NULL)
		return(EBUSY);

	m->m_len = 2;

	if(cmdrsp)
		m->m_data[0] = 3;
	else
		m->m_data[0] = 1;

	m->m_data[1] = cmd;

	if(pf)
		m->m_data[1] |= 0x10;

	switch(cmd)
	{
		case DISC:
		case SABM:
		case DM:
		case UA:
			break;

		case RR:
		case RNR:
		case REJ:
			m->m_data[1] |= nr<<5;
			break;

		default:
#if DISDN_DEBUG > 1
			printf("b_disdn.c: not implemented-A LAP 0x%x\n",cmd);
#endif
			break;
	}

	IS_ENQUEUE(&chan->co_q, m);
	disdn_b_startout(hd->unit);
	return(0);
}

static int
disdn_b_connect(struct hdlc *hd)
{
	chan_t *chan = &disdn_sc[hd->unit>>1].sc_chan[hd->unit&1];

	chan->hdlc_state = GOTD2CON;
	return(0);
}

static int
disdn_b_disconnect(struct hdlc *hd)
{
	chan_t *chan = &disdn_sc[hd->unit>>1].sc_chan[hd->unit&1];

	chan->hdlc_state = DISCON;
	return(0);
}

static int
disdn_dequeue(struct hdlc *hd)
{
	struct mbuf *m;

	IS_DEQUEUE(&hd->uack_q, m, 13);

	if(!m)
		return(0);

	m_freem(m);
	return(1);
}

int
disdn_d_startout(int unit)
{
	struct disdn_softc *sc = &disdn_sc[unit];
	struct disdn_ll_func *ifs = sc->sc_ll_func;
	struct mbuf *m;
	int stat;

	if(IS_QEMPTY(&sc->sc_co_q) && IS_QEMPTY(&sc->sc_o_q))
	{
		return(0);
	}

	if((stat = (*ifs->ll_d_stat)(sc->SC_UNIT)))
        {
#if DISDN_DEBUG > 2
		if (stat_debug)
			printf("startout stat\n");
#endif
		return(0);
	}

	IS_DEQUEUE(&sc->sc_co_q, m, 14);

	if(m)
	{
		if(m->m_data[1] == 0)
		{
			if(sc->sc_state < GOTTEI)
			{
				m_freem(m);
				return(0);
			}
			m->m_data[1] = sc->sc_tei;
		}

		(*ifs->ll_d_out)(sc->SC_UNIT, m);
		m_freem(m);
		return(0);
	}

	if(sc->sc_state < GOTD2CON)
	{
		return(0);
	}

	if(sc->hdlc.unack == 0)
	{
		struct hdlc *hd = &sc->hdlc;

		IS_DEQUEUE(&sc->sc_o_q,m,15);

		if(m)
		{
			m->m_data[0] = 0;
			m->m_data[1] = sc->sc_tei;
			m->m_data[2] = hd->vs<<1;
			m->m_data[3] = hd->vr<<1;

			if((*ifs->ll_d_out)(sc->SC_UNIT, m) > 0)
			{
				hd->unack++;
				IS_ENQUEUE(&hd->uack_q, m);
				hd->vs = (hd->vs + 1) & hd->mmask;
				return(0);
			}
			IS_PREPEND(&sc->sc_o_q, m);
		}
	}
	return(0);
}

static int
disdn_d_resend(struct hdlc *hd)
{
	struct disdn_softc *sc = &disdn_sc[hd->unit];
	struct disdn_ll_func *ifs = sc->sc_ll_func;
	struct mbuf *m = hd->uack_q.isq_head;
	
	if(!m || (sc->sc_state < GOTD2CON))
		return(0);

	if ((*ifs->ll_d_stat)(sc->SC_UNIT))
	{
#if DISDN_DEBUG > 2
		if (stat_debug)
			printf("resend stat\n");
#endif
                return (-1);
	}
	return((*ifs->ll_d_out)(sc->SC_UNIT, m) >= 0);
}

static int
disdn_d_sendctl(struct hdlc *hd, int cmdrsp, int cmd, int pf, int nr)
{
	struct disdn_softc *sc = &disdn_sc[hd->unit];
	struct mbuf *m;

	if((m = isdn_getbuf(4)) == NULL)
		return(EBUSY);

	m->m_len = 3;

	if(cmdrsp)
		m->m_data[0] = 0;
	else
		m->m_data[0] = 2;

	m->m_data[1] = 0;
	m->m_data[2] = cmd;

	switch(cmd)
	{
		case DISC:
		case SABME:
		case DM:
		case UA:
			if(pf)
				m->m_data[2] |= 0x10;
			break;

		case RR:
		case RNR:
		case REJ:
			m->m_len = 4;
			m->m_data[3] = nr<<1;

			if(pf)
				m->m_data[3] |= 1;
			break;

		default:
#if DISDN_DEBUG > 1
			printf("b_disdn.c: not implemented-B LAP 0x%x\n",cmd);
			break;
#endif
	}

	IS_ENQUEUE(&sc->sc_co_q, m);
	disdn_d_startout(sc->sc_unit);
	return(0);
}

static int
disdn_d_connect(struct hdlc *hd)
{
	struct disdn_softc *sc = &disdn_sc[hd->unit];

	sc->sc_state = GOTD2CON;
	return(0);
}

static int
disdn_d_disconnect(struct hdlc *hd)
{
	struct disdn_softc *sc = &disdn_sc[hd->unit];

	if(sc->sc_state >= GOTTEI)
		sc->sc_state = GOTTEI;
	else
		sc->sc_state = DISCON;

	isdn_clean_q(&sc->sc_o_q, 6);
	return(0);
}

static int
tei_get(int unit)
{
	register struct disdn_softc *sc = &disdn_sc[unit];
	struct disdn_ll_func *ifs = sc->sc_ll_func;
	int x,i;
	struct mbuf *m;
	struct timeval ltime;
	int cs;

	x = splimp();
	sc->sc_flags &= ~TEITIMEOUT;

	if(sc->sc_state > DISCON)
	{
		if(sc->hdlc.state < LAP_SETUP)
		{
			lap_estlink(&sc->hdlc, 10);
			lapstate(&sc->hdlc, LAP_SETUP);
		}
		splx(x);
		return(sc->sc_tei);
	}

	if ((*ifs->ll_d_stat)(sc->SC_UNIT) == 2)
        {
#if DISDN_DEBUG > 2
		if (stat_debug)
			printf("tei_get stat\n");
#endif
		sc->sc_flags |= TEITIMEOUT;
		timeout((TIMEOUT_FUNC_T) tei_get, (caddr_t) unit, hz);
		splx(x);
		return(-1);
	}

	if((m = isdn_getbuf(8)) == NULL)
	{
		sc->sc_flags |= TEITIMEOUT;
		timeout((TIMEOUT_FUNC_T) tei_get, (caddr_t) unit, 10);
		splx(x);
		return(-1);
	}

	m->m_len = 8;
	m->m_data[0] = 0xfc;
	m->m_data[1] = 0xff;
	m->m_data[2] = 3;
	m->m_data[3] = 0xf;

	if(sc->sc_tei == 0xff)
	{
		cs = splclock();
		ltime = time;
		splx(cs);

		for(i=0; i < 16 ; i++, sc->sc_ri++)
		{
			sc->sc_ri |= unit+i;
			sc->sc_ri <<= i;
			sc->sc_ri ^= (ltime.tv_sec >> 16) ^ ltime.tv_usec;
			sc->sc_ri <<= i;
			sc->sc_ri ^= ltime.tv_sec ^ (ltime.tv_usec >> 16);

			if(sc->sc_ri)
				break;
		}

		*(u_short *) & m->m_data[4] = sc->sc_ri;
		m->m_data[6] = 1;
		m->m_data[7] = 0xff;
	}
	else	/* sc_tei != 0xff */
	{
		m->m_data[4] = 0;
		m->m_data[5] = 0;
		m->m_data[6] = 7;
		m->m_data[7] = sc->sc_tei;
	}

	IS_ENQUEUE(&sc->sc_co_q, m);

	disdn_d_startout(sc->sc_unit);

	sc->sc_flags |= TEITIMEOUT;
	timeout((TIMEOUT_FUNC_T) tei_get, (caddr_t) unit, 2 * hz);

	splx(x);
	return(-1);
}

static void
tei_checkresp(int unit)
{
	register struct disdn_softc *sc = &disdn_sc[unit];
	struct disdn_ll_func *ifs = sc->sc_ll_func;
	struct mbuf *m;
	struct timeval ltime;
	int cs;

	if ((*ifs->ll_d_stat)(sc->SC_UNIT) == 2)
        {
#if DISDN_DEBUG > 2
		if (stat_debug)
			printf("checkresp stat\n");
#endif
		return;
	}
	
	if((m = isdn_getbuf(8)) == NULL)
		return;

	cs = splclock();
	ltime = time;
	splx(cs);

	m->m_len = 8;

	m->m_data[0] = 0xfc;
	m->m_data[1] = 0xff;
	m->m_data[2] = 3;
	m->m_data[3] = 0xf;

	sc->sc_ri <<= 1;
	sc->sc_ri ^= (ltime.tv_sec >> 16) ^ ltime.tv_usec;
	sc->sc_ri <<= 1;
	sc->sc_ri ^= ltime.tv_sec ^ (ltime.tv_usec >> 16);

	*(u_short *) & m->m_data[4] = sc->sc_ri;

	m->m_data[6] = 5;
	m->m_data[7] = sc->sc_tei;

	IS_ENQUEUE(&sc->sc_co_q,m);
	disdn_d_startout(sc->sc_unit);
}

#endif	/* NDISDN > 0 */
