/*
 * Copyright (c) 1995 Dietmar Friede and Juergen Krause.  All rights reserved.
 *
 * Copyright (c) 1996 Arne Helme. All rights reserved.
 *
 * Copyright (c) 1996 Gary Jennejohn. All rights reserved. 
 *
 * Copyright (c) 1995, 1996 Hellmuth Michaelis. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Dietmar Friede,
 *	Juergen Krause, Arne Helme, Gary Jennejohn and Hellmuth Michaelis.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *---------------------------------------------------------------------------
 *
 *	last edit-date: [Sun Jul 8 06:54 1996]
 *
 *	-hm	bugfix from Gary Jennejohn: in isdn_check():
 *			old: if((an= ctrl->appl) < Isdn_Appl)
 *			new: if(((an= ctrl->appl) < Isdn_Appl) && (an >= 0))
 *	-hm	adding ip accounting message
 *	-hm	new format for isdn_ipacct()
 *	-hm	new format for isdn_info(), added protocol type
 *	-hm	new format for isdn_ipacct(), dynamic report
 *	-hm	exchanging ii driver with more powerful ipi driver
 *	-hm	remove -Wall compiler warning causes
 *	-hm	isdn protocol now repoted by isdn_disconn_ind()
 *	-hm	converting to message structures
 *	-hm	fixing -Wall compiler warning
 *	-hm	adding Arne's NetBSD/Teles S0/16.3 support
 *	-hm	applying Gary's timeout/idle_time fixes
 *	-hm	local/remote disconnect indication
 *	-gj	unify for use with 2.1R, -stable, -current & NetBSD
 *	-hm	adding Arnes unify patches
 *	-hm	fixing a bug in ISDN_HANGUP processing
 *	-hm	reversing the ISDN_HANGUP bugfix, caused callback problems!
 *	-hm	fixing things from Joerg's code review
 *	-gj	delete isdnwrite, non-existent (-Wunused complained)
 *
 *---------------------------------------------------------------------------*/

#include "isdn.h"
#include "ipi.h"
#include "ity.h"
#include "itel.h"

#ifdef __FreeBSD__
#include "ispy.h"
#else /* __NetBSD__ */
/* Under NetBSD we don't define ispy as a separate pseudo-device */
#define NISPY 1
#endif /* __FreeBSD__ */

#if NISDN > 0

/* #define TO_DEBUG */		/* timeout debugging */

#define TYPNR		3

#define N_ISDN_APPL	(NIPI + NITY + NITEL + NISPY)

#include <sys/param.h>
#include <sys/ioctl.h>
#include <sys/kernel.h>
#include <sys/systm.h>
#include <sys/conf.h>
#include <sys/mbuf.h>
#include <sys/proc.h>

#if BSD > 199306 && defined(DEVFS)
#include <sys/devfsext.h>
#endif /*DEVFS*/

#if BSD > 199306 || defined (__NetBSD__)
#include <i386/include/bisdn_ioctl.h>
#else
#include <machine/bisdn_ioctl.h>
#endif

#include <bisdn/b_isdn.h>

#ifdef __FreeBSD__
#define TIMEOUT_FUNC_T  timeout_func_t
#endif /* __FreeBSD__ */
#ifdef __NetBSD__
#define TIMEOUT_FUNC_T  void *
#define memcpy(x,y,z)   bcopy(y,x,z)
#endif /* __NetBSD__ */

isdn_appl_t isdn_appl[N_ISDN_APPL];

isdn_ctrl_t isdn_ctrl[N_ISDN_CTRL];

int Isdn_Appl;
int Isdn_Ctrl;
int ispy_applnr;

static u_long o_flags, r_flags;
static u_char appl_list[TYPNR];

#define BUFN	16
static struct isqueue daemon_q;

static void isdn_attach __P((void));
static void isdn_check __P((void *));
static void passout __P((struct mbuf *m));

#if BSD > 199306 && defined(__FreeBSD__)
static	d_open_t	isdnopen;
static	d_close_t	isdnclose;
static	d_read_t	isdnread;
static	d_ioctl_t	isdnioctl;

#define CDEV_MAJOR 55
static struct cdevsw isdn_cdevsw = 
	{ isdnopen,	isdnclose,	isdnread,	nowrite,	/*55*/
	  isdnioctl,	nostop,		nullreset,	nodevtotty,/* isdn */
	  seltrue,	nommap,		NULL,	"isdn",	NULL,	-1 };

#ifdef DEVFS
#define NISDNDEVS	32 /* bits in a long - see isdnopen () */
void *devfs_token[NISDNDEVS];
int next_isdn;
#endif
#endif /* __FreeBSD__ */

u_short isdn_state= 0;
static isdn_timeout= 0;

#ifdef __NetBSD__
/* Dummy attach routine for NetBSD */
void
isdnattach(int unused)
{
}
#endif /* __NetBSD__ */

static void
isdn_attach(void)
{
	isdn_appl_t *appl;
	int i, an;

	an = 0;
	
	appl_list[0] = 0;

	for(i=0; i<NIPI; i++, an++)
	{
		appl = &isdn_appl[an];
		appl->ctrl = -1;
		appl->state = 0;
		appl->appl = an;
		appl->typ = ATYP_IP;
		appl->drivno = ipi_attach(an);
		appl->PassUp = ipi_input;
		appl->PassDown = ipi_out;
		appl->Connect = ipi_connect;
		appl->DisConn = ipi_disconnect;
		appl->idle_time = 0;
		appl->timeout = 0;
	}

	appl_list[1] = an;

	for(i= 0 ; i<NITY; i++,an++)
	{
		appl = &isdn_appl[an];
		appl->ctrl = -1;
		appl->state = 0;
		appl->appl = an;
		appl->typ = ATYP_TTY;
		appl->drivno = ityattach(an);
		appl->PassUp = ity_input;
		appl->PassUpMsg = ity_msg;
		appl->PassDown = ity_out;
		appl->Connect = ity_connect;
		appl->DisConn = ity_disconnect;
		appl->idle_time = 0;
		appl->timeout = 0;
	}

	appl_list[2] = an;

	for(i=0; i<(NITEL+NISPY); i++, an++)
	{
		appl = &isdn_appl[an];
		appl->ctrl = -1;
		appl->state = 0;
		appl->appl = an;
		appl->typ = ATYP_TEL;
		appl->drivno = itelattach(an);
		appl->PassUp = itel_input;
		appl->PassDown = itel_out;
		appl->Connect = itel_connect;
		appl->DisConn = itel_disconnect;
		appl->idle_time = 0;
		appl->timeout = 0;
	}
#if NISPY > 0
	isdn_appl[ispy_applnr].headsize =
		sizeof(u_char) + sizeof(u_long) + sizeof(struct timeval);
#endif
	Isdn_Appl = an;
	daemon_q.isq_maxlen= BUFN;
#if BSD > 199306 && defined(DEVFS)
	/* the "master" ISDN device, used to send/receive to/from iid */
	devfs_token[next_isdn++] = 
		devfs_add_devswf(&isdn_cdevsw, 0, DV_CHR, 0, 0, 
				 0600, "isdn/isdn");
#endif
}

int
isdn_ctrl_attach(int n)
{
	int c = Isdn_Ctrl;

	if(c == 0)
		isdn_attach();

	if ((Isdn_Ctrl += n) <= N_ISDN_CTRL)
	{
#if BSD > 199306 && defined(DEVFS)
		/* this assumes that n is always 2 and c therefore */
		/* a multiple of 2 XXXXX */
		devfs_token[next_isdn++] = 
		    devfs_add_devswf(&isdn_cdevsw, ((c>>1)+16), DV_CHR, 0, 0, 
				 0600, "isdn/icard%d", (c>>1));
#endif
		return (c);
	}

	Isdn_Ctrl = c;
	return (-1);
}

/*
 * isdnopen() New open on device.
 * 
 */
int
isdnopen(dev_t dev, int flag, int fmt, struct proc *p)
{
	if (minor(dev) > 31)
		return (ENXIO);

	if (o_flags & (1 << minor(dev)))
		return (EBUSY);

	o_flags |= (1 << minor(dev));

	return (0);
}

int
isdnclose(dev_t dev, int flag, int fmt, struct proc *p)
{
	o_flags &= ~(1 << minor(dev));

	if(minor(dev) == 0)
		isdn_clean_q(&daemon_q, 11);

	return (0);
}

int
isdnread(dev_t dev, struct uio * uio, int ioflag)
{
	int             x;
	int             error = 0;

	if(minor(dev))
	{
		return(ENODEV);
	}

	x = splimp();

	r_flags= 0;

	if(IS_QEMPTY(&daemon_q))
	{
		r_flags = 1;
		error= tsleep((caddr_t) &daemon_q, PZERO + 1, "isdnin", hz);
	}

	if(!IS_QEMPTY(&daemon_q))
	{
		struct	mbuf *m;

		IS_DEQUEUE(&daemon_q,m,16);

		if(m && m->m_len)
			error = uiomove(m->m_data, m->m_len, uio);
		else
			error= EIO;
		
		if(m)
			m_freem(m);
	}
	
	splx(x);
	return error;
}

int
isdnioctl(dev_t dev, int cmd, caddr_t data, int flag, struct proc *p)
{
	int             err, x;
	isdn_appl_t    *appl;
	isdn_ctrl_t    *ctrl;
	unsigned        ab, an, cn;

	err = 0;

	switch (cmd)
	{

		case ISDN_DIAL:
		{
			dial_t     *d= (dial_t*)data;

			ab = appl_list[d->atyp];
			an = d->appl + ab;
			cn = d->ctrl;

			if (an >= Isdn_Appl || cn >= Isdn_Ctrl)
				return (ENODEV);

			appl = &isdn_appl[an];
			if (ISBUSY(appl->ctrl) || appl->state)
			{
				return (EBUSY);
			}

			appl->idle_time = d->idle_time;
			appl->timeout = d->timeout;
			appl->service = d->service;
			appl->blocksize = d->blocksize;
			appl->state = 1;

#ifdef TO_DEBUG
printf("ISDN_DIAL timeout=%ld, idle_time=%ld\n", appl->timeout, appl->idle_time);
#endif

			x = splimp();

			err = (*isdn_ctrl[cn].connect) (cn, an,
				d->service ,d->dst_telno, d->src_telno);

			if(err)
				appl->state= 0;

			splx(x);
			return(err);
		}
		break;

		case ISDN_HANGUP:
		{
			hangup_t *hp = (hangup_t *)data;
	
			ab = appl_list[(int)hp->appl_type];
			an = hp->appl_no + ab;
			cn = hp->channel;
			
			x = splimp();
			
			if (cn < Isdn_Ctrl)
				err = (*isdn_ctrl[cn].disconnect) (cn, hp->cause);

/*XXX*/ 		isdn_disconn_ind(an, ISDNUNKN, 0, 1);

			splx(x);
		}
		break;

		case ISDN_ACCEPT:
		{

		/* accepts connecting application an and controller cn      */
		/* if d->alert an alert is send and/or a timeout is setup   */
		/* to alert every 1 to 1.5s. The application might after    */
		/* some time accept the connection like an ansering machine */
		/* There has to be a connect req every 1 to 1.5 seconds as  */
		/* long as the connection is "open".                        */

			accept_t     *d= (accept_t*)data;

			ab= appl_list[d->atyp];
			an = d->appl + ab;
			cn = d->ctrl;

			if (an >= Isdn_Appl || cn >= Isdn_Ctrl)
				return (ENODEV);

			appl = &isdn_appl[an];
			appl->idle_time= d->idle_time;
			appl->timeout= d->timeout;
			appl->service= d->service;
			appl->blocksize= d->blocksize;

#ifdef TO_DEBUG
printf("ISDN_ACCEPT timeout=%ld idle_time=%ld\n", appl->timeout, appl->idle_time);
#endif

			x = splimp();
			while((err = (*isdn_ctrl[cn].accept) (cn, an
			     , d->service, d->alert, d->blocksize)) == EBUSY)
			{
				err = tsleep((caddr_t) data, PZERO | PCATCH, "iaccept", 2);
				if (err != EWOULDBLOCK)
				{
					splx(x);
					return (err);
				}
			}
			splx(x);
		}
		break;

		case ISDN_APPL_MESG:
		{
			appl_mesg_t     *p = (appl_mesg_t *) data;
			int x;

			ab= appl_list[p->atyp];
			an = p->appl + ab;
			if (an >= Isdn_Appl)
				return (ENODEV);
			appl = &isdn_appl[an];
			x=splimp();
			if(appl->PassUpMsg)
				(*appl->PassUpMsg) (appl->drivno, p->len, p->data);
			splx(x);
			break;
		}

		case ISDN_STIMEOUT:
		{
			timo_t *t = (timo_t *) data;
			isdn_appl_t *appl = &isdn_appl[t->appl];
			appl->timeout = t->timeout;
		}
		break;		

		default:
		{
			cn= dev&0xf;
			if (cn >= Isdn_Ctrl)
				return (ENODEV);
			ctrl = &isdn_ctrl[cn];
			if (ctrl->ioctl)
				return((*ctrl->ioctl) (cn, cmd, data, flag));
			err = ENODEV;
		}
		break;
	}
	return (err);
}

void
isdn_output(unsigned an)
{
	isdn_appl_t    *appl = &isdn_appl[an];
	isdn_ctrl_t    *ctrl;
	int x;

	if(ISFREE(appl->ctrl))
		return;

	ctrl = &isdn_ctrl[appl->ctrl];

	x= splimp();
	(*ctrl->output) (appl->ctrl);
	splx(x);
}

int
isdn_dial(int an)
{
	isdn_appl_t    *appl = &isdn_appl[an];

	if (ISFREE(appl->ctrl))
	{
		struct mbuf *m;
		
		if(appl->state)
			return(0);

		if((m = isdn_getbuf(sizeof(message_dial_t))) != NULL)
		{
			message_dial_t *mp = (message_dial_t *)m->m_data;

			mp->header.message_type = MSG_DIAL;
			mp->header.appl_type = appl->typ;
			mp->header.appl_no = an-appl_list[appl->typ];
			mp->header.channel = appl->ctrl;
			mp->header.service = appl->service;

			passout(m);
			return(0);
		}
	}
	return(1);
}

struct mbuf *
isdn_out(int an)
{
	isdn_appl_t    *appl = &isdn_appl[an];
	struct mbuf *m;
	int x;

	if (ISFREE(appl->ctrl))
		return(NULL);
	x= splimp();
	m= (*appl->PassDown) (appl->drivno);
	splx(x);
	return(m);
}

void
isdn_msg(int an, struct mbuf *m)
{
	isdn_appl_t    *appl = &isdn_appl[an];
	message_msg_t *mp = (message_msg_t *)m->m_data;

	mp->header.message_type = MSG_MESSAGE;
	mp->header.appl_type = appl->typ;
	mp->header.appl_no = an-appl_list[appl->typ];
	mp->header.channel = appl->ctrl;
	mp->header.service = appl->service;

	passout(m);
}

int
isdn_binput(int an, int len, char *buf, int dir)
{
	isdn_appl_t    *appl = &isdn_appl[an];
	struct mbuf *m;

	if((m = isdn_getbuf(len+appl->headsize)) != NULL)
	{
		m->m_len = len+appl->headsize;
		memcpy(&m->m_data[appl->headsize],buf,len);
		return ((*appl->PassUp) (appl->drivno, m, dir));
	}
	return(0);
}


int
isdn_minput(int an, struct mbuf *m, int dir)
{
	isdn_appl_t    *appl = &isdn_appl[an];

	if(!m)
		return(0);

	return ((*appl->PassUp) (appl->drivno, m, dir));
}

void
isdn_accept_ind(int appl, int typ, int cn, int service, int id, char *d, char *s)
{
	struct mbuf *m;

	if((m = isdn_getbuf(sizeof(message_accept_t))) != NULL)
	{
		message_accept_t *mp = (message_accept_t *)m->m_data;

		mp->header.message_type = MSG_ACCEPT;
		mp->header.appl_type = typ;
		mp->header.appl_no = appl;
		mp->header.channel = cn;
		mp->header.service = service;
	
		mp->alert_id = id;
		strcpy(mp->dst_telno, d);
		strcpy(mp->src_telno, s);

		passout(m);
	}
}

void
isdn_info(int an, int proto, int len, char *data)
{
	struct mbuf *m;
	isdn_appl_t *appl = &isdn_appl[an];

	if(len > INFO_MAX)
		len = INFO_MAX;

	if((m = isdn_getbuf(sizeof(message_info_t))) != NULL)
	{
		message_info_t *mp = (message_info_t *)m->m_data;

		mp->header.message_type = MSG_INFO;
		mp->header.appl_type = appl->typ;
		mp->header.appl_no = an-appl_list[appl->typ];
		mp->header.channel = appl->ctrl;
		mp->header.service = appl->service;

		mp->protocol = proto;
		mp->length = len;
		memcpy(mp->info, data, len);

		passout(m);
	}
}

void
isdn_ipacct(int an, int final, int ib, int ob, int cib, int cob, int ip, int op)
{
	struct mbuf *m;
	isdn_appl_t *appl = &isdn_appl[an];
	
	if((m = isdn_getbuf(sizeof(message_ipacct_t))) != NULL)
	{
		message_ipacct_t *mp = (message_ipacct_t *)m->m_data;

		mp->header.message_type = MSG_IPACCT;
		mp->header.appl_type = appl->typ;
		mp->header.appl_no = an-appl_list[appl->typ];
		mp->header.channel = appl->ctrl;
		mp->header.service = appl->service;

		mp->flag = final;
		mp->input_bytes = ib;
		mp->output_bytes = ob;
		mp->compr_input_bytes = cib;
		mp->compr_output_bytes = cob;
		mp->input_packets = ip;
		mp->output_packets = op;

		passout(m);
	}
}

#ifdef TO_DEBUG
int to_debug;
#endif

static void
isdn_check(void *chan)
{
	int x,i;
	struct timeval ltime;
	int cs;

	x = splimp();

	isdn_timeout = 0;

	for(i=0; i < Isdn_Ctrl; i++)
	{
		int an;
		isdn_ctrl_t    *ctrl = &isdn_ctrl[i];

		if(((an = ctrl->appl) < Isdn_Appl) && (an >= 0))
		{
			isdn_appl_t    *appl = &isdn_appl[an];

			/* could be that idle_time is 0 but there's a timeout */
			if(appl->idle_time || appl->timeout)
			{
				isdn_timeout = 1;
				cs = splclock();
				ltime = time;
				splx(cs);
#ifdef TO_DEBUG
/* if (to_debug) */
printf("an %d tv_sec %ld lastact %ld idle_time %ld timeout %ld call_start %ld\n",
an, ltime.tv_sec, ctrl->lastact, appl->idle_time, appl->timeout, ctrl->call_start);
#endif
				if(ltime.tv_sec >=
					(ctrl->lastact + appl->idle_time))
				{
					if( (appl->timeout == 0) ||
					    (((ltime.tv_sec - ctrl->call_start)
					         % (appl->timeout)) == 0) )
					{
							  isdn_disconnect(an,0);
							  break;
					}
				}
			}
		}
	}

	if(isdn_timeout)
		timeout((TIMEOUT_FUNC_T)isdn_check,(void *)&isdn_timeout,hz/2);
	splx(x);
}
					
void
isdn_conn_ind(int an, int cn, int dial)
{
	isdn_appl_t    *appl = &isdn_appl[an];
	struct mbuf *m;
	int cs;

	if((m = isdn_getbuf(sizeof(message_connect_t))) != NULL)
	{
		message_connect_t *mp = (message_connect_t *)m->m_data;

		mp->header.message_type = MSG_CONNECT;
		mp->header.appl_type = appl->typ;
		mp->header.appl_no = an-appl_list[appl->typ];
		mp->header.channel = cn;
		mp->header.service = appl->service;

		mp->active_flag = dial;
		
		passout(m);
	}
	if (appl->Connect)
		(*appl->Connect) (appl->drivno);

	cs = splclock();
	isdn_ctrl[cn].call_start = time.tv_sec;
	splx (cs);
	
	if((isdn_timeout == 0) && appl->idle_time)
	{
		isdn_timeout = 1;
		timeout((TIMEOUT_FUNC_T)isdn_check,(void *)&isdn_timeout,hz/2);
	}
}

void
isdn_disconn_ind(unsigned an, int protocol, int cause, int localflag)
{
	isdn_appl_t *appl = &isdn_appl[an];
	struct mbuf *m;

	if(an >= Isdn_Appl)
		return;

	appl->state = 0;

	if(appl->DisConn)
		(*appl->DisConn)(appl->drivno);

	if((m = isdn_getbuf(sizeof(message_disconnect_t))) != NULL)
	{
		message_disconnect_t *mp = (message_disconnect_t *)m->m_data;

		mp->header.message_type = MSG_DISCONN;
		mp->header.appl_type = appl->typ;
		mp->header.appl_no = an-appl_list[appl->typ];
		mp->header.channel = appl->ctrl;
		mp->header.service = appl->service;

		mp->protocol = protocol;
		mp->cause = cause;
		mp->localflag = localflag;

		passout(m);
	}
	appl->ctrl= -1;
}

void
isdn_disconnect(int an, int cause)
{
	isdn_appl_t    *appl = &isdn_appl[an];

	if (ISBUSY(appl->ctrl))
	{
		int x;
		x = splimp();
		(*isdn_ctrl[appl->ctrl].disconnect)(appl->ctrl,cause);
		splx(x);
	}
}

static void
passout(struct mbuf *m)
{
	int             x;

	x = splimp();
	if ((o_flags & 1) == 0)
	{
		m_freem(m);
		splx(x);
		return;
	}

	if(IS_QFULL(&daemon_q))
	{
		struct mbuf *m1;
		IS_DEQUEUE(&daemon_q,m1,17);
		m_freem(m1);
	}

	IS_ENQUEUE(&daemon_q,m);

	if (r_flags)
	{
		r_flags = 0;
		wakeup((caddr_t) &daemon_q);
	}
	splx(x);
}

struct mbuf*
isdn_getbuf(int len)
{
	struct mbuf *m;

	if(len > MCLBYTES)
	{
/* XXX -hm */	printf("isdn_getbuf: len > MCLBYTES !!\n");
		return(NULL);
	}

	MGETHDR(m, M_DONTWAIT, MT_DATA);

	if(!m)
	{
/* XXX -hm */	printf("isdn_getbuf: MGETHDR failed !!\n");
		return(NULL);
	}

	if(len >= MHLEN)
	{
		MCLGET(m, M_DONTWAIT);
		if ((m->m_flags & M_EXT) == 0)
		{
			m_freem(m);
/* XXX -hm */	printf("isdn_getbuf: MCLGET failed !!\n");			
			return (NULL);
		}
	}
	m->m_len= len;
	return(m);
}

void
isdn_clean_q(struct isqueue *q, int index)
{
	struct mbuf *m;
	while(!IS_QEMPTY(q))
	{
		IS_DEQUEUE(q,m, index);
		if(m) m_freem(m);
	}
}

#if BSD > 199306 && defined(__FreeBSD__)
static isdn_devsw_installed = 0;

static void
isdn_drvinit(void *unused)
{
	dev_t dev;

	if( ! isdn_devsw_installed ) {
		dev = makedev(CDEV_MAJOR,0);
		cdevsw_add(&dev,&isdn_cdevsw,NULL);
		isdn_devsw_installed = 1;
    	}
}

SYSINIT(isdndev,SI_SUB_DRIVERS,SI_ORDER_MIDDLE+CDEV_MAJOR,isdn_drvinit,NULL)
#endif /* __FreeBSD__ */
#endif /* NISDN > 0 */
