/*
 * Copyright (c) 1995 Dietmar Friede and Juergen Krause.  All rights reserved.
 *
 * Copyright (c) 1996 Arne Helme. All rights reserved.
 *
 * Copyright (c) 1996 Gary Jennejohn. All rights reserved. 
 *
 * Copyright (c) 1995, 1996 Hellmuth Michaelis. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Dietmar Friede,
 *	Juergen Krause, Arne Helme, Gary Jennejohn and Hellmuth Michaelis.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *---------------------------------------------------------------------------*
 *
 *	bisdn header file b_isdn.h
 *	--------------------------
 *
 *	last edit-date: [Sun Jul 08 09:50:12 1996]
 *
 *	-hm	added isdn_ipacct() declaration
 *	-hm	new isdn_ipacct() parameters
 *	-hm	added protocol type to isdn_info(), removed type !
 *	-hm	Arne's NetBSD / Teles 16.3 support
 *	-gj	took debug trace out of IS_DEQUEUE (kills -current)
 *	-gj	eliminated __P() constructs per Joerg's comments
 *
 *---------------------------------------------------------------------------*/

typedef struct
{
	short	appl;
	u_short state;
	time_t	timeout;
	time_t	idle_time;
	u_short service;
	u_short blocksize;
	short	dummy;
	u_short headsize;
	short	ctrl;
	short	typ;
	short	drivno;	 /* Number of the high level Driver */
		/* pass data from isdn interface upstream to appl. */
	int	(*PassUp) (int no, struct mbuf *m, int dir);
		/* pass info from isdn interface upstream to appl. */
	int	(*PassUpInfonot_used) (void);
		/* pass info from isdn interface upstream to appl. */
	int	(*PassUpMsg) (int no, int len, char *buf);
		/* get data from application */
	struct mbuf *(*PassDown) (int no);
		/* Connect Indication */
	void	(*Connect) (int no);
		/* Disconnect Indication */ 
	void	(*DisConn) (int no);
} isdn_appl_t;

/* The following code is taken from the if.h IF_Q* stuff */
/* We do not need drop control */

struct	isqueue
{
	struct	mbuf *isq_head;
	struct	mbuf *isq_tail;
	int	isq_len;
	int	isq_maxlen;
	int	dropdummy;
};

#define	IS_QFULL(isq)		((isq)->isq_len >= (isq)->isq_maxlen)
#define	IS_QEMPTY(isq)		((isq)->isq_len == 0)
#define	IS_ENQUEUE(isq, m) { \
	(m)->m_nextpkt = 0; \
	if ((isq)->isq_tail == 0) \
		(isq)->isq_head = m; \
	else \
		(isq)->isq_tail->m_nextpkt = m; \
	(isq)->isq_tail = m; \
	(isq)->isq_len++; \
}
#define	IS_PREPEND(isq, m) { \
	(m)->m_nextpkt = (isq)->isq_head; \
	if ((isq)->isq_tail == 0) \
		(isq)->isq_tail = (m); \
	(isq)->isq_head = (m); \
	(isq)->isq_len++; \
}
#define	IS_DEQUEUE(isq, m, index) { \
	(m) = (isq)->isq_head; \
	if (m) { \
		if (((isq)->isq_head = (m)->m_nextpkt) == 0) \
			(isq)->isq_tail = 0; \
		(m)->m_nextpkt = 0; \
		(isq)->isq_len--; \
	} \
}

#define	OUTREADY	1

typedef struct
{
	short	ctrl;
	u_short state;
	short	islisten;
	short	unit;
	short	appl;
	time_t	lastact;
	time_t	call_start;
	u_short service;
	int	(*connect) (int, int, int, char *, char *);
	int	(*accept) (int, int, int, int, int);
	int	(*disconnect) (int, int);
	void	(*output) (int);
	int	(*ioctl) (int, int, caddr_t, int);
	int	(*getstate_Not_Used) (int);
} isdn_ctrl_t;

struct mbuf *isdn_getbuf (int);
void isdn_accept_ind (int, int, int, int, int, char *, char *);
int isdn_binput (int, int, char *, int);
void isdn_clean_q (struct isqueue *, int);
struct mbuf * isdn_out (int);
void isdn_conn_ind (int, int, int);
int isdn_ctrl_attach (int);
int isdn_dial (int);
void isdn_disconn_ind (unsigned, int, int, int);
void isdn_disconnect (int, int);
void isdn_info (int, int, int, char *);
int isdn_minput (int, struct mbuf *, int);
void isdn_msg (int, struct mbuf *);
void isdn_output (unsigned);
void isdn_ipacct (int, int, int, int, int, int, int, int);

/* general prototypes */

int ipi_input (int, struct mbuf*, int);
void ipi_connect (int);
void ipi_disconnect (int);
int ipi_attach (int);
struct mbuf *ipi_out (int);

int ity_input (int, struct mbuf*, int);
void ity_connect (int);
void ity_disconnect (int);
int ity_msg (int, int, char*);
int ityattach (int);
struct mbuf *ity_out (int);

int itel_input (int, struct mbuf*, int);
void itel_connect (int);
void itel_disconnect (int);
int itelattach (int);
struct mbuf *itel_out (int);

/* EOF */
