/*---------------------------------------------------------------------------*
 *
 *	lap.c: Link Access Procedures LAP
 *	This code will work for LAPB and LAPD
 *
 *	The code is based on the LAPB code of Phil Karn, KA9Q
 *	Parts Copyright 1991 Phil Karn, KA9Q
 *	Copyright 1995 Dietmar Friede
 *
 *---------------------------------------------------------------------------*
 *
 *	last edit-date: [Tue May 21 11:46:36 1996]
 *
 *	-hm	changes for bisdn integration
 *	-hm	adding Arne's NetBSD changes
 *
 *---------------------------------------------------------------------------*/

#ifdef __FreeBSD__
#include <param.h>
#include <ioctl.h>
#include <kernel.h>
#include <systm.h>
#include <mbuf.h>
#endif /* __FreeBSD__ */
#ifdef __NetBSD__
#include <sys/param.h>
#include <sys/ioctl.h>
#include <sys/kernel.h>
#include <sys/systm.h>
#include <sys/device.h>
#include <sys/mbuf.h>
#endif /* __NetBSD__ */

#include <machine/stdarg.h>

#if BSD > 199306 || defined (__NetBSD__)
#include <i386/include/bisdn_ioctl.h>
#else
#include <machine/bisdn_ioctl.h>
#endif

#include <bisdn/b_isdn.h>
#include <bisdn/b_isdnlap.h>
#include <bisdn/b_disdn.h>

#ifdef __FreeBSD__
#define TIMEOUT_FUNC_T  timeout_func_t
#endif /* __FreeBSD__ */
#ifdef __NetBSD__
#define TIMEOUT_FUNC_T  void *
#endif /* __NetBSD__ */

static void lapinvrex __P((struct hdlc *));
static void lapenqresp __P((struct hdlc *));
static void lapclrex __P((struct hdlc *));
static int lapackours __P((struct hdlc *, u_short));
static void laptimer __P((struct hdlc *, int));
static void lapsettimer __P((struct hdlc *, int, int));
static int lapsendctl __P(( struct hdlc *, int , int , int));
static u_long timediff __P((struct timeval*));
static int lap_output __P((struct hdlc *));

int
lap_input(struct hdlc * hd, int cmdrsp, u_char * bp)
{
	int             control;
	int             class;	/* General class (I/S/U) of frame */
	u_short         type;	/* Specific type (I/RR/RNR/etc) of frame */
	char            pf;	/* extracted poll/final bit */
	char            poll = 0;
	char            final = 0;
	u_short         nr;	/* ACK number of incoming frame */
	u_short         ns;	/* Seq number of incoming frame */
	int		val= 0;

	if (bp == NULL || hd == NULL)
		return(-1);

	/* Extract the various parts of the control field for easy use */
	nr= ns= 0;
	control = *bp;
	type = lap_ftype(control);
	class = type & 0x3;

	switch(hd->proto)
	{
	case MOD8HDLC:
		pf = control & PF;
		switch (class)
		{
		case I:
		case I + 2:
			ns = (control >> 1) & hd->mmask;
		case S:		/* Note fall-thru */
			nr = (control >> 5) & hd->mmask;
			break;
		}
		break;
	case MOD128HDLC:
		switch (class)
		{
		case I:
		case I + 2:
			ns = (control >> 1);
		case S:		/* Note fall-thru */
			nr = (bp[1] >> 1) & hd->mmask;
			pf= bp[1]&1;
			break;
		default:
			pf = control & PF;
		}
		break;
	default:
		return(-1);
	}

	if (pf)
	{
		if (cmdrsp)
			poll = 1;
		else
			final = 1;
	}

	switch (hd->state)
	{
	case LAP_DISCONNECTED:
		switch (type)
		{
		case SABM:	/* Initialize or reset link */
		case SABME:	/* Initialize or reset link */
			lapsendctl(hd, LAP_RESPONSE, UA, pf);
			lapclrex(hd);
			hd->unack = hd->vr = hd->vs = 0;
			lapstate(hd, LAP_CONNECTED);
			hd->srt = hz;
			hd->mdev = 0;
			lapsettimer(hd,T1, 2 * hd->srt);
			laptimer(hd,T3);
			break;
		case DM:	/* Ignore to avoid infinite loops */
			break;
		default:	/* All others get DM */
			if (poll)
				lapsendctl(hd, LAP_RESPONSE, DM, pf);
			break;
		}
		break;
	case LAP_SETUP:
		switch (type)
		{
		case SABM:	/* Initialize or reset link */
		case SABME:	/* Simultaneous open */
			lapsendctl(hd, LAP_RESPONSE, UA, pf);
			break;
		case DISC:
			lapsendctl(hd, LAP_RESPONSE, DM, pf);
			break;
		case UA:	/* Connection accepted */
			/* Note: xmit queue not cleared */
			laptimer(hd,T3);
			hd->unack = hd->vr = hd->vs = 0;
			lapstate(hd, LAP_CONNECTED);
			break;
		case DM:	/* Connection refused */
			laptimer(hd,T0);
			hd->reason = LB_DM;
			lapstate(hd, LAP_DISCONNECTED);
			break;
		default:	/* All other frames ignored */
			break;
		}
		break;
	case LAP_DISCPENDING:
		switch (type)
		{
		case SABM:	/* Initialize or reset link */
		case SABME:
			lapsendctl(hd, LAP_RESPONSE, DM, pf);
			break;
		case DISC:
			lapsendctl(hd, LAP_RESPONSE, UA, pf);
			break;
		case UA:
		case DM:
			laptimer(hd,T0);
			lapstate(hd, LAP_DISCONNECTED);
			break;
		default:
			if (poll)
				lapsendctl(hd, LAP_RESPONSE, DM, pf);
			break;
		}
		break;
	case LAP_CONNECTED:
		switch (type)
		{
		case SABM:	/* Initialize or reset link */
		case SABME:
			lapsendctl(hd, LAP_RESPONSE, UA, pf);
			lapclrex(hd);
			laptimer(hd,T3);
			hd->unack = hd->vr = hd->vs = 0;
			lapstate(hd, LAP_CONNECTED);	/* Purge queues */
			break;
		case DISC:
			lapsendctl(hd, LAP_RESPONSE, UA, pf);
			laptimer(hd,T0);
			hd->reason = LB_NORMAL;
			lapstate(hd, LAP_DISCONNECTED);
			break;
		case DM:
			hd->reason = LB_DM;
			lapstate(hd, LAP_DISCONNECTED);
			break;
		case UA:
			lap_estlink(hd,1);
			lapstate(hd, LAP_SETUP);
			break;
		case FRMR:
			lap_estlink(hd,2);
			lapstate(hd, LAP_SETUP);
			break;
		case RR:
		case RNR:
			hd->flags.remotebusy = (control == RNR) ? 1 : 0;
			if (poll)
				lapenqresp(hd);
			lapackours(hd, nr);
			break;
		case REJ:
			hd->flags.remotebusy = 0;
			if (poll)
				lapenqresp(hd);
			lapackours(hd, nr);
			laptimer(hd,T3);
			/* This may or may not actually invoke transmission,
			   depending on whether this REJ was caused by our
			   losing his prior ACK. */
			lapinvrex(hd);
			break;
		case I:
			lapackours(hd, nr);

			if (ns != hd->vr)
			{
				if (poll)
					lapenqresp(hd);
				hd->response = 0;
				break;
			}
			hd->flags.rejsent = 0;
			hd->vr = (hd->vr + 1) & hd->mmask;
			if (poll)
			{
				lapsendctl(hd, LAP_RESPONSE, RR, PF);
			} else
			{
				hd->response = RR;
			}
			val= 1;
			break;
		default:
			break;
		}
		break;
	case LAP_RECOVERY:
		switch (type)
		{
		case SABM:	/* Initialize or reset link */
		case SABME:
			lapsendctl(hd, LAP_RESPONSE, UA, pf);
			lapclrex(hd);
			laptimer(hd,T3);
			hd->unack = hd->vr = hd->vs = 0;
			lapstate(hd, LAP_CONNECTED);	/* Purge queues */
			break;
		case DISC:
			lapsendctl(hd, LAP_RESPONSE, UA, pf);
			laptimer(hd,T0);
			hd->response = UA;
			hd->reason = LB_NORMAL;
			lapstate(hd, LAP_DISCONNECTED);
			break;
		case DM:
			hd->reason = LB_DM;
			lapstate(hd, LAP_DISCONNECTED);
			break;
		case UA:
			lap_estlink(hd,3);
			lapstate(hd, LAP_SETUP);	/* Re-establish */
			break;
		case FRMR:
			lap_estlink(hd,4);
			lapstate(hd, LAP_SETUP);	/* Re-establish link */
			break;
		case RR:
		case RNR:
			hd->flags.remotebusy = (control == RNR) ? 1 : 0;
			if (final)
			{
				laptimer(hd,T0);
				lapackours(hd, nr);
				if (hd->unack != 0)
				{
					lapinvrex(hd);
				} else
				{
					laptimer(hd,T3);
					lapstate(hd, LAP_CONNECTED);
				}
			} else
			{
				if (poll)
					lapenqresp(hd);
				lapackours(hd, nr);
				laptimer(hd,T1);
			}
			break;
		case REJ:
			hd->flags.remotebusy = 0;
			if (final)
			{
				laptimer(hd,T0);
				lapackours(hd, nr);
				if (hd->unack != 0)
				{
					lapinvrex(hd);
				} else
				{
					laptimer(hd,T3);
					lapstate(hd, LAP_CONNECTED);
				}
			} else
			{
				if (poll)
					lapenqresp(hd);
				lapackours(hd, nr);
				if (hd->unack != 0)
				{
					lapinvrex(hd);
				}
				laptimer(hd,T1);
			}
			break;
		case I:
			lapackours(hd, nr);	/** == -1) */
			laptimer(hd,T1);
			if (ns != hd->vr)
			{
				if (!hd->flags.rejsent)
				{
					hd->flags.rejsent = 1;
					lapsendctl(hd, LAP_RESPONSE, REJ, pf);
				} else if (poll)
					lapenqresp(hd);

				hd->response = 0;
				break;
			}
			hd->flags.rejsent = 0;
			hd->vr = (hd->vr + 1) & hd->mmask;
			if (poll)
			{
				lapsendctl(hd, LAP_RESPONSE, RR, PF);
			} else
			{
				hd->response = RR;
			}
			val= 1;
			break;
		default:
			break;
		}
		break;
	}

	lap_output(hd);
	if (hd->response != 0)
	{
		lapsendctl(hd, LAP_RESPONSE, hd->response, pf);
		hd->response = 0;
	}
	return(val);
}

/* Establish data link */
void
lap_estlink(struct hdlc *hd,int c)
{
	lapclrex(hd);
	hd->retries = 0;
	switch(hd->proto)
	{
	case MOD8HDLC:
		lapsendctl(hd, LAP_COMMAND, SABM, PF);
		break;
	case MOD128HDLC:
		lapsendctl(hd, LAP_COMMAND, SABME, PF);
	}
	laptimer(hd,T1);
}

static int
lapackours(struct hdlc *hd, u_short n)
{
	int             acked = 0;	/* Count of frames acked by this ACK */
	u_short         oldest;	/* Seq number of oldest unacked I-frame */
	long           rtt, abserr;

	/* Free up acknowledged frames by purging frames from the I-frame
	   transmit queue. Start at the remote end's last reported V(r) and
	   keep going until we reach the new sequence number. If we try to
	   free a null pointer, then we have a frame reject condition. */

	oldest = (hd->vs - hd->unack) & hd->mmask;
	while (hd->unack != 0 && oldest != n)
	{
		if((*hd->dequeue)(hd) == 0)
			return(-1);

		hd->unack--;
		acked++;
		if (hd->flags.rtt_run && (hd->rtt_seq == oldest))
		{
			/* A frame being timed has been acked */
			hd->flags.rtt_run = 0;
			/* Update only if frame wasn't retransmitted */
			if (!hd->flags.retrans)
			{
				rtt = timediff(&hd->rtt_time);
				abserr = (rtt > hd->srt) ? rtt - hd->srt :
					hd->srt - rtt;

				/* Run SRT and mdev integrators */
				hd->srt = ((hd->srt * 7) + rtt + 4) >> 3;
				hd->mdev = ((hd->mdev * 3) + abserr + 2) >> 2;
				/* Update timeout */
				lapsettimer(hd,T1, 4 * hd->mdev + hd->srt);
			}
		}
		hd->flags.retrans = 0;
		hd->retries = 0;
		oldest = (oldest + 1) & hd->mmask;
	}
	if (hd->unack == 0)
	{
		/* All frames acked, stop timeout */
		laptimer(hd,T0);
	} else if (acked != 0)
	{
		/* Partial ACK; restart timer */
		laptimer(hd,T1);
	}
	return 0;
}

/* Clear exception conditions */
static void
lapclrex(struct hdlc *hd)
{
	hd->flags.remotebusy = 0;
	hd->flags.rejsent = 0;
	hd->response = 0;
	laptimer(hd,T0);
}
/* Enquiry response */
static void
lapenqresp(struct hdlc *hd)
{
	lapsendctl(hd, LAP_RESPONSE, RR,PF);
	hd->response = 0;
	laptimer(hd,T0);
}
/* Invoke retransmission */
static void
lapinvrex(struct hdlc *hd)
{
	hd->vs -= hd->unack;
	hd->vs &= hd->mmask;
	hd->unack = 0;
/*
	(*hd->resend)(hd);
*/
}
/* Send S or U frame to currently connected station */
static int
lapsendctl( struct hdlc *hd, int cmdrsp, int cmd, int pf)
{
	return ((*hd->sendctl)(hd, cmdrsp, cmd, pf, hd->vr));
}

/* Start data transmission on link, if possible
 * Return number of frames sent
 */
static int
lap_output(struct hdlc *hd)
{
	if (hd == NULL
	    || (hd->state != LAP_RECOVERY && hd->state != LAP_CONNECTED)
	    || hd->flags.remotebusy)
		return 0;

	if((*hd->start_out)(hd->unit) == 0)
		return(0);
	laptimer(hd,T1);
	if (!hd->flags.rtt_run)
	{
		/* Start round trip timer */
		hd->rtt_seq = (hd->vs-1) & hd->mmask;
		hd->rtt_time = time;
		hd->flags.rtt_run = 1;
	}
	return 1;
}
/* Set new link state */
void
lapstate( struct hdlc *hd, int s)
{
	int             oldstate;

	if(hd->state == s)
		return;
	oldstate = hd->state;
	hd->state = s;
	switch(s)
	{
	case LAP_CONNECTED:
		(*hd->connect)(hd);
		break;
	case LAP_DISCONNECTED:
		(*hd->disconnect)(hd);
		laptimer(hd,T0);
		break;
	}
}

/* Called whenever timer T1 or T3 expires */
static void
laprecover(struct hdlc *hd)
{
	int x;

	x=splimp();
	hd->flags.retrans = 1;
	hd->retries++;

	switch (hd->state)
	{
	case LAP_SETUP:
		if (hd->n2 != 0 && hd->retries > hd->n2)
		{
			hd->reason = LB_TIMEOUT;
			lapstate(hd, LAP_DISCONNECTED);
		} else
		{
			switch(hd->proto)
			{
			case MOD8HDLC:
				lapsendctl(hd, LAP_COMMAND, SABM, PF);
				break;
			case MOD128HDLC:
				lapsendctl(hd, LAP_COMMAND, SABME, PF);
			}
			laptimer(hd,T1);
		}
		break;
	case LAP_DISCPENDING:
		if (hd->n2 != 0 && hd->retries > hd->n2)
		{
			hd->reason = LB_TIMEOUT;
			lapstate(hd, LAP_DISCONNECTED);
		} else
		{
			lapsendctl(hd, LAP_COMMAND, DISC, PF);
			laptimer(hd,T1);
		}
		break;
	case LAP_CONNECTED:
	case LAP_RECOVERY:
		if (hd->n2 != 0 && hd->retries > hd->n2)
		{
			/* Give up */
			lapsendctl(hd, LAP_RESPONSE, DM, PF);
			hd->reason = LB_TIMEOUT;
			lapstate(hd, LAP_DISCONNECTED);
		} else
		{
			/* Transmit poll */
			lapsendctl(hd, LAP_COMMAND, RR, PF);
			hd->response = 0;
			laptimer(hd,T3);
		}
		break;
	}
	splx(x);
}

#ifdef NOTDEF
/* Transmit query */
static void
laptxenq(struct hdlc *hd)
{
	char            ctl;
	u_char    *bp;

	if((*hd->resend)(hd, PF, hd->vs-hd->unack ,hd->vr) <= 0)
		lapsendctl(hd, LAP_COMMAND, RR, PF);
	hd->response = 0;
	laptimer(hd,T1);
}
#endif

u_short
lap_ftype(u_short c)
{
	if ((c & 1) == 0)	/* I-frame */
		return I;
	if (c & 2)	/* U-frames use all except P/F bit for type */
		return (u_short) (uchar(c) & ~PF);
	else			/* S-frames use low order 4 bits for type */
		return (u_short) (uchar(c) & 0xf);
}

static void
laptimer(struct hdlc * hd, int tp)
{
	if(hd->timeout)
	{
		untimeout((TIMEOUT_FUNC_T)laprecover,hd);
	}
	hd->timeout= tp;
	if(tp)
		timeout((TIMEOUT_FUNC_T)laprecover,hd,hd->toutval[tp]);
}

static void
lapsettimer(struct hdlc * hd, int tp, int val)
{
	if(tp && val)
		hd->toutval[tp]= val;
}

static u_long
timediff(struct timeval *t1)
{
	long t;
	t= (time.tv_usec - t1->tv_usec) / 1000;
	return(t + (time.tv_sec - t1->tv_sec)*1000);
}
