/* Upper sub-layer (LAP) definitions */

/* Control field templates */
#define	I	0x00	/* Information frames */
#define	S	0x01	/* Supervisory frames */
#define	RR	0x01	/* Receiver ready */
#define	RNR	0x05	/* Receiver not ready */
#define	REJ	0x09	/* Reject */
#define	U	0x03	/* Unnumbered frames */
#define	SABME	0x6f	/* Set Asynchronous Balanced Mode */
#define	SABM	0x2f	/* Set Asynchronous Balanced Mode */
#define	DISC	0x43	/* Disconnect */
#define	DM	0x0f	/* Disconnected mode */
#define	UA	0x63	/* Unnumbered acknowledge */
#define	FRMR	0x87	/* Frame reject */
#define	UI	0x03	/* Unnumbered information */
#define	PF	0x10	/* Poll/final bit */

#define	MMASK	7	/* Mask for modulo-8 sequence numbers */

/* FRMR reason bits */
#define	W	1	/* Invalid control field */
#define	X	2	/* Unallowed I-field */
#define	Y	4	/* Too-long I-field */
#define	Z	8	/* Invalid sequence number */

#define	LAP_DISCONNECTED	1
#define LAP_LISTEN		2
#define	LAP_SETUP		3
#define	LAP_DISCPENDING		4
#define	LAP_CONNECTED		5
#define	LAP_RECOVERY		6

#define LAP_COMMAND		2
#define LAP_RESPONSE		0
#define	LB_NORMAL	0		/* Normal close */
#define	LB_DM		1		/* Received DM from other end */
#define	LB_TIMEOUT	2		/* Excessive retries */

#define MOD8HDLC	1
#define MOD128HDLC	2

#define        uchar(x) ((unsigned char)(x))

#define        T0	0
#define        T1	1
#define        T3	2

/* Per-connection link control block
 * One exists for each logical Level 2 connection
 */
struct hdlc {
	u_short	unit;

        int     (*resend) __P((struct hdlc*));
        int     (*sendctl) __P((struct hdlc *, int, int, int, int));
        int     (*start_out) __P((int));
        int     (*connect) __P((struct hdlc *));
        int     (*disconnect) __P((struct hdlc *));
        int     (*dequeue) __P((struct hdlc *));

	struct {
		char rejsent;		/* REJ frame has been sent */
		char remotebusy;	/* Remote sent RNR */
		char rtt_run;		/* Round trip "timer" is running */
		char retrans;		/* A retransmission has occurred */
	} flags;

	char reason;			/* Reason for connection closing */

	char response;			/* Response owed to other end */
	u_char proto;			/* Protocol version */
	u_char vs;			/* Our send state variable */
	u_char vr;			/* Our receive state variable */
	u_char unack;			/* Number of unacked frames */
	struct	isqueue uack_q;
	u_char timeout;
	u_char mmask;
	u_short toutval[3];
	int maxframe;			/* Transmit flow control level, frames */
	u_short paclen;			/* Maximum outbound packet size, bytes */
	u_short window;			/* Local flow control limit, bytes */
	u_short pthresh;			/* Poll threshold, bytes */
	unsigned retries;		/* Retry counter */
	unsigned n2;			/* Retry limit */
	int state;			/* Link state */
	struct timeval rtt_time;	/* Stored clock values for RTT, ticks */
	u_short rtt_seq;		/* Sequence number being timed */
	u_long srt;			/* Smoothed round-trip time, ms */
	u_long mdev;			/* Mean rtt deviation, ms */
};

