/*
 * Copyright (c) 1995 Dietmar Friede and Juergen Krause.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Dietmar Friede and
 *	Juergen Krause.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Dietmar Friede AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE D.Friede OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *---------------------------------------------------------------------------*
 *
 *	last edit-date: [Fri Jul  5 14:03:55 1996]
 *
 *	-gj	unify for 2.1R, -stable, -current and NetBSD
 *	-hm	adding arnes unify patches
 *	-hm	small devfs patch from Gary
 *
 *---------------------------------------------------------------------------*/

#include "itel.h"

#ifdef __FreeBSD__
#include "ispy.h"
#else /* __NetBSD__ */
/* Under NetBSD we don't define ispy as a separate pseudo-device */
#define NISPY 1
#endif /* __NetBSD */

#if NITEL+NISPY > 0

#include <sys/param.h>
#include <sys/systm.h>

#if BSD > 199306 || defined (__NetBSD__)
#include <sys/conf.h>
#endif

#include <sys/mbuf.h>
#include <sys/ioctl.h>
#include <sys/tty.h>
#include <sys/proc.h>

#if BSD > 199306 || defined (__NetBSD__)
/* user.h not required */
#else
#include <sys/user.h>
#endif

#include <sys/uio.h>
#include <sys/kernel.h>

#if BSD > 199306 && defined(DEVFS)
#include <sys/devfsext.h>
#endif /*DEVFS*/

#if BSD > 199306 || defined (__NetBSD__)
#include <i386/include/bisdn_ioctl.h>
#else
#include <machine/bisdn_ioctl.h>
#endif

#include <bisdn/b_isdn.h>

int nitel = (NITEL+NISPY);
int ispy_applnr;

static int applnr[(NITEL+NISPY)];
static struct isqueue i_q[(NITEL+NISPY)];
static struct isqueue o_q[(NITEL+NISPY)];
static u_short itel_state[(NITEL+NISPY)];
static unsigned long ispyabscnt;
static int next_if = 0;

#if NISPY > 0
int ispy_no = NITEL;
#else
int ispy_no = -1;
#endif

#define ITEL_SIZE	2048
#define OPEN		1
#define CONNECT		2
#define READ_WAIT	4
#define WRITE_WAIT	8

#define min(a,b)        ((a)<(b)?(a):(b))

#if BSD > 199306 && defined(__FreeBSD__)
static	d_open_t	itelopen;
static	d_close_t	itelclose;
static	d_read_t	itelread;
static	d_write_t	itelwrite;
static	d_ioctl_t	itelioctl;

#define CDEV_MAJOR 57
static struct cdevsw itel_cdevsw = 
	{ itelopen,	itelclose,	itelread,	itelwrite,	/*57*/
	  itelioctl,	nostop,		nullreset,	nodevtotty,/* itel */
	  seltrue,	nommap,		NULL,	"itel",	NULL,	-1 };

#ifdef	DEVFS
void	*devfs_token [NITEL+NISPY];
#endif
#endif /* __FreeBSD_version >= 199306 */

int
itelattach(int ap)
{
	if(next_if >= (NITEL+NISPY))
		return(-1);
	applnr[next_if]= ap;
	i_q[next_if].isq_maxlen= 4;
	o_q[next_if].isq_maxlen= 4;
	if(next_if == ispy_no)
	{
		ispy_applnr= ap;
		i_q[next_if].isq_maxlen= 32;
	}
#if BSD > 199306 && defined(DEVFS)
	if(next_if == ispy_no)
		devfs_token [next_if]  =
			devfs_add_devswf(&itel_cdevsw, next_if, DV_CHR, 0, 0, 
                                0600, "isdn/ispy");
	else
		devfs_token [next_if]  =
			devfs_add_devswf(&itel_cdevsw, next_if, DV_CHR, 0, 0, 
				0600, "isdn/itel%*0d", 2, next_if);
#endif
	return(next_if++);
}

int
itel_input(int no, struct mbuf *m, int out)
{
	if(m == NULL)
		return(0);

	if(IS_QFULL(&i_q[no]))
	{
		struct mbuf *m1;
		IS_DEQUEUE(&i_q[no],m1,18);
		m_freem(m1);
	}

	if(no == ispy_no)
	{
		m->m_data[0]= out;
		*(u_long *) &m->m_data[1]= ispyabscnt++;
		*(struct timeval *) &m->m_data[5]= time;
	}
	IS_ENQUEUE(&i_q[no],m);

	if(itel_state[no] & READ_WAIT)
	{
		itel_state[no] &= ~READ_WAIT;
		wakeup((caddr_t) &i_q[no]);
	}
	return(1);
}

struct mbuf *
itel_out(int no)
{
	struct mbuf    *m;

	IS_DEQUEUE(&o_q[no], m,19);
	if(itel_state[no] & WRITE_WAIT)
	{
		itel_state[no] &= ~WRITE_WAIT;
		wakeup((caddr_t)&o_q[no]);
	}
	return(m);

}

void
itel_connect(int no)
{
	isdn_clean_q(&i_q[no], 7);
	isdn_clean_q(&o_q[no], 8);
	itel_state[no] |= CONNECT;
}

void
itel_disconnect(int no)
{
	int s;
	s= itel_state[no];
	itel_state[no] &= OPEN;
	if(itel_state[no])
	{
		if(s & READ_WAIT)
			wakeup((caddr_t) &i_q[no]);
		if(s & WRITE_WAIT)
			wakeup((caddr_t) &o_q[no]);
	}
	isdn_clean_q(&i_q[no], 9);
	isdn_clean_q(&o_q[no], 10);
}

int
#if BSD > 199306 || defined (__NetBSD__)
itelopen(dev_t dev, int flag, int fmt, struct proc *p)
#else
itelopen(dev_t dev, int flag)
#endif
{
	int             no= minor(dev);

	if (no > (NITEL+NISPY))
		return (ENXIO);

	if(no == ispy_no)
	{
		if(itel_state[no]&OPEN) return(EBUSY);
		itel_state[no] |= OPEN | CONNECT;
		return(0);
	}

	if(!(itel_state[no] & CONNECT))
		return(EIO);

	if(itel_state[no] & OPEN)
		return(0);
	itel_state[no] |= OPEN;

	return (0);
}

int
#if BSD > 199306 || defined (__NetBSD__)
itelclose(dev_t dev, int flag, int fmt, struct proc *p)
#else
itelclose(dev_t dev, int flag)
#endif
{
	int             no= minor(dev);

	if(itel_state[no] & READ_WAIT)
		wakeup((caddr_t) &i_q[no]);
	if(itel_state[no] & WRITE_WAIT)
		wakeup((caddr_t) &o_q[no]);
	itel_state[no] &= CONNECT;
	return (0);
}

int
#if BSD > 199306 || defined (__NetBSD__)
itelioctl (dev_t dev, int cmd, caddr_t data, int flag, struct proc *p)
#else
itelioctl (dev_t dev, int cmd, caddr_t data, int flag)
#endif
{
        switch (cmd) {
            default:
                return (ENOTTY);
        }
        return (0);
}

int
#if BSD > 199306 || defined (__NetBSD__)
itelread(dev_t dev, struct uio * uio, int ioflag)
#else
itelread(dev_t dev, struct uio * uio)
#endif
{
	int             no= minor(dev);
	int             x;
	int             error = 0;
	struct mbuf	*m;

	if((itel_state[no] & CONNECT) == 0)
		return(EIO);

	while(IS_QEMPTY(&i_q[no]) && (itel_state[no] & CONNECT))
	{
		itel_state[no] |= READ_WAIT;
		if((error = tsleep((caddr_t) &i_q[no], TTIPRI | PCATCH, "itel", 0 )) != 0)
		{
			itel_state[no] &= ~READ_WAIT;
			return(error);
		}
	}

	x = splimp();
	IS_DEQUEUE(&i_q[no],m,20)
	if(m && m->m_len)
		error = uiomove(m->m_data, m->m_len, uio);
	else error= EIO;
	if(m)
		m_freem(m);
	splx(x);
	return error;
}

int
#if BSD > 199306 || defined (__NetBSD__)
itelwrite(dev_t dev, struct uio * uio, int ioflag)
#else
itelwrite(dev_t dev, struct uio * uio)
#endif
{
	int             no= minor(dev);
	int             x;
	int             error = 0;
	struct mbuf	*m;

	if((itel_state[no] & CONNECT) == 0)
		return(EIO);

	while(IS_QFULL(&o_q[no]) && (itel_state[no] & CONNECT))
	{
		itel_state[no] |= WRITE_WAIT;
		if((error= tsleep((caddr_t) &o_q[no], TTIPRI | PCATCH, "witel", 0 )) != 0)
		{
			itel_state[no] &= ~WRITE_WAIT;
			return(error);
		}
	}

	x = splimp();
	if((itel_state[no] & CONNECT) == 0)
	{
		splx(x);
		return(0);
	}

	if((m = isdn_getbuf(2048)) != NULL)
	{
		m->m_len = min(ITEL_SIZE, uio->uio_resid);
		error = uiomove(m->m_data, m->m_len, uio);
		IS_ENQUEUE(&o_q[no],m);
		isdn_output(applnr[no]);
	}
	splx(x);
	return error;
}

#if BSD > 199306 && defined(__FreeBSD__)
static itel_devsw_installed = 0;

static void 	itel_drvinit(void *unused)
{
	dev_t dev;

	if( ! itel_devsw_installed ) {
		dev = makedev(CDEV_MAJOR, 0);
		cdevsw_add(&dev,&itel_cdevsw, NULL);
		itel_devsw_installed = 1;
    	}
}

SYSINIT(iteldev,SI_SUB_DRIVERS,SI_ORDER_MIDDLE+CDEV_MAJOR,itel_drvinit,NULL)
#endif /* __FreeBSD__ */
#endif
