/*
 * Copyright (c) 1995 Dietmar Friede and Juergen Krause.  All rights reserved.
 *
 * Copyright (c) 1996 Gary Jennejohn. All rights reserved.
 *
 * Copyright (c) 1996 Arne Helme. All rights reserved.  
 *
 * Copyright (c) 1995, 1996 Hellmuth Michaelis. All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Dietmar Friede,
 *	Juergen Krause, Gary Jennejohn, Arne Helme and Hellmuth Michaelis.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *---------------------------------------------------------------------------*
 *
 *	last edit-date: [Mon Jun  3 11:37:08 1996]
 *
 *	-hm	adding support for FreeBSD 2.1 (untested)
 *	-hm	fixing compiler -Wall warnings
 *	-gj	unify for 2.1R, -stable, -current and NetBSD
 *	-hm	adding Arne's unify patches
 *	-hm	small devfs patch from Gary
 *
 *---------------------------------------------------------------------------*/

#include "ity.h"
#if NITY > 0

#include <sys/param.h>
#include <sys/systm.h>

#if BSD > 199306 || defined (__NeBSD__)
#include <sys/conf.h>
#endif

#include <sys/mbuf.h>
#include <sys/ioctl.h>
#include <sys/select.h>
#include <sys/tty.h>
#include <sys/proc.h>

#if BSD > 199306 || defined (__NeBSD__)
/* user.h not used */
#else
#include <sys/user.h>
#endif

#include <sys/conf.h>
#include <sys/file.h>
#include <sys/uio.h>
#include <sys/kernel.h>
#include <sys/syslog.h>
#include <sys/types.h>

#if BSD > 199306 && defined(DEVFS)
#include <sys/devfsext.h>
#endif /*DEVFS*/

#if BSD > 199306 || defined (__NeBSD__)
#include <i386/include/bisdn_ioctl.h>
#else
#include <machine/bisdn_ioctl.h>
#endif

#include <bisdn/b_isdn.h>

#if BSD > 199306 && defined(__FreeBSD__)
static	d_open_t	ityopen;
static	d_close_t	ityclose;
static	d_read_t	ityread;
static	d_write_t	itywrite;
static	d_ioctl_t	ityioctl;
static	d_stop_t	itystop;
static	d_devtotty_t	itydevtotty;

#define CDEV_MAJOR 56
static struct cdevsw ity_cdevsw = 
	{ ityopen,	ityclose,	ityread,	itywrite,	/*56*/
	  ityioctl,	itystop,	noreset,	itydevtotty,/* ity */
	  ttselect,	nommap,		NULL,	"ity",	NULL,	-1 };

#ifdef	DEVFS
static void	*devfs_token[NITY];
static void	*devfs_token_out[NITY];
#endif
#endif /* __FreeBSD_version >= 199306 */

#define	ITY_CONN	1
#define	ITY_IINTR	2

#ifdef __FreeBSD__
#define TIMEOUT_FUNC_T  timeout_func_t
#endif /* __FreeBSD__ */
#ifdef __NetBSD__
#define TIMEOUT_FUNC_T  void *
#endif /* __NetBSD__ */

extern isdn_appl_t isdn_appl[];

int ityparam __P((struct tty *tp, struct termios *t));
int ityselect __P((dev_t dev, int rw, struct proc *p));

static void itystart __P((struct tty *tp));

int nity = NITY;
int itydefaultrate = 64000;

struct tty ity_tty[NITY];

static short ity_addr[NITY];
static int applnr[NITY];
static int itystate[NITY];
static struct isqueue i_q[NITY], o_q[NITY];
static int next_if = 0;

#define	UNIT(x)		(minor(x)&0x3f)
#define	OUTBOUND(x)	((minor(x)&0x80)==0x80)

int
ityattach(int ap)
{
	if(next_if >= NITY)
		return(-1);

	applnr[next_if]= ap;
	i_q[next_if].isq_maxlen= 8;
	o_q[next_if].isq_maxlen= 4;
#if BSD > 199306 && defined(DEVFS)
	devfs_token[next_if] = 
		devfs_add_devswf(&ity_cdevsw, next_if, DV_CHR, 0, 0, 
				0600, "isdn/ity%*0d", 2, next_if);
	devfs_token_out[next_if] = 
		devfs_add_devswf(&ity_cdevsw,(next_if + 0x82), DV_CHR, 0, 0, 
				 0600, "isdn/ityo%d", next_if);
#endif
	return(next_if++);
}

/* ARGSUSED */
int
ityopen(dev_t dev, int flag, int mode, struct proc * p)
{
	struct tty *tp;
	int    unit;
	int             error = 0;
	int x;

	unit = UNIT(dev);
	if (unit >= next_if)
		return (ENXIO);

	tp = &ity_tty[unit];
	tp->t_oproc = itystart;
	tp->t_param = ityparam;
	tp->t_dev = dev;
	if ((tp->t_state & TS_ISOPEN) == 0)
	{
#if defined(TS_WOPEN)
		tp->t_state |= TS_WOPEN;
#endif		
		ttychars(tp);
		if (tp->t_ispeed == 0)
		{
			tp->t_iflag = TTYDEF_IFLAG;
			tp->t_oflag = TTYDEF_OFLAG;
			tp->t_cflag = TTYDEF_CFLAG;
			tp->t_lflag = TTYDEF_LFLAG;
			tp->t_ispeed = tp->t_ospeed = itydefaultrate;
		}
		ityparam(tp, &tp->t_termios);
		ttsetwater(tp);
	} else if (tp->t_state & TS_XCLUDE && p->p_ucred->cr_uid != 0)
		return (EBUSY);
	x= spltty();

	if(OUTBOUND(dev))
	{
		tp->t_cflag |= CLOCAL;
		(*linesw[tp->t_line].l_modem) (tp, 1);
	}

	while (!(flag & O_NONBLOCK) && !(tp->t_cflag & CLOCAL) &&
	       !(tp->t_state & TS_CARR_ON) && !OUTBOUND(dev))
	{
#if defined(TS_WOPEN)
		tp->t_state |= TS_WOPEN;

		if((error = ttysleep(tp, (caddr_t) &tp->t_rawq,
			TTIPRI | PCATCH, ttopen, 0)) != 0)
#else
		if((error = ttysleep(tp, (caddr_t) &tp->t_rawq,
			TTIPRI | PCATCH, "ityopen" , 0)) != 0)
#endif
			break;
	}
	if (error == 0)
		error = (*linesw[tp->t_line].l_open) (dev, tp);
	splx(x);
	return (error);
}

/* ARGSUSED */
int
ityclose(dev_t dev, int flag, int mode, struct proc *p)
{
	struct tty *tp;
	int        ity;
	int    unit;
	int x;

	unit = UNIT(dev);
	if (unit >= next_if)
		return (ENXIO);

	ity = ity_addr[unit];

	x= splimp();
	if((tp= &ity_tty[unit]) && !OUTBOUND(dev))
		(*linesw[tp->t_line].l_close) (tp, flag);
	isdn_disconnect(applnr[unit],0);
	ttyclose(tp);
	splx(x);

	return (0);
}

int
ityread(dev_t dev, struct uio *uio, int flag)
{
	struct tty *tp = &ity_tty[UNIT(dev)];

	return ((*linesw[tp->t_line].l_read) (tp, uio, flag));
}

int
itywrite(dev_t dev, struct uio *uio, int flag)
{
	int             unit = UNIT(dev);
	struct tty *tp = &ity_tty[unit];

	return ((*linesw[tp->t_line].l_write) (tp, uio, flag));
}

int
ity_msg(int no, int len, char *buf)
{
	struct tty *tp = &ity_tty[no];
	int i;

	if (!(tp->t_state & TS_ISOPEN))
		return(0);

	tp->t_rawcc += len;
	i= b_to_q((char *)buf, len, &tp->t_rawq);
	ttwakeup(tp);
	if (tp->t_state & TS_TTSTOP
	    && (tp->t_iflag & IXANY
		|| tp->t_cc[VSTART] == tp->t_cc[VSTOP]))
	{
		tp->t_state &= ~TS_TTSTOP;
		tp->t_lflag &= ~FLUSHO;
		ttstart(tp);
	}
	return(len);
}

static void
ity_iintr(int no)
{
	struct tty *tp = &ity_tty[no];
	int x, i, l;
	struct mbuf *m;

	itystate[no] &= ~ITY_IINTR;

	x= splimp();
	if(IS_QEMPTY(&i_q[no]))
	{
		splx(x);
		return;
	}

	if (!(tp->t_iflag & (ICRNL | IGNCR | IMAXBEL | INLCR | ISTRIP | IXOFF | IXON))
	    && !(tp->t_lflag & (ECHO | ECHONL | ICANON | IEXTEN | ISIG | PENDIN))
	    && !(tp->t_state & (TS_CNTTB | TS_LNCH))
	    && linesw[tp->t_line].l_rint == ttyinput)
	{
		m= i_q[no].isq_head;
		tp->t_rawcc += m->m_len;
		l= min(64,m->m_len);
		i= b_to_q(m->m_data, m->m_len, &tp->t_rawq);
		l-= i;

		if(l >= m->m_len)
		{
			IS_DEQUEUE(&i_q[no], m, 21);
			m_freem(m);
		}
		else
		{
			m->m_data+=l;
			m->m_len-= l;
		}
		ttwakeup(tp);
		if (tp->t_state & TS_TTSTOP
		    && (tp->t_iflag & IXANY
			|| tp->t_cc[VSTART] == tp->t_cc[VSTOP]))
		{
			tp->t_state &= ~TS_TTSTOP;
			tp->t_lflag &= ~FLUSHO;
			ttstart(tp);
		}
		if(!IS_QEMPTY(&i_q[no]))
		{
			itystate[no] |= ITY_IINTR;
			timeout((TIMEOUT_FUNC_T)ity_iintr,(caddr_t)no,2);
		}
		splx(x);
		return;
	}

	IS_DEQUEUE(&i_q[no], m, 22);
	for(i= 0; i<m->m_len; i++)
		(*linesw[tp->t_line].l_rint)(m->m_data[i], tp);

	m_freem(m);
	if(!IS_QEMPTY(&i_q[no]))
	{
		itystate[no] |= ITY_IINTR;
		timeout((TIMEOUT_FUNC_T)ity_iintr,(caddr_t)no,2);
	}
	splx(x);
}

int
ity_input(int no, struct mbuf *m, int dir)
{
	struct tty *tp = &ity_tty[no];
	int x;

	if(!m) return(0);

	x= splimp();
	if (!(tp->t_state & TS_ISOPEN) || IS_QFULL(&i_q[no]))
	{
		m_freem(m);
		splx(x);
		return(0);
	}

	IS_ENQUEUE(&i_q[no],m);
	splx(x);

	if(itystate[no] & ITY_IINTR)
		return(1);
	itystate[no] |= ITY_IINTR;
	timeout((TIMEOUT_FUNC_T)ity_iintr,(caddr_t)no,1);
	return(1);

}

static void 
itystart(struct tty *tp)
{
	struct clist *rbp;
	int x, s;
	struct mbuf *m;
	int no= UNIT(tp->t_dev);
	isdn_appl_t    *appl= &isdn_appl[applnr[no]];

	if (no >= next_if)
		return;

	x = splimp();
	if(tp->t_state & (TS_TIMEOUT|TS_BUSY|TS_TTSTOP))
	{
		splx(x);
		return;
	}

	rbp = &tp->t_outq;
	if(!(itystate[no] & ITY_CONN))
	{
		if( (OUTBOUND(tp->t_dev)) &&
		    (m = isdn_getbuf(sizeof(message_msg_t))) )
		{
			message_msg_t *mp = (message_msg_t *)m->m_data;

			mp->length = q_to_b(rbp, mp->msg, MSG_MAX);

			isdn_msg(applnr[no],m);
			goto out;
		}
		splx(x);
		return;
	}
	tp->t_state |= TS_BUSY;
	if(IS_QFULL(&o_q[no]))
	{
		splx(x);
		return;
	}

	s= min(appl->blocksize,1024);
	if((rbp->c_cc) && (m= isdn_getbuf(s)))
	{
		m->m_data+= 4;

		m->m_len = q_to_b(rbp, m->m_data, s);
		m->m_pkthdr.len = m->m_len;
		IS_ENQUEUE(&o_q[no],m);
		isdn_output(applnr[no]);
		if(IS_QFULL(&o_q[no]))
		{
			splx(x);
			return;
		}
	}
	tp->t_state &= ~TS_BUSY;
out:

#if !defined(TS_ASLEEP)
        ttwakeup(tp);
#else
	if (rbp->c_cc <= tp->t_lowat)
	{
		if (tp->t_state & TS_ASLEEP)
		{
			tp->t_state &= ~TS_ASLEEP;
			wakeup((caddr_t)rbp);
		}
		selwakeup(&tp->t_wsel);
	}
#endif
	
	splx(x);
}

struct mbuf *
ity_out(int no)
{
	struct mbuf    *m;
	struct tty *tp = &ity_tty[no];
	int x;

	x=splimp();
	IS_DEQUEUE(&o_q[no], m, 23);

#if BSD > 199306 || defined (__NeBSD__)
	tp->t_state &=~ (TS_BUSY|TS_FLUSH);
	if (tp->t_line)
		(*linesw[tp->t_line].l_start)(tp);
	else
		itystart(tp);
#else
	tp->t_state &= ~TS_BUSY;

	if(!(tp->t_state & TS_TIMEOUT))
	{
		tp->t_state |= TS_TIMEOUT;
		timeout(ttrstrt, (caddr_t)tp, 2);
	}
#endif
	splx(x);
	return(m);
}

void
ity_connect(int no)
{
	struct tty *tp = &ity_tty[no];

	itystate[no]|= ITY_CONN;
	(*linesw[tp->t_line].l_modem) (tp, 1);
	tp->t_state &= ~(TS_BUSY|TS_FLUSH);
/*
	if (tp->t_line)
		(*linesw[tp->t_line].l_start)(tp);
	else
		itystart(tp);
*/
}

void
ity_disconnect(int no)
{
	struct tty *tp = &ity_tty[no];

	isdn_clean_q(&i_q[no], 24);
	isdn_clean_q(&o_q[no], 25);
	itystate[no]&= ~ITY_CONN;
	if(tp) (*linesw[tp->t_line].l_modem) (tp, 0);
}

int
ityioctl(dev_t dev, int cmd, caddr_t data, int flag, struct proc *p)
{
	struct tty *tp;
	int    unit = UNIT(dev);
	int    error;

	tp = &ity_tty[unit];

	error = (*linesw[tp->t_line].l_ioctl) (tp, cmd, data, flag,p);
	if (error >= 0)
		return (error);

#ifdef __FreeBSD__
	error = ttioctl(tp, cmd, data, flag);
#endif /* __FreeBSD__ */
#ifdef __NetBSD__
	error = ttioctl(tp, cmd, data, flag, p);
#endif /* __NetBSD__ */

	if (error >= 0)
		return (error);

	switch (cmd)
	{
	case TIOCMGET:
		error= TIOCM_LE | TIOCM_DTR | TIOCM_RTS | TIOCM_CTS | TIOCM_DSR;
		if(OUTBOUND(tp->t_dev) && (itystate[unit] & ITY_CONN))
			error |= TIOCM_CD;
		*(int *)data = error;
		return(0);
	case TIOCSDTR:
		break;
	case TIOCCDTR:
		isdn_disconnect(applnr[unit],0);
		break;
	default:
		return (ENOTTY);
	}
	return (0);
}

int
ityparam(struct tty *tp, struct termios *t)
{
	if (t->c_ispeed == 0)
		t->c_ispeed = t->c_ospeed;
	if (!t->c_ospeed || !t->c_ispeed)
		isdn_disconnect(applnr[UNIT(tp->t_dev)],0);
	return (0);
}

/*
 * Stop output on a line.
 */
/* ARGSUSED */
void
itystop(struct tty *tp, int flag)
{
	int    s;

	s = splimp();
	if (tp->t_state & TS_BUSY)
	{
		if ((tp->t_state & TS_TTSTOP) == 0)
			tp->t_state |= TS_FLUSH;
	}
	splx(s);
}

int
ityselect(dev_t dev, int rw, struct proc *p)
{
	return (ttselect(dev, rw, p));
}

struct tty *
itydevtotty(dev)
	dev_t	dev;
{
	u_int	unit;

	unit = UNIT(dev);
	if (unit >= NITY)
		return (NULL);
	return (&ity_tty[unit]);
}

#if BSD > 199306 && defined(__FreeBSD__)
static ity_devsw_installed = 0;

static void
ity_drvinit(void *unused)
{
	dev_t dev;

	if( ! ity_devsw_installed ) {
		dev = makedev(CDEV_MAJOR, 0);
		cdevsw_add(&dev,&ity_cdevsw, NULL);
		ity_devsw_installed = 1;
    	}
}

SYSINIT(itydev,SI_SUB_DRIVERS,SI_ORDER_MIDDLE+CDEV_MAJOR,ity_drvinit,NULL)
#endif /* __FreeBSD_version >= 199512 */
#endif /* NITY > 0 */
